<?php    
namespace App\Http\Controllers\Admin;
    
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Illuminate\Support\Facades\Mail;
use App\Mail\RegisterEmail;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Str; 
use App\Http\Controllers\Controller;
use Illuminate\Pagination\Paginator;
use Illuminate\Pagination\LengthAwarePaginator;
use App\Models\{User,Uniquecode,UserMember,FieldSetting,Order, TicketBooking};
use DB, Storage, Validator, Config;
use App\Mail\MasterRegistrationOrderSummaryEmail;
use App\Mail\UserRegistrationOrderSummaryEmail;
use App\Mail\MasterHotelBookingSummaryEmail;
use App\Mail\HotelBookingSummaryEmail;
    
class AttendeesController extends Controller
{

    public function __construct()
    {
        session_start();
        $this->now = Carbon::now();
    }
    
    public function index($slug)
    {
        unset($_SESSION['advance_search_filter']);
        unset($_SESSION['export_advance_search_filter']);

        $title = 'Attendees Users';

        $orders = DB::table('users')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('tickets.slug', $slug)
            ->where('users.status', 1)
            ->select('users.*', 'tickets.name as ticket_name', 'tickets.category as ticket_category')
            ->paginate(50);

        $pagination = 'yes';
        return View('admin.attendees.index', compact('title','orders', 'pagination'));
    }
 
    public function resendemail($user_id)
    {        
        $user_data = DB::table('users')
                    ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
                    ->select('users.id as user_id','users.ticket_id','users.order_id','users.name as user_name','users.email as user_email','users.country_code','users.phone as user_phone','users.master_user','users.festival_dates','users.eticket_path', 'tickets.category as ticket_category','tickets.name as ticket_name')
                    ->where('users.id', $user_id)->first();
        
        $order_data = Order::where('unique_order_id', $user_data->order_id)->with('coupons')->first();
        
        if($user_data->master_user == 1) {
            $body = [
                'user_name' => $user_data->user_name,
                'order_data' => $order_data,
                'package_name' => $user_data->ticket_category,
                'ticket_name' => $user_data->ticket_name,
                'festival_dates' => $user_data->festival_dates,
                'eticket_path' => $user_data->eticket_path
            ];

            Mail::to($user_data->user_email)->send(new MasterRegistrationOrderSummaryEmail($body));
            
        } else {
            $body = [
                'user_name' => $user_data->user_name,
                'order_data' => $order_data,
                'package_name' => $user_data->ticket_category,
                'ticket_name' => $user_data->ticket_name,
                'festival_dates' => $user_data->festival_dates,
                'eticket_path' => $user_data->eticket_path
            ];

            Mail::to($user_data->user_email)->send(new UserRegistrationOrderSummaryEmail($body));
        }
        
        return redirect()->back();
    }

    //advance filter
    public function advanceFilter(Request $request)
    {   
        if ($request->isMethod('post')) {
            $_SESSION['advance_search_filter'] = $request->all();
            $_SESSION['export_advance_search_filter'] = $request->all();

            $data = $this->getAdvanceFilterData($request->all());
            
            $data = $data->paginate();
            
            $pagination = 'yes';
            $title = 'Attendees Users';
            return view('admin.attendees.index', compact('title'))->with('orders', $data)->with('pagination', $pagination);
        } else {
            return redirect()->route('attendees');
        }
    }

    public function exportAdvanceFilterData()
    {
        $ticket_booking_data = $this->getAdvanceFilterData($_SESSION['export_advance_search_filter']);
        
        $ticket_booking_data = $ticket_booking_data->get();
        
        $headerRow = [
            'Sr.No.', 
            'Ticket Name', 
            'Name', 
            'Email ID', 
            'Phone',
            'Country',
            'State',
            'School Name',
            'School Address',
            'Festival Dates',
            'Profile Photo',
            'Student Id',
            'Eticket',
            'Created',
            'Order ID', // New
            //'Ticket category', // New
            'Master User', // New
            'Age Group', // New
            'Status', // New
            //'Sequence No', // New
            'Is Print', // New
            'UTM Source', // New
            'UTM Medium', // New
            'UTM Campaign', // New
        ];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Attendees Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($ticket_booking_data as $data)
        {            
            if($data->master_user !=  0)
            {
                $masterUser = DB::table('users')->where('id', $data->master_user)->select('name')->first();
                $masterUserName = $masterUser->name;
            }
            else
            {
                $masterUserName = '';
            }

            $arr = [
                $i,
                $data->ticket_category.' - '.$data->ticket_name,
                $data->name,
                $data->email,
                $data->country_code.' '.$data->phone,
                $data->country,
                $data->state,
                $data->school_name,
                $data->school_address,
                $data->festival_dates,
                $data->image_url,
                $data->student_id,
                $data->eticket_path,
                date('Y-m-d h:i:s A', strtotime($data->created_at)),
                $data->order_id,
                $masterUserName,
                $data->age_group,
                $data->status,
                $data->is_printed,
                $data->utm_source,
                $data->utm_medium,
                $data->utm_campaign,
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function getAdvanceFilterData($data)
    {
        //dd($data);
        $ticket_booking = DB::table('users')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('users.status', 1)
            ->select('users.*', 'tickets.name as ticket_name', 'tickets.category as ticket_category');
        
        if(!empty($data['search_field'])) {
            $ticket_booking->where(function($query) use($data) {
                if(isset($data['search_type']) && $data['search_type'] == 'All') {
                    foreach($data['search_field'] as $key => $value) {
                        if($value == 'name') {
                            if($data['input_value'][$key] != '') {
                                $query->where('users.name', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'email') {
                            if($data['input_value'][$key] != '') {
                                $query->where('users.email', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'phone') {
                            if($data['input_value'][$key] != '') {
                                $query->where(DB::raw("CONCAT(users.country_code,'',users.phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }
                        
                        if($value == 'order_id') {
                            if($data['input_value'][$key] != '') {
                                $query->where('users.order_id', '=', $data['input_value'][$key]);
                            }
                        }

                        if($value == 'date') {
                            if($data['date_type'][$key] == 'between') {
                                if($data['start_date'][$key] != '' && $data['end_date'][$key] != '') {
                                    $query->where(function($query3) use($data, $key) {
                                        $query3->whereDate('users.created_at', '>=', $data['start_date'][$key]);
                                        $query3->whereDate('users.created_at', '<=', $data['end_date'][$key]);
                                    });
                                }
                            } else if($data['date_type'][$key] == 'equal_to') {
                                if($data['start_date'][$key] != '') {
                                    $query->whereDate('users.created_at', '=', $data['start_date'][$key]);
                                }
                            }
                        }

                        if($value == 'ticket_name') {
                            if($data['input_value'][$key] != '') {
                                $query->where(DB::raw("CONCAT(tickets.category,' - ',tickets.name)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }
                    }
                } else {
                    foreach($data['search_field'] as $key => $value) {
                        if($value == 'name') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('users.name', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'email') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('users.email', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'phone') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere(DB::raw("CONCAT(users.country_code,'',users.phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'order_id') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('users.order_id', '=', $data['input_value'][$key]);
                            }
                        }

                        if($value == 'date') {
                            if($data['date_type'][$key] == 'between') {
                                if($data['start_date'][$key] != '' && $data['end_date'][$key] != '') {
                                    $query->where(function($query3) use($data, $key) {
                                        $query3->whereDate('users.created_at', '>=', $data['start_date'][$key]);
                                        $query3->whereDate('users.created_at', '<=', $data['end_date'][$key]);
                                    });
                                }
                            } else if($data['date_type'][$key] == 'equal_to') {
                                if($data['start_date'][$key] != '') {
                                    $query->orWhereDate('users.created_at', '=', $data['start_date'][$key]);
                                }
                            }
                        }

                        if($value == 'ticket_name') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere(DB::raw("CONCAT(tickets.category,' - ',tickets.name)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }
                    }
                }
                
            });
        }

        return $ticket_booking;
    }

    public function advanceFilterAddMore(Request $request)
    {
        $numItems = $request->numItems+1;

        return view('admin.attendees.ajax.add_more_fields')->with('numItems', $numItems);
    }

    public function exportAllData()
    {
        $ticket_booking = DB::table('users')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('users.status', 1)
            ->select('users.*', 'tickets.name as ticket_name', 'tickets.category as ticket_category')->get();

        $headerRow = [
            'Sr.No.', 
            'Ticket Name', 
            'Name', 
            'Email ID', 
            'Phone',
            'Country',
            'State',
            'School Name',
            'School Address',
            'Festival Dates',
            'Profile Photo',
            'Student Id',
            'Eticket',
            'Created',
            'Order ID', // New
            //'Ticket category', // New
            'Master User', // New
            'Age Group', // New
            'Status', // New
            //'Sequence No', // New
            'Is Print', // New
            'UTM Source', // New
            'UTM Medium', // New
            'UTM Campaign', // New
        ];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Attendees Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($ticket_booking as $data)
        {
            if($data->master_user !=  0)
            {
                $masterUser = DB::table('users')->where('id', $data->master_user)->select('name')->first();
                $masterUserName = $masterUser->name;
            }
            else
            {
                $masterUserName = '';
            }
            // echo $masterUser->name;
            // dd($masterUser);


            $arr = [
                $i,
                $data->ticket_category.' - '.$data->ticket_name,
                $data->name,
                $data->email,
                $data->country_code.' '.$data->phone,
                $data->country,
                $data->state,
                $data->school_name,
                $data->school_address,
                $data->festival_dates,
                $data->image_url,
                $data->student_id,
                $data->eticket_path,
                date('Y-m-d h:i:s A', strtotime($data->created_at)),
                $data->order_id,
                $masterUserName,
                $data->age_group,
                $data->status,
                $data->is_printed,
                $data->utm_source,
                $data->utm_medium,
                $data->utm_campaign,
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }

        // dd($arr);

        fclose($csvFile);
    }
}