<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use App\Models\Coupon;
use App\Models\Ticket;
use Illuminate\Http\Request;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use DB, Storage;
use Carbon\Carbon;
use Validator;
use DataTables;


class CouponsController extends Controller
{
    public function __construct()
    {
        $this->now = Carbon::now();

    }

    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = DB::table('coupons')->select(['id', 'name', 'code', 'max_ticket', 'coupon_type', 'amount', 'starting_at','ending_at','created_at', 'status'])->get()->toArray();
            
            return Datatables::of($data)->addIndexColumn()
                ->editColumn('status', function ($data) {
                    if ($data->status == 0) return 'Deactivated';
                    if ($data->status == 1) return 'Activated';
                })
                ->editColumn('coupon_type', function ($data) {
                    if ($data->coupon_type == 1) return 'Fixed';
                    if ($data->coupon_type == 2) return 'Percentage';
                })
                ->addColumn('action', function($row){
                    
                    if($row->status == 1) {
                        $status = 0;
                        $status_label = 'Deactivate';
                    } else {
                        $status = 1;
                        $status_label = 'Activate';
                    }

                    $btn = '<div class="dropdown ticket-option">
                                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
                                    <div class="dot-three"></div>
                                </button>
                                <div class="dropdown-menu">
                                    <a href="'.(route('coupons-edit',$row->id)).'" class="dropdown-item " >
                                        <i class="fas fa-pencil-alt"></i> Edit
                                    </a>
                                    <a href="'.(route('coupons-view',$row->id)).'" class="dropdown-item " >
                                        <i class="fas fa-eye"></i> View
                                    </a>
                                    <button class="dropdown-item status-action-btn" type="button" data-status="'.$status.'" data-id="'.$row->id.'">
                                        <i class="fas fa-cog"></i> '.$status_label.'
                                    </button>
                                    
                                </div>
                            </div>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
            
        }

        return view('admin.coupons.index');
    }

    public function add(Request $request)
    {
        $ticket =Ticket::select('id','name','category')->where('status','1')->get();

        return view('admin.coupons.add', compact('ticket'));
    }

    public function save(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'code' => 'required',
            'max_ticket' => 'required',
            'description' => 'required',
            'ticket_id' => 'required',
            'coupon_type' => 'required',
            'amount' => 'required|numeric|min:1|',
            'starting_at' => 'required',
            'ending_at' => 'required',
        ],[
            'name.required' => 'Name is required.',
            'code.required' => 'Code is required.',
            'max_ticket.required' => 'Coupon quantity is required.',
            'description.required' => 'Description is required.',
            'ticket_id.required' => 'Ticket category is required.',
            'coupon_type.required' => 'Coupon type is required.',
            'amount.required' => 'Amount is required.',
            'starting_at.required' => 'Starting date is required.',
            'ending_at.required' => 'Ending date is required.',
        ]);

        if ($validator->passes()) {
            $startingDate=explode("T",$request->starting_at);
            $startingDate= implode(" ",$startingDate);

            $endingDate=explode("T",$request->ending_at);
            $endingDate= implode(" ",$endingDate);

            if(in_array('Cart', $request->ticket_id)) {
                $ticket_id = 'Cart';
            } else {
                $ticket_id = implode(",",$request->ticket_id);
            }

            $data = [
                'ticket_id' => $ticket_id,
                'name' => $request->name,
                'code' => $request->code,
                'max_ticket' => $request->max_ticket,
                'description' => $request->description,
                'coupon_type' => $request->coupon_type,
                'amount' => $request->amount,
                'starting_at' => $startingDate,
                'ending_at' => $endingDate,
                'status' => "1",
                'created_at' => $this->now,
                'updated_at' => $this->now
            ];
            $result = Coupon::create($data);
            return true;
        } else {
            return response()->json(['error'=>$validator->errors()->all()]);
        }
    }

    public function edit(Request $request)
    {
		$coupons = DB::table('coupons')->where('id', $request->id)->first();
        //dd($request->id);
		return View('admin.coupons.edit', compact('coupons'));
    }

    public function update(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'max_ticket' => 'required',
            'starting_at' => 'required',
            'ending_at' => 'required',
        ],[
            'max_ticket.required' => 'Coupon quantity is required.',
            'starting_at.required' => 'Starting date is required.',
            'ending_at.required' => 'Ending date is required.',
        ]);

        if ($validator->passes()) {
            $data = [
                'max_ticket' => $request->max_ticket,
                'starting_at' => $request->starting_at,
                'ending_at' => $request->ending_at,
            ]; 
             
			$tickets = DB::table('coupons')->where('id', $request->id)->update($data);
			return true;
        } else {
            return response()->json(['error'=>$validator->errors()->all()]);
        }
    }

    public function view(Request $request)
    {
        $coupon = Coupon::where('id', $request->id)->first();

        $all_ticket_added = [];
        if($coupon->ticket_id == 'Cart') {
            $all_ticket_added[] = [
                "name" => "Cart",
                "category" => '',
            ];
        } else {
            $tickets = explode(',', $coupon->ticket_id);
            foreach($tickets as $key => $value) {
                $ticket = Ticket::select(['id', 'name', 'category'])->where('id', $value)->first();
                $all_ticket_added[] = [
                    "name" => $ticket->name,
                    "category" => $ticket->category,
                ];
            }
        }

        return view('admin.coupons.view')->with('coupons', $coupon)->with('all_ticket_added', $all_ticket_added);
    }

    public function status(Request $request)
    {
        $Coupon = Coupon::where('id', $request->id)->update(['status' => $request->status]);

        return true;
    }

    public function delete(Request $request)
    {
        $Coupon = Coupon::where('id', $request->id)->delete();

        return true;
    }
}
