<?php

namespace App\Http\Controllers\Admin;

use App\Models\Admin\Admin;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\{User, UserMember, Invitees, InterfaceUserRegistrations, Order, TicketBooking, Ticket};
use DB;
use Carbon\Carbon;

class DashboardController extends Controller
{
    /**
     * Instantiate a new controller instance.
     *
     * @return void
     */

    public function __construct()
    {
        $this->now = Carbon::now();
    }

    public function index()
    {
        $title = 'Admin Dashboard';

        $orderDetail = DB::table('orders')
            ->where('transection_id', '!=', 'bypass')
            ->where('status', 1)
            ->get();

        $final_data = [];
        $total_user_arr = [];
        $total_order_arr = [];
        $total_amount_arr = [];
        $total_refund_arr = [];
        $tickets = Ticket::where('status', 1)->orWhere('status', 2)->select('id', 'name', 'category')->get();
        foreach ($tickets as $key => $value) {

            $total_users = DB::table('users')->where('ticket_id', $value->id)->where('status', 1)->select('id')->get()->toArray();
            $final_data[$value->category][$value->name]['total_users'] = count($total_users);

            $total_orders = DB::table('orders')
                ->leftJoin('users', 'users.order_id', '=', 'orders.unique_order_id')
                ->where('users.ticket_id', $value->id)
                ->where('orders.status', 1)
                ->select('orders.unique_order_id', 'orders.total', 'orders.refundable_amount', 'orders.ticket_qty')
                ->groupBy('orders.unique_order_id')
                ->get()
                ->toArray();

            $final_data[$value->category][$value->name]['total_orders'] = count($total_orders);
            $final_data[$value->category][$value->name]['total_amount'] = array_sum(array_column($total_orders, 'total'));
            $total_amount_arr[] = array_sum(array_column($total_orders, 'total'));
            $total_refund_arr[] = array_sum(array_column($total_orders, 'refundable_amount'));

            $total_user_arr[] = count($total_users);
            $total_order_arr[] = count($total_orders);
        }

        $total_amount = array_sum($total_amount_arr);
        $total_refund = array_sum($total_refund_arr);
        $total_user_count = array_sum($total_user_arr);
        $total_order_count = array_sum($total_order_arr);


        $refundable_orders = DB::table('orders')
            ->leftJoin('users', 'users.order_id', '=', 'orders.unique_order_id')
            ->where('orders.status', 1)
            ->where('orders.refundable_amount', '>', 0)
            ->select('orders.unique_order_id', 'orders.total', 'orders.refundable_amount', 'orders.ticket_qty')
            ->groupBy('orders.unique_order_id')
            ->get()
            ->toArray();

        $total_refund_user_count = array_sum(array_column($refundable_orders, 'ticket_qty'));
        $total_refund_order_count = count(array_column($refundable_orders, 'unique_order_id'));
        
        $refundable_data = [
            'total_refundable_users' => $total_refund_user_count,
            'total_refund_orders' => $total_refund_order_count,
            'total_refundable_amount' => $total_refund
        ];

        $total_sale_data = [
            'total_users' => $total_user_count,
            'total_orders' => $total_order_count,
            'total_amount' => $total_amount
        ];

        /* 
        $final_order_counts = [];
        $refundable_counts = [];
        $final_bookings_count = [];
        foreach($orderDetail as $key => $value) {
            $ticketBookings = DB::table('users')->where('order_id', $value->unique_order_id)->where('status', 1);
            $final_order_counts[$value->unique_order_id] = ['order_id' => $value->unique_order_id, 'booking_count' => $ticketBookings->count(), 'total' => $value->total];
            if($value->refundable_amount != 0){
                $refundable_counts[$value->unique_order_id] = ['order_id' => $value->unique_order_id, 'booking_count' => $ticketBookings->count(), 'refundable' => $value->refundable_amount];
            }
            $catTicktBookings = $ticketBookings->get();
            
            foreach($catTicktBookings as $key2 => $value2) {
                $tickets = Ticket::where('id', $value2->ticket_id)->where('status', 1)->orWhere('status', 2)->first();
                
                $final_bookings_count[$tickets->category][$tickets->name][$value->unique_order_id] = ['order_id' => $value->unique_order_id, 'booking_count' => $ticketBookings->count(), 'total' => $value->total];
            }
        } */


        return view('admin.dashboard.index', compact('title', 'refundable_data', 'total_sale_data', 'final_data'));
    }

    public function eventStatus($status)
    {
        // echo $status; die();

        $result = DB::table('master_settings')->where('module_name', 'event_status')->update(['status' => $status, 'updated_at' => $this->now]);
        $data = array('type' => 'Success', 'message' => 'Event Status have been Updated successfully', 'status' => $status);
        echo json_encode($data);
        exit();

        // return redirect("admin/index")->with('title', 'event_status');
    }

    public function mastersearch(Request $request)
    {
        $title = 'Master Search';

        $data = [];
        if (!empty($request->input(['post_type']))) {
            $field = $request->field;
            $search = $request->search;

            if (!empty($field) && !empty($search)) {
                $users = User::where($request->field, 'LIKE', '%' . $request->search . '%')->get();
                $data[] = ['user_data' => $users];

                $users_member = UserMember::where($request->field, 'LIKE', '%' . $request->search . '%')->get();
                $data[] = ['user_members_data' => $users_member];

                $complementary_invitees = Invitees::where($request->field, 'LIKE', '%' . $request->search . '%')->get();
                $data[] = ['complementary_invitees' => $complementary_invitees];

                $invited_user = InterfaceUserRegistrations::where('user_interface_id', 1)->where($request->field, 'LIKE', '%' . $request->search . '%')->with('invitedBy')->get();
                $data[] = ['invited_user' => $invited_user];

                $interface_user = InterfaceUserRegistrations::where('user_interface_id', '!=', 1)->where($request->field, 'LIKE', '%' . $request->search . '%')->with('userInterfaces')->get();
                $data[] = ['interface_user' => $interface_user];
            }
        }

        return View('admin.dashboard.mastersearch', compact('title', 'data'));
    }
}
