<?php

namespace App\Http\Controllers\Admin\Email;
use App\Http\Controllers\Controller;
 
use App\Libraries\Email; 
use App\Models\Email\Template;
use App\Models\Event;
use Carbon\Carbon;
use Illuminate\Http\Request; 
use DB, Storage, Validator, Config;
use DataTables;

use Illuminate\Support\Facades\Mail;
use App\Mail\RegistrationSummaryEmail;

/**
 * HomeController Controller
 *
 * Add your methods in the class below
 *
 * This file will render views\home
 */
class TemplateController extends Controller {


    public function __construct() {
        $this->now = Carbon::now();
    }
	
    function index(Request $request){
        if ($request->ajax()) {
            $data = DB::table('email_templates AS ET')->select(["ET.id", "ET.alias", "ET.subject","ET.cc_email", "ET.bcc_email", "ET.reply_to_email", "ET.status"])->get()->toArray();
            
            return Datatables::of($data)->addIndexColumn()
                ->editColumn('status', function ($data) {
                    if ($data->status == 0) return 'Deactivated';
                    if ($data->status == 1) return 'Activated';
                })
                ->addColumn('action', function($row){
                    
                    if($row->status = 1) {
                        $status = 0;
                        $status_label = 'Deactivate';
                    } else {
                        $status = 1;
                        $status_label = 'Activate';
                    } 
                    $btn = '<div class="dropdown ticket-option">
                                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
                                    <div class="dot-three"></div>
                                </button>

                                <div class="dropdown-menu">
                                    <a href="'.(route('email-edit',$row->id)).'" class="dropdown-item " >
                                        <i class="fas fa-pencil-alt"></i> Edit
                                    </a>
                                    <a href="'.(route('email-view',$row->id)).'" class="dropdown-item " >
                                        <i class="fas fa-eye"></i> View
                                    </a>
                                    <button class="dropdown-item openmodal" type="button"  data-toggle="modal" data-target="#emailModal" data-type="send" data-id="'.$row->id.'">
                                        <i class="fas fa-mail"></i> Send Email
                                    </button>
                                    <a class="dropdown-item delete-action" type="button" data-id="'.$row->id.'">
                                        <i class="fas fa-trash"></i> Delete
                                    </a> 
                                    
                                </div>
                            </div>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view('admin.emails.index');
    }

    function add(Request $request){		
		return view('admin.emails.add'); 
    }
	
	function edit($id,Request $request){
		$email_data	=	DB::table('email_templates')->where('id', $id)->first();	
		return view('admin.emails.edit')->with('email_data',$email_data); 
    }
	
	function store(Request $request){
		$formFields			=	$request->all();
        $validator			=	Validator::make($request->all(), [
            'label'			=>	'bail|required',
            'subject'		=>	'bail|required',
            // 'sender_name'	=>	'bail|required',
            'body'  		=>	'bail|required',
            'reply_to_email'=>	'bail|nullable|email',
            // 'reply_to_name'	=>	'required',
            'cc_email'		=>	'bail|nullable|email',
            // 'cc_name'	=>	'required',
            'bcc_email'		=>	'bail|nullable|email',
            // 'bcc_name'	=>	'required',
            // 'status'		=>	'required',
        ]);

        if ($validator->fails()){
			return response()->json(['error' => $validator->errors()->messages()]);
		} 		
		foreach ($formFields as $key => $value) {
			if($value != 'null'){
				$formFields[$key] = $value;
			}
		}
		unset($formFields['_token']);
		if(empty($formFields['id'])) {
			$message = 'created';
			$formFields['alias']		=	str_replace(" ","",$formFields['label']);
			$formFields['status']		=	1;
			$formFields['created_at']	=	$this->now;
			DB::table('email_templates')->insert($formFields);
		}else{
			$message = 'updated';
			$formFields['updated_at']	=	$this->now;
			DB::table('email_templates')->where('id',$formFields['id'])->update($formFields);
		}
		return response()->json(["status" => true,'message'=> 'Email Template has been '.$message.' successfully.']);  
	}
	// Preview E-Mail in Admin Panel
    function view(Request $request){
		$email = DB::table('email_templates')->where('id', $request->id)->first();
		$event = DB::table('events')->where('id', 1)->first();
		$url = $event->url;
		$templateVar = [
			'##SITE_URL##' => $event->url, 
			'##TITLE##' => $event->name, 
			'##SITE_LOGO##' => $event->logo, 
			'##BANNER_IMAGE##' => $event->banner_img,
			'##BANNER_IMG##' => $event->banner_img, 
			'##EVENT_DATE##' => $event->event_date, 
			'##EVENT_TIME##' => $event->event_time, 
			'##EVENT_ADDRESS##' => $event->event_address, 
			'##SITE_LINK##' => $url,
			'##NAME##' => "Delegate",
			'##BOOKING_ID##' => "BASIC202310",
			'##USERNAME##' => 'delegate@dreamcast.in',
			'##PASSWORD##' => mt_rand(1000, 9999),
			'##PACKAGE_NAME##' => "General Tickets",
			'##TICKET_QTY##' => "General Tickets",
			'##CURRENCY##' => $event->Currency,
			'##TOTAL##' => 100,
			'##SUB_TOTAL##' => 100,
			'##DISCOUNT_PRICE##' => 0, 
		]; 
		$email->body  = strtr($email->body, $templateVar);
		$email->subject  = strtr($email->subject, $templateVar);
        return view('admin.emails.view')->with('email', $email);  
    }

	function delete(Request $request){
		if (empty($request->id)) {
			return response()->json(["status"=>true,'message'=> 'Email Template id been not Not found.']);
		} 
		DB::table('email_templates')->where('id', $request->id)->delete();
		return response()->json(["status"=>true,'message'=> 'Form Builder has been deleted successfully.']);
    } 
	
	public function send_mail(Request $request)
    {
        return view('admin.emails.send_mail')->with('id',  $request->id);
    }

    function send(Request $request){ 
        if (empty($request->email)) {
			return response()->json(["status" => false,'message'=> 'Please enter email address.']); 
        } else {
			$emailTemplate = DB::table('email_templates')->where('id', $request->id)->first();
			$event = DB::table('events')->where('id', 1)->first();
			$alias = $emailTemplate->alias; 
			$templateVar = [
				'##SITE_URL##' => $event->url, 
				'##TITLE##' => $event->name, 
				'##SITE_LOGO##' => $event->logo, 
				'##BANNER_IMAGE##' => $event->banner_img, 
				'##EVENT_DATE##' => $event->event_date, 
				'##EVENT_TIME##' => $event->event_time, 
				'##EVENT_ADDRESS##' => $event->event_address, 
				'##SITE_LINK##' => $event->url,
				'##NAME##' => "Delegate",
				'##BOOKING_ID##' => "BASIC202310",
				'##USERNAME##' => 'delegate@dreamcast.in',
				'##PASSWORD##' => mt_rand(1000, 9999),
				'##PACKAGE_NAME##' => "General Tickets",
				'##TICKET_QTY##' => "General Tickets",
				'##CURRENCY##' => $event->Currency,
				'##TOTAL##' => 100,
				'##SUB_TOTAL##' => 100,
				'##DISCOUNT_PRICE##' => 0, 
			];
			$emailTemplate->body  			= strtr($emailTemplate->body, $templateVar);
			$emailTemplate->subject			= strtr($emailTemplate->subject, $templateVar); 
			$emailTemplate->eticket_path	= strtr($emailTemplate->eticket_path, $templateVar);
			$sendMailStatus =  Mail::to($request->email)->send(new RegistrationSummaryEmail($emailTemplate));
            // $sendMailStatus = Email::setTemplate($alias,$templateVar)->send($request->email, 'Delegate'); 
			return response()->json(["status" => true,'message'=> 'Email sent successfully']);  
        }
    } 
}
