<?php
    
namespace App\Http\Controllers\Admin;
    
use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Support\Str;
use App\Http\Controllers\Controller;
use DB, Storage, Validator, Config;
use App\Models\{Event, EventTab};

class EventController extends Controller
{
    public function __construct() {
        $this->now = Carbon::now();
    }
    
    public function index(Request $request) {
    }

    public function event_details() {
        $event_details = Event::first();
        $tab_list = EventTab::all();

        return view('admin.event.event-details',compact('tab_list'))->with('event_details', $event_details);
    }

    public function add() {
        return view('admin.event.add-tab');
    }

    public function updateEventDetails(Request $request) {
        $data = [
            "prefix_uniquecode" => $request->prefix_uniquecode,
            "title" => $request->title,
            "description" => $request->description,
            "organizer_message" => $request->organizer_message,
            "event_date" => $request->event_date,
            "event_time" => $request->event_time,
            "event_address" => $request->event_address,
            "festival_dates" => $request->festival_dates,
            "festival_month_year" => $request->festival_month_year,
            "url" => $request->url,
            "home_url" => $request->home_url,
            "facebook" => $request->facebook,
            "twitter" => $request->twitter,
            "linkedin" => $request->linkedin,
            "whatsapp" => $request->whatsapp,
            "payment_bypass" => $request->payment_bypass
        ];

        $bucket_upload_name = env('APP_NAME');

        if(isset($request->banner_img) && !empty($request->banner_img)) {
            $banner_img_data = $request->banner_img;
            $banner_img_name = $banner_img_data->getClientOriginalName();
            $upload_name= "Smart-Reg/$bucket_upload_name/assets/".time().str_replace(' ', '_', $banner_img_name);
            $banner_img_path = Storage::disk('s3')->put($upload_name, file_get_contents($banner_img_data), 'public');
            $banner_img_path = Storage::disk('s3')->url($upload_name);
            
            $data["banner_img"] = $banner_img_path;
        }

        if(isset($request->logo) && !empty($request->logo)) {
            $logo_data = $request->logo;
            $logo_name = $logo_data->getClientOriginalName();
            $upload_name= "Smart-Reg/$bucket_upload_name/assets/".time().str_replace(' ', '_', $logo_name);
            $logo_path = Storage::disk('s3')->put($upload_name, file_get_contents($logo_data), 'public');
            $logo_path = Storage::disk('s3')->url($upload_name);

            $data["logo"] = $logo_path;
        }

        if(isset($request->favicon_icon) && !empty($request->favicon_icon)) {
            $secound_logo_data = $request->favicon_icon;
            $secound_logo_name = $secound_logo_data->getClientOriginalName();
            $upload_name= "Smart-Reg/$bucket_upload_name/assets/".time().str_replace(' ', '_', $secound_logo_name);
            $secound_logo_path = Storage::disk('s3')->put($upload_name, file_get_contents($secound_logo_data), 'public');
            $secound_logo_path = Storage::disk('s3')->url($upload_name);

            $data["favicon_icon"] = $secound_logo_path;
        }
        
        $update = Event::where('id', 1)->update($data);

        return redirect()->route('event_details');
    }

    public function uploadImg(Request $request) {
        $image = $request->image;
        $imageInfo = explode(";base64,", $image);

        $imgExt = str_replace('data:image/', '', $imageInfo[0]);      
        $image = str_replace(' ', '+', $imageInfo[1]);

        if($imgExt == 'png' || $imgExt == 'jpg' || $imgExt == 'jpeg' || $imgExt == 'gif')
        {
            $imageName = uniqid()."".time().".".$imgExt;
            $bucket_upload_name = env('APP_NAME');
            $document_path = Storage::disk('s3')->put("Smart-Reg/$bucket_upload_name/assets".$imageName, base64_decode($image));    
            $document_path = Storage::disk('s3')->url("Smart-Reg/$bucket_upload_name/assets/".$imageName, base64_decode($image));

            // echo $document_size = Storage::disk('s3')->size("jlf2023/Banners/".$imageName, base64_decode($image));
            // die();

            $actual_filename = $request->filename;

            Banner::insert(
                [
                    'event_id' => 1, 
                    'name' => $actual_filename,
                    'url' => $document_path,
                    'status' => 1,
                    'created_at' => $this->now,
                    'updated_at' => $this->now,
                ]
            );

            return response()->json(['response'=>'success', 'file_name'=>$document_path]);
        }
        else
        {
            return "Error";
        }
    }

    public function eventTabSave(Request $request) {

         $validator = Validator::make($request->all(), [
            'tab_name' => 'required',
            
            
        ],[
            'tab_name.required' => 'Event Tab is required.',
            
        ]);

         if ($validator->passes()) {

             $EventTab = new EventTab();
             $data = [
                  'tab_name'    =>$request->tab_name,
                  
                ];
          
              $EventTab->create($data);
              return true;

        } else {
            return response()->json(['error'=>$validator->errors()->all()]);
        }
    }

    public function eventTabStatus(Request $request) {
         $id = $request->id;

         $tab = EventTab::where('id',$id)->select('id','status')->first();
          
         if($tab->status=='1'){ $status ='0'; }else{ $status ='1'; }
         
         EventTab::where('id', $id)->update([
           'status' => $status
         ]);
         
         return true;
    }

    public function tabRemove(Request $request) {
         $id = $request->id;
         EventTab::where('id',$id)->delete();

         return true;
    }

    public function eventTabupdate(Request $request) {

        $validator = Validator::make($request->all(), [           
            'content'  => 'required',            
        ],[            
            'description.required' => 'Description is required.',
        ]);

        if ($validator->passes()) {
            $id = $request->id;
            EventTab::where('id',$id)->update(['description'=>$request->content]);
            return true;
        } else {
            return response()->json(['error'=>$validator->errors()->all()]);
        }
    }

}