<?php

namespace App\Http\Controllers\Admin;
use App\Http\Controllers\Controller;
   
use App\Models\FormBuilder;
use Carbon\Carbon;
use Illuminate\Http\Request;
use DB, Storage, Validator, Config;
use DataTables;


/**
 * HomeController Controller
 *
 * Add your methods in the class below
 *
 * This file will render views\home
 */
class FormBuilderController extends Controller {


    public function __construct() {
        $this->now = Carbon::now();
    }
	
    function index(Request $request){
        if ($request->ajax()) {
            $data = DB::table('form_templates')->select(["id", "name", "status", "alias"])->groupBy('name')->get()->toArray();
            return Datatables::of($data)->addIndexColumn()
                ->editColumn('status', function ($data) {
                    if ($data->status == 0) return 'Deactivated';
                    if ($data->status == 1) return 'Activated';
                })
                ->addColumn('action', function($row){
                    if($row->status = 1) {
                        $status = 0;
                        $status_label = 'Deactivate';
                    } else {
                        $status = 1;
                        $status_label = 'Activate';
                    } 
                    $btn = '<div class="dropdown ticket-option">
                                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
                                    <div class="dot-three"></div>
                                </button>
                                <div class="dropdown-menu">
                                    <a href="'.(route('form-builder-edit',$row->alias)).'" class="dropdown-item " >
                                        <i class="fas fa-pencil-alt"></i> Edit
                                    </a>
                                    <a class="dropdown-item delete-action" type="button" data-alias="'.$row->alias.'">
                                        <i class="fas fa-trash"></i> Delete
                                    </a> 
                                    
                                </div>
                            </div>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.form_builder.index');
    }

    function add(Request $request)
    {
        $user_types = DB::table('user_types')->select('type', 'category', 'slug', 'id')->get()->toArray();

		return view('admin.form_builder.add', compact('user_types')); 
    }
	
	function edit($alias,Request $request){
        $user_types = DB::table('user_types')->select('type', 'category', 'slug', 'id')->get()->toArray();

		$form_data = DB::table('form_templates')->where(['alias'=>$request->alias,'status'=>1])->get()->toArray();
		$name	=	!empty($form_data[0]->name) ? $form_data[0]->name : '';
		return View('admin.form_builder.edit', compact('form_data','alias','name','user_types'));
    }
	
	function store(Request $request){
		$formFields			=	$request->all(); 
		$validator			=	Validator::make($request->all(), [
            'name'				=>	'required',
            'sr_no.*'		=>	'required',
            'field_title.*'		=>	'required',
            'field_name.*'		=>	'required',
            'field_type.*'		=>	'required',
            'required.*'		=>	'required',
            'unique_field.*'	=>	'required',
            'filter.*'			=>	'required',
            'value'				=>	'nullable', 
        ]);
 
        if ($validator->fails()){
			return response()->json(['error' => $validator->errors()->messages()]);
		}

        $form_data = [];
        $alias = $formFields['name'];
		foreach($formFields['field_title'] as $key => $value) {
            $field_data = [];
            
            $field_data = [
                'sr_no' 			=> $formFields['sr_no'][$key],
                'name' 			=> $formFields['name'],
                'alias' 		=> $alias,
                'field_title' 	=> $value,
                'field_name' 	=> $formFields['field_name'][$key],
                'field_type' 	=> $formFields['field_type'][$key],
                'required' 		=> $formFields['required'][$key],
                'unique_field' 	=> $formFields['unique'][$key],
                'filter'		=> $formFields['filter'][$key],
                'value' 		=> $formFields['value'][$key],
            ];

            $form_data[] = $field_data;
        }

        if(!empty($form_data)) {
            DB::table('form_templates')->where('alias', $alias)->delete();

            $save_all = DB::table('form_templates')->insert($form_data);
        }

		if($save_all) {
			return response()->json(["status" => true, 'message'=> 'Form has been updated successfully.']);
		}else{
			return response()->json(["status" => false, 'message'=> 'Something went wrong.']);
		} 
	}

    function addFormField(Request $request){
		return View('admin.form_builder.add_form_field');
	}
	
    function removeFormField(Request $request){		
		if(isset($request->id)) {
			DB::table('form_templates')->where('id', $request->id)->delete();
			return true;
		}else{
			return false;
		}
	}
    
	function delete(Request $request){
		if (empty($request->alias)) {
			return response()->json(["status"=>false,'message'=> 'Form Builder alias not found.']);
		}

		DB::table('form_templates')->where('alias', $request->alias)->delete();

		return response()->json(["status" =>	true,'message'=> 'Form Builder has been deleted successfully.']);
    }
}
