<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use App\Models\Interfaces;
use App\Models\Invitees;
use App\Models\InterfaceUserRegistrations;
use App\Models\UserInterface;
use Illuminate\Http\Request;
use DB, Storage;
use Carbon\Carbon;
use Validator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use App\Mail\InviteeEmail;
use App\Mail\InterfacrUserRegEmail;
use App\Mail\ImportInterfacrUserEmail;
use App\Services\MbadgeService;


class InterfacesController extends Controller
{
    public function __construct()
    {
        session_start();
        $this->now = Carbon::now();

        $prefix_uniquecode = DB::table('events')->select('prefix_uniquecode', 's3bucket_name')->where('status', 1)->first();
        $this->unique_code_prefix = $prefix_uniquecode->prefix_uniquecode;
        $this->s3bucket_name = $prefix_uniquecode->s3bucket_name;
    }

    public function index($interface_id)
    {
        unset($_SESSION['advance_search_filter']);
        unset($_SESSION['export_advance_search_filter']);

        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();

        $data = DB::table('users')->where('user_type_id', $interface_id)->where('status', 1)->paginate(50);

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required'])->orderBy('sr_no', 'asc')->get()->toArray();

        $title = $interface_data->category;
        return view('admin.interfaces.index', compact('title'))->with('interface_data', $interface_data)->with('data', $data)->with('interface_form_data', $interface_form_data);
    }

    public function add(Request $request)
    {
        $interface_id = $request->interface_id;
        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();
        $title = $interface_data->category;

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required'])->orderBy('sr_no', 'asc')->get()->toArray();

        return view('admin.interfaces.add', compact('title', 'interface_form_data', 'interface_id'));
    }

    public function save(Request $request, MbadgeService $MbadgeService)
    {
        //Validation dynamic
        $user_data_array = [];

        $interface_data = DB::table('user_types')->where('id', $request->interface_id)->first();
        $title = $interface_data->category;

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();
        foreach ($interface_form_data as $key => $value) {
            //dd($value);
            /* dd($request->$value->field_name); */

            if ($value->required == 1) {
                $field = $value->field_name;
                if ($request->input($field) == Null) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is required.']]);
                }
            }

            if ($value->unique_field == 1) {
                $field = $value->field_name;
                $field_value = $request->input($field);
                $data_exist = InterfaceUserRegistrations::where($field, $field_value)->where('user_interface_id', $request->interface_id)->count();
                if ($data_exist != 0) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is already exist.']]);
                }
            }

            if ($value->field_type == 'phone') {
                $field = $value->field_name;
                $field_value = $request->input($field);
                $field_length = strlen($field_value);

                if (strpos($field_value, '+') !== false || $field_length < 8 || $field_length > 15) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is invalid.']]);
                }
            }

            if ($value->field_type == 1 && $request->input($value->field_name) != '') {
                $field_value = $request->input($value->field_name);
                if (!filter_var($field_value, FILTER_VALIDATE_EMAIL)) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is invalid.']]);
                }
            }

            if ($value->field_type == 'checkbox') {
                $user_data_array[$value->field_name] = implode(',', $request->input($value->field_name));
            } else {
                $user_data_array[$value->field_name] = $request->input($value->field_name);
            }
        }

        $interface_data = DB::table('user_types')->where('id', $request->interface_id)->first();
        
    /*     $unique_code = $MbadgeService->getCode();
        $qrcode_path = $MbadgeService->baarcode($unique_code);
        $eticket_path = $MbadgeService->eticket($qrcode_path, $unique_code,ucwords($user_data_array['name']),$interface_data->mbadge); */
        $unique_code = '';
        $qrcode_path = '';
        $eticket_path = '';
        
        $data = [
            'user_type_id' => $request->interface_id,
            'unique_code' => $unique_code,
            'qrcode_path' => $qrcode_path,
            'eticket_path' => $eticket_path,
            'status' => 1,
            'created_at' => $this->now,
            'updated_at' => $this->now,
        ];

        $final_user_data = array_merge($user_data_array, $data);

        $result = DB::table('users')->insert($final_user_data);

        return response()->json(['type' => 'Success', 'message' => 'User added successfully.']);
    }

    public function edit(Request $request)
    {
        $interface_id = $request->interface_id;
        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();
        $title = $interface_data->category;

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required'])->orderBy('sr_no', 'asc')->get()->toArray();

        $user_data = DB::table('users')->where('id', $request->user_id)->first();

        return view('admin.interfaces.edit', compact('title', 'interface_form_data', 'interface_id', 'user_data'));
    }

    public function update(Request $request, MbadgeService $MbadgeService)
    {
        //Validation dynamic
        $user_data_array = [];

        $interface_data = DB::table('user_types')->where('id', $request->interface_id)->first();
        $title = $interface_data->category;

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();
        foreach ($interface_form_data as $key => $value) {
            //dd($value);
            /* dd($request->$value->field_name); */

            if ($value->required == 1) {
                $field = $value->field_name;
                if ($request->input($field) == Null) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is required.']]);
                }
            }

            if ($value->unique_field == 1) {
                $field = $value->field_name;
                $field_value = $request->input($field);
                $data_exist = DB::table('users')->where($field, $field_value)->where('user_type_id', $request->interface_id)->count();
                if ($data_exist != 0) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is already exist.']]);
                }
            }

            if ($value->field_type == 'phone') {
                $field = $value->field_name;
                $field_value = $request->input($field);
                $field_length = strlen($field_value);

                if (strpos($field_value, '+') !== false || $field_length < 8 || $field_length > 15) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is invalid.']]);
                }
            }

            if ($value->field_type == 1 && $request->input($value->field_name) != '') {
                $field_value = $request->input($value->field_name);
                if (!filter_var($field_value, FILTER_VALIDATE_EMAIL)) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is invalid.']]);
                }
            }

            if ($value->field_type == 'checkbox') {
                $user_data_array[$value->field_name] = implode(',', $request->input($value->field_name));
            } else {
                $user_data_array[$value->field_name] = $request->input($value->field_name);
            }
        }


        $user_data = DB::table('users')->where('id', $request->user_id)->first();
        //$eticket_path = $MbadgeService->eticket($user_data->qrcode_path, $user_data->unique_code, ucwords($user_data_array['name']),$interface_data->mbadge);
        $eticket_path = '';

        $data = [
            'user_type_id' => $request->interface_id,
            'eticket_path' => $eticket_path,
            'status' => 1,
            'updated_at' => $this->now,
        ];

        $final_user_data = array_merge($user_data_array, $data);

        $result = DB::table('users')->where('id', $request->user_id)->update($final_user_data);

        return response()->json(['type' => 'Success', 'message' => 'User updated successfully.']);
    }

    public function view(Request $request)
    {
        $interface_data = DB::table('user_types')->where('id', $request->interface_id)->first();
        $title = $interface_data->category;

        $user_data = DB::table('users')->where('id', $request->user_id)->where('user_type_id', $request->interface_id)->first();

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();
        $interface_id = $request->interface_id;
        return view('admin.interfaces.view', compact('title', 'user_data', 'interface_form_data', 'interface_id'));
    }

    public function delete($interface_id, $user_id)
    {
        DB::table('users')->where('id', $user_id)->where('user_type_id', $interface_id)->delete();

        return redirect()->route('interfaces', ['interface_id' => $interface_id]);
    }

    public function import(Request $request)
    {
        $interface_data = DB::table('user_types')->where('id', $request->interface_id)->first();
        $title = $interface_data->category;

        $required_form_fields = DB::table('form_templates')->where('alias', $interface_data->category)->where('required', 1)->select(['id', 'field_title'])->orderBy('sr_no', 'asc')->get()->toArray();
        $unique_form_fields = DB::table('form_templates')->where('alias', $interface_data->category)->where('unique_field', 1)->select(['id', 'field_title'])->orderBy('sr_no', 'asc')->get()->toArray();

        $interface_id = $request->interface_id;
        return view('admin.interfaces.import', compact('title', 'interface_id', 'required_form_fields', 'unique_form_fields'));
    }

    public function sampleExcel($interface_id)
    {
        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();

        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();
        $filename = " samplecsv.csv";

        $headerRow = [];

        foreach ($interface_form_data as $key => $value) {
            $headerRow[] = $value->field_title;
        }

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/csv');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        fputcsv($csvFile, $headerRow, $delimiter, $enclosure);
        fclose($csvFile);
    }

    public function uploadinterfaceuser(Request $request, MbadgeService $MbadgeService)
    {
        try {
            if (!empty($request->file)) {
                $interface_data = DB::table('user_types')->where('id', $request->interface_id)->first();
                $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();

                $date = date('Y-m-d H:i:s');
                $target_file = $_FILES["file"]["tmp_name"];
                $csvFile = fopen($target_file, 'r');

                $all_data[] = fgetcsv($csvFile);
                while (($row = fgetcsv($csvFile)) !== FALSE) {
                    $all_data[] = $row;
                }

                $duplicate = [];
                foreach ($all_data as $key => $value) {

                    if (count($interface_form_data) != count($value)) {
                        $data = array('type' => 'Error', 'message' => 'Fields galat dale hai, Sample sheet dekh kr shi kr le.');
                        echo json_encode($data);
                        exit();
                    }

                    if ($key != 0) {
                        $error = false;
                        $user_basic_data_add = [];

                        foreach ($value as $field_key => $field_value) {
                            $row_number = $key + 1;

                            if ($interface_form_data[$field_key]->required == 1 && $field_value == '') {

                                $data = array('type' => 'Error', 'message' => 'Row number ' . $row_number . ' pr ' . $interface_form_data[$field_key]->field_title . ' field required hai, empty kyu bhja hai.');
                                echo json_encode($data);
                                exit();
                            }

                            if ($interface_form_data[$field_key]->unique_field == 1) {
                                $data_exist = DB::table('users')->where($interface_form_data[$field_key]->field_name, $field_value)->where('user_type_id', $request->interface_id)->count();

                                if ($data_exist > 0) {
                                    $data = array('type' => 'Error', 'message' => 'Row number ' . $row_number . ' pr ' . $interface_form_data[$field_key]->field_title . ' field unique hai, shi kr le.');
                                    echo json_encode($data);
                                    exit();
                                }

                                if (isset($duplicate[$interface_form_data[$field_key]->field_name]) && in_array($field_value, $duplicate[$interface_form_data[$field_key]->field_name])) {
                                    $data = array('type' => 'Error', 'message' => 'Row number ' . $row_number . ' pr ' . $interface_form_data[$field_key]->field_title . ' field duplicate hai, shi kr le.');
                                    echo json_encode($data);
                                    exit();
                                }
                                $duplicate[$interface_form_data[$field_key]->field_name][] = $field_value;
                            }

                            if ($interface_form_data[$field_key]->field_type == 'email') {
                                if (!filter_var($field_value, FILTER_VALIDATE_EMAIL)) {
                                    $data = array('type' => 'Error', 'message' => 'Row number ' . $row_number . ' pr ' . $interface_form_data[$field_key]->field_title . ' galat hai, shi kr le.');
                                    echo json_encode($data);
                                    exit();
                                }
                            }

                            if ($interface_form_data[$field_key]->field_type == 'phone') {
                                $field_length = strlen($field_value);

                                if (strpos($field_value, '+') !== false || $field_length < 8 || $field_length > 15) {
                                    $data = array('type' => 'Error', 'message' => 'Row number ' . $row_number . ' pr ' . $interface_form_data[$field_key]->field_title . ' galat hai, shi kr le.');
                                    echo json_encode($data);
                                    exit();
                                }

                                $pattern = '/^[0-9]+$/';
                                if (!preg_match($pattern, $field_value)) {
                                    $data = array('type' => 'Error', 'message' => 'Row number ' . $row_number . ' pr ' . $interface_form_data[$field_key]->field_title . ' galat hai, shi kr le.');
                                    echo json_encode($data);
                                    exit();
                                }
                            }

                            $user_basic_data_add[$interface_form_data[$field_key]->field_name] = trim(strip_tags($field_value));
                        }

                        if ($error == true) {
                            //array_push($removed_rows, $value);
                            //if the data need to be download which is not added in the db
                            $removed_rows[] = $value;
                        } else {
                            /* $unique_code = $MbadgeService->getCode();
                            $qrcode_path = $MbadgeService->baarcode($unique_code);
                            $eticket_path = $MbadgeService->eticket($qrcode_path, $unique_code,ucwords($user_basic_data_add['name']),$interface_data->mbadge); */
                            $unique_code = '';
                            $qrcode_path = '';
                            $eticket_path = '';

                            $user_basic_data = [
                                'user_type_id' => $request->interface_id,
                                'unique_code' => $unique_code,
                                'qrcode_path' => $qrcode_path,
                                'eticket_path' => $eticket_path,
                                'status' => 1,
                                'created_at' => $this->now,
                                'updated_at' => $this->now,
                            ];

                            //array_push($added_rows, $value);
                            $added_rows[] = array_merge($user_basic_data_add, $user_basic_data);
                        }
                    }
                }
                $insert = DB::table('users')->insert($added_rows);
            }


            if (!empty($removed_rows)) {
                $_SESSION['csv_removed_data'] = $removed_rows;
            }

            $data = array('type' => 'Success', 'message' => 'Data Uploaded Successfully.');
        } catch (Exception $e) {
            //echo 'Message: ' . $e->getMessage();
            $data = array('type' => 'Error', 'message' => $e->getMessage());
        }
        
        echo json_encode($data);
        exit();
    }

    public function downloadRemovedData($interface_id)
    {
        if (isset($_SESSION['csv_removed_data']) && !empty($_SESSION['csv_removed_data'])) {
            $remove_data = $_SESSION['csv_removed_data'];
            unset($_SESSION['csv_removed_data']);
            $interface_data = DB::table('user_types')->where('id', $interface_id)->first();
            $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();
            $filename = 'Removed Data.csv';
            $headerRow = [];

            foreach ($interface_form_data as $key => $value) {
                $headerRow[] = $value->field_name;
            }

            $delimiter = ',';
            $enclosure = '"';
            $csvFile = fopen('php://output', 'w');
            header('Content-type: application/csv');
            header('Content-Disposition: attachment; filename="' . $filename . '"');
            fputcsv($csvFile, $headerRow, $delimiter, $enclosure);

            foreach ($remove_data as $arr) {
                fputcsv($csvFile, $arr, $delimiter, $enclosure);
            }
            fclose($csvFile);
        }
    }

    public function exportInterfaceData($interface_id)
    {
        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();
        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();
        $user_interface_data = DB::table('users')->where('status', 1)->where('user_type_id', $interface_id)->get();

        foreach ($interface_form_data as $key => $value) {
            $headerRow[] = $value->field_title;
        }
        $headerRow = array_merge($headerRow, ['Unique Code', 'Eticket Path',  'Created']);

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=' . $interface_data->category . ' Report.csv');
        fputcsv($csvFile, $headerRow, $delimiter, $enclosure);

        $i = 1;
        foreach ($user_interface_data as $data) {
            $arr = [];
            foreach ($interface_form_data as $key => $value) {
                $field_name = $value->field_name;
                $arr[] = $data->$field_name;
            }
            $arr = array_merge($arr, [$data->unique_code, $data->eticket_path,  date('Y-m-d h:i:s A', strtotime($data->created_at))]);
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function clearInterfaceData($interface_id)
    {
        DB::table('users')->where('user_type_id', $interface_id)->delete();
        
        return redirect()->route('interfaces', ['interface_id' => $interface_id]);
    }

    //advance filter
    public function advanceFilter(Request $request)
    {
        $interface_id = $request->interface_id;
        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();
        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();

        if ($request->isMethod('post')) {
            $_SESSION['advance_search_filter'] = $request->all();
            $_SESSION['export_advance_search_filter'] = $request->all();

            $data = $this->getAdvanceFilterData($request->all(), $interface_id);

            $data = $data->paginate();
            //dd($order_data);
            $title = $interface_data->category;

            $pagination = 'yes';
            return view('admin.interfaces.index', compact('title'))->with('interface_data', $interface_data)->with('data', $data)->with('interface_form_data', $interface_form_data);
        } else {
            return redirect()->route('interfaces', ['interface_id' => $interface_id]);
        }
    }

    public function exportAdvanceFilterData()
    {
        $interface_id = $_SESSION['export_advance_search_filter']['interface_id'];

        $user_interface_data = $this->getAdvanceFilterData($_SESSION['export_advance_search_filter'], $interface_id)->get();

        $interface_data = DB::table('user_types')->where('id', $interface_id)->first();
        $interface_form_data = DB::table('form_templates')->where('alias', $interface_data->category)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->orderBy('sr_no', 'asc')->get()->toArray();

        foreach ($interface_form_data as $key => $value) {
            $headerRow[] = $value->field_title;
        }
        $headerRow = array_merge($headerRow, ['Unique Code', 'Eticket Path',  'Created']);

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=' . $interface_data->category . ' Reports.csv');
        fputcsv($csvFile, $headerRow, $delimiter, $enclosure);

        $i = 1;
        foreach ($user_interface_data as $data) {
            $arr = [];
            foreach ($interface_form_data as $key => $value) {
                $field_name = $value->field_name;
                $arr[] = $data->$field_name;
            }
            $arr = array_merge($arr, [$data->unique_code, $data->eticket_path,  date('Y-m-d h:i:s A', strtotime($data->created_at))]);
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function getAdvanceFilterData($data, $interface_id)
    {
        //dd($data);
        $user_interface_registeration = DB::table('users')->where('user_type_id', $interface_id)->where('status', 1);

        if (!empty($data['search_field'])) {
            $user_interface_registeration->where(function ($query) use ($data) {
                if (isset($data['search_type']) && $data['search_type'] == 'All') {
                    foreach ($data['search_field'] as $key => $value) {
                        if ($value == 'name') {
                            if ($data['input_value'][$key] != '') {
                                $query->where('name', 'LIKE', '%' . $data['input_value'][$key] . '%');
                            }
                        }

                        if ($value == 'email') {
                            if ($data['input_value'][$key] != '') {
                                $query->where('email', 'LIKE', '%' . $data['input_value'][$key] . '%');
                            }
                        }

                        if ($value == 'phone') {
                            if ($data['input_value'][$key] != '') {
                                $query->where(DB::raw("CONCAT(country_code,'',phone)"), 'LIKE', '%' . $data['input_value'][$key] . '%');
                            }
                        }

                        if ($value == 'unique_code') {
                            if ($data['input_value'][$key] != '') {
                                $query->where('unique_code', '=', $data['input_value'][$key]);
                            }
                        }

                        if ($value == 'date') {
                            if ($data['date_type'][$key] == 'between') {
                                if ($data['start_date'][$key] != '' && $data['end_date'][$key] != '') {
                                    $query->where(function ($query3) use ($data, $key) {
                                        $query3->whereDate('created_at', '>=', $data['start_date'][$key]);
                                        $query3->whereDate('created_at', '<=', $data['end_date'][$key]);
                                    });
                                }
                            } else if ($data['date_type'][$key] == 'equal_to') {
                                if ($data['start_date'][$key] != '') {
                                    $query->whereDate('created_at', '=', $data['start_date'][$key]);
                                }
                            }
                        }
                    }

                    /* if(in_array('name',$data['search_field']) || in_array('email',$data['search_field']) || in_array('phone',$data['search_field'])) {
                        $query->whereHas('users', function ($query2) use($data){
                            foreach($data['search_field'] as $key => $value) {
                                if($value == 'name' || $value == 'email') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where($value, 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                } else if($value == 'phone') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where(DB::raw("CONCAT(country_code,'',phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                }
                            }
                        });
                    } */
                } else {
                    foreach ($data['search_field'] as $key => $value) {
                        if ($value == 'name') {
                            if ($data['input_value'][$key] != '') {
                                $query->orWhere('name', 'LIKE', '%' . $data['input_value'][$key] . '%');
                            }
                        }

                        if ($value == 'email') {
                            if ($data['input_value'][$key] != '') {
                                $query->orWhere('email', 'LIKE', '%' . $data['input_value'][$key] . '%');
                            }
                        }

                        if ($value == 'phone') {
                            if ($data['input_value'][$key] != '') {
                                $query->orWhere(DB::raw("CONCAT(country_code,'',phone)"), 'LIKE', '%' . $data['input_value'][$key] . '%');
                            }
                        }

                        if ($value == 'unique_code') {
                            if ($data['input_value'][$key] != '') {
                                $query->orWhere('unique_code', '=', $data['input_value'][$key]);
                            }
                        }

                        if ($value == 'date') {
                            if ($data['date_type'][$key] == 'between') {
                                if ($data['start_date'][$key] != '' && $data['end_date'][$key] != '') {
                                    $query->where(function ($query3) use ($data, $key) {
                                        $query3->whereDate('created_at', '>=', $data['start_date'][$key]);
                                        $query3->whereDate('created_at', '<=', $data['end_date'][$key]);
                                    });
                                }
                            } else if ($data['date_type'][$key] == 'equal_to') {
                                if ($data['start_date'][$key] != '') {
                                    $query->orWhereDate('created_at', '=', $data['start_date'][$key]);
                                }
                            }
                        }
                    }

                    /* if(in_array('name',$data['search_field']) || in_array('email',$data['search_field']) || in_array('phone',$data['search_field'])) {
                        $query->orWhereHas('users', function ($query2) use($data){
                            $user_val_arr = [];
                            foreach($data['search_field'] as $key => $value) {
                                if($value == 'name' || $value == 'email') {
                                    $user_val_arr[] = $value;
                                    if($data['input_value'][$key] != '') {
                                        if(count($user_val_arr) == 1) {
                                            $query2->where($value, 'LIKE', '%'.$data['input_value'][$key].'%');
                                        } else {
                                            $query2->orWhere($value, 'LIKE', '%'.$data['input_value'][$key].'%');
                                        }
                                    }
                                }  else if($value == 'phone') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where(DB::raw("CONCAT(country_code,'',phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                }
                            }
                        });
                    } */
                }
            });
        }

        return $user_interface_registeration;
    }

    public function advanceFilterAddMore(Request $request)
    {
        $numItems = $request->numItems + 1;

        return view('admin.interfaces.ajax.add_more_fields')->with('numItems', $numItems);
    }

    /* ------------------------------------------------------------------------Comman functions--------------------------------------------------------------------------------------- */
    public function unique_code()
    {
        $unique_code = DB::table('online_registration_unique_codes')->where('is_used', 0)->first();
        DB::table('online_registration_unique_codes')->where('id', $unique_code->id)->update(['is_used' => 1, 'modified' => Carbon::now()]);
        return $unique_code->unique_code;
    }

    public function generateQr($unique_code)
    {
        $document = QrCode::format('png')->margin(2)->size(500)->backgroundColor(255, 255, 255)->generate($unique_code);
        $upload_name = "Smart-Reg/'.$this->s3bucket_name.'/qrcodes/" . $unique_code . ".png";
        $document_path = Storage::disk('s3')->put($upload_name, $document, 'public');
        $document_path = Storage::disk('s3')->url($upload_name);

        return $document_path;
    }

    public function eticket($qr_path, $unique_code, $name, $company, $mbadge)
    {

        $new        = imagecreatefrompng($qr_path);
        $master     = imagecreatefrompng($this->mbadge_bg);

        imagecopyresampled($master, $new, 110, 450, 0, 0, 300, 300, 500, 500);

        //$font = realpath('public/arial.ttf');
        $white = imagecolorallocate($master, 255, 255, 255);
        $black = imagecolorallocate($master, 0, 0, 0);
        $bold_font = realpath('public/SF-Pro-Text-Bold.otf');
        $font = realpath('public/SF-Pro-Text-Regular.otf');

        //Unique Code Print ============================================================================================================================
        $unique_code_font_size = 20;

        list($x, $y) = $this->ImageTTFCenter($master, $unique_code, $font, $unique_code_font_size);
        imagettftext($master, $unique_code_font_size, 0, $x, $y + 330, $black, $font, $unique_code);

        //Name Print ===================================================================================================================================
        $new_name = wordwrap($name, 30, "<br>\n");
        $new_name = str_replace(array("\r", "\n"), '', $new_name);
        $final_name = explode('<br>', $new_name);

        $name_count = count($final_name);
        //dd($name_count);
        if ($name_count == 1) {
            $name_font_size = 20;

            $name1 = $final_name[0];
            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 280, $black, $font, $name1);
        } else if ($name_count == 2) {

            $name_font_size = 20;

            $name1 = $final_name[0];
            $name2 = $final_name[1];

            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 260, $black, $font, $name1);

            list($x, $y) = $this->ImageTTFCenter($master, $name2, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 300, $black, $font, $name2);
        } else if ($name_count == 3) {
            $name_font_size = 20;

            $name1 = $final_name[0];
            $name2 = $final_name[1];
            $name3 = $final_name[2];

            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 250, $black, $font, $name1);

            list($x, $y) = $this->ImageTTFCenter($master, $name2, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 280, $black, $font, $name2);

            list($x, $y) = $this->ImageTTFCenter($master, $name3, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 310, $black, $font, $name3);
        } else {
            $name_font_size = 20;

            $name1 = $final_name[0];
            $name2 = $final_name[1];
            $name3 = $final_name[2];
            $name4 = $final_name[3];

            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 230, $black, $font, $name1);

            list($x, $y) = $this->ImageTTFCenter($master, $name2, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 260, $black, $font, $name2);

            list($x, $y) = $this->ImageTTFCenter($master, $name3, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 290, $black, $font, $name3);

            list($x, $y) = $this->ImageTTFCenter($master, $name4, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 320, $black, $font, $name4);
        }

        //company Print ============================================================================================================================
        $new_company = wordwrap($company, 40, "<br>\n");
        $new_company = str_replace(array("\r", "\n"), '', $new_company);
        $final_company = explode('<br>', $new_company);

        $company_count = count($final_company);
        //dd($name_count);
        if ($company_count == 1) {
            $company_font_size = 15;

            $company1 = $final_company[0];
            list($x, $y) = $this->ImageTTFCenter($master, $company1, $font, $company_font_size);
            imagettftext($master, $company_font_size, 0, $x, 360, $black, $font, $company1);
        } else {

            $company_font_size = 15;

            $company1 = $final_company[0];
            $company2 = $final_company[1];

            list($x, $y) = $this->ImageTTFCenter($master, $company1, $font, $company_font_size);
            imagettftext($master, $company_font_size, 0, $x, 360, $black, $font, $company1);

            list($x, $y) = $this->ImageTTFCenter($master, $company2, $font, $company_font_size);
            imagettftext($master, $company_font_size, 0, $x, 380, $black, $font, $company2);
        }

        // header('Content-Type: image/png');

        $masterImg = imagepng($master, 'public/eticket/' . $unique_code . ".png");

        if ($masterImg) {
            $upload_name = "Smart-Reg/.$this->s3bucket_name./eticket/" . time() . $unique_code . ".png";
            $eticketpath = Storage::disk('s3')->put($upload_name, file_get_contents('public/eticket/' . $unique_code . '.png'), 'public');
            $eticketpath = Storage::disk('s3')->url($upload_name);
            unlink('public/eticket/' . $unique_code . '.png');
        }

        // echo $eticketpath; die();
        return $eticketpath;
    }

    function ImageTTFCenter($image, $text, $font, $size, $angle = 0)
    {
        $xi = imagesx($image);
        $yi = imagesy($image);

        $box = imagettfbbox($size, $angle, $font, $text);

        $xr = abs(max($box[2], $box[4]));
        $yr = abs(max($box[5], $box[7]));

        $x = intval(($xi - $xr) / 2);
        $y = intval(($yi + $yr) / 2);
        // die();

        return array($x, $y);
    }
}
