<?php

namespace App\Http\Controllers\Admin;

use App\Models\Admin\Admin;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\{User, UserMember, Invitees, InterfaceUserRegistrations, Order, TicketBooking, Ticket};
use DB;
use Carbon\Carbon;

class OngroundDashboardController extends Controller
{
    /**
     * Instantiate a new controller instance.
     *
     * @return void
     */

    public function __construct()
    {
        $this->now = Carbon::now();

        $this->export_arr = [
            'key' => [
                'Name',
                'Email',
                'Phone',
                'Company',
                'Designation',
                'GST Number',
                'Category',
            ],
            'value' => [
                'name',
                'email',
                'phone',
                'organization',
                'designation',
                'gst_number',
                'category'
            ]
        ];
    }

    public function index(Request $request)
    {
        $title = 'Onground Dashboard';


        $date = '';
        if ($request->date_filter != '') {
            $date = $request->date_filter;
        }
        return view('admin.onground_dashboard.index', compact('title', 'date'));
    }

    /* ===============interface====================== */
    public function interface(Request $request)
    {
        $interfaces = DB::table('user_types')
            ->where('type', 'Interface')
            ->where('status', 1)->get();

        $interface_data = [];
        foreach ($interfaces as $key => $value) {
            $user_count = DB::table('users')
                ->where('user_type_id', $value->id)
                ->where('status', 1);

            if ($request->date != '') {
                $user_count = $user_count->whereDate('created_at', $request->date);
            }

            $user_count = $user_count->pluck('id');
            $interface_data[$value->category]['interface_id'] = $value->id;
            $interface_data[$value->category]['total'] = count($user_count);
        }
        //dd($interface_data);

        return view('admin.onground_dashboard.interface_data', compact('interface_data'));
    }

    /* =================redemption======================== */
    public function redemption(Request $request)
    {
        $redemption_data = [];
        $redemption = DB::table('redeem')
            ->leftJoin('users', 'users.id', '=', 'redeem.user_id')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->leftJoin('user_types', 'user_types.id', '=', 'users.user_type_id');

        if ($request->date != '') {
            $redemption = $redemption->whereDate('redeem.created_at', $request->date);
        }

        $redemption = $redemption->groupBy('redeem.user_id')
            ->select(['redeem.user_id', 'tickets.name as ticket_name', 'tickets.category as ticket_category', 'user_types.category as user_category'])
            ->get();
        
        $category_wise_data = [];
        foreach ($redemption as $key => $value) {

            $user_category = $value->ticket_category != '' ? $value->ticket_category . ' - ' . $value->ticket_name : $value->user_category;

            $category_wise_data[$user_category][] = $value->user_id;
        }
        
        foreach ($category_wise_data as $key => $value) {
            $redemption_data[$key]['total'] = count($value);
        }

        return view('admin.onground_dashboard.redemption_data', compact('redemption_data'));
    }

    public function exportRedemptionReport($category)
    {
        $redeemption_data = DB::table('redeem')
            ->leftJoin('users', 'users.id', '=', 'redeem.user_id')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->leftJoin('user_types', 'user_types.id', '=', 'users.user_type_id')
            ->select(['users.*', 'redeem.user_id', 'redeem.created_at as redemption_date', 'tickets.name as ticket_name', 'tickets.category as ticket_category', 'user_types.category as user_category']);

        if ($category != 'All') {
            $redeemption_data = $redeemption_data->where('user_types.category', $category)
                ->orWhere(DB::raw("CONCAT(tickets.category,' - ',tickets.name)"), 'LIKE', '%' . $category . '%');
        }

        $redeemption_data = $redeemption_data->get();

        $header_row_arr = $this->export_arr['key'];

        $headerRow = array_merge($header_row_arr, ['Ticket Name/Interface Name', 'Created']);

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Redemption Report.csv');
        fputcsv($csvFile, $headerRow, $delimiter, $enclosure);

        $i = 1;
        foreach ($redeemption_data as $data) {
            $user_category = $data->user_category != '' ? $data->user_category : $data->ticket_category . ' - ' . $data->ticket_name;

            $custom_arr = [];
            foreach ($this->export_arr['value'] as $key => $value) {
                if ($value == 'name') {
                    $custom_arr[] = $data->name . ' ' . $data->last_name;
                } else if ($value == 'phone') {
                    $custom_arr[] = $data->country_code . ' ' . $data->phone;
                } else {
                    $custom_arr[] = $data->$value;
                }
            }

            $default_arr = [
                $user_category,
                date('Y-m-d h:i:s A', strtotime($data->redemption_date))
            ];

            $arr = array_merge($custom_arr, $default_arr);
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    /* =================zapping==================== */
    public function zapping(Request $request)
    {
        $zapping = DB::table('zapping_data')
            ->select(['zapping_data.id', 'zapping_data.location', 'zapping_data.unique_code', 'zapping_data.category'])
            ->groupBy('zapping_data.location', 'zapping_data.unique_code');

        if ($request->date != '') {
            $zapping = $zapping->whereDate('zapping_data.created_at', $request->date);
        }

        $zapping = $zapping->get();

        $zapping_arr = [];
        foreach ($zapping as $key => $value) {
            $zapping_arr[$value->location][$value->category][] = $value->unique_code;
        }

        $zapping_data = [];
        $total_count = [];
        foreach ($zapping as $key => $value) {
            $zapping_data[$value->location][$value->category]['total'] = count($zapping_arr[$value->location][$value->category]);

            $total_count[] = count($zapping_arr[$value->location][$value->category]);
        }

        $zapping_count = DB::table('zapping_data')->select('unique_code');
        if ($request->date != '') {
            $zapping_count = $zapping_count->whereDate('created_at', $request->date);
        }

        $total_zapping_count = count($zapping_count->get());
        $total_unique_count = count($zapping_count->groupBy('unique_code')->get());
        
        $hourly_zapping = '';
        if ($request->date != '') {
        $hourly_zapping = DB::table('zapping_data')->whereDate('created_at', $request->date)
            ->select(DB::raw("date_format(created_at,'%h %p') as hour"), DB::raw("DATE_FORMAT(DATE_ADD(created_at, interval 1 hour), '%h %p') as hour_end"), DB::raw("COUNT(category) totalNumber"))
            ->groupBy('hour')
            ->get()
            ->toArray();
        }

        return view('admin.onground_dashboard.zapping_data', compact('zapping_data', 'total_zapping_count', 'total_unique_count', 'hourly_zapping'));
    }

    public function exportAllZappingReport()
    {
        $zapping_data = DB::table('zapping_data')
            ->leftJoin('users', 'users.unique_code', '=', 'zapping_data.unique_code')
            ->select(['users.*', 'zapping_data.category as zapping_category', 'zapping_data.location', 'zapping_data.created_at as zapping_date'])
            ->get();

        $header_row_arr = $this->export_arr['key'];

        $headerRow = array_merge($header_row_arr, ['Category', 'Location', 'Created']);

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Redemption Report.csv');
        fputcsv($csvFile, $headerRow, $delimiter, $enclosure);

        $i = 1;
        foreach ($zapping_data as $data) {
            $custom_arr = [];
            foreach ($this->export_arr['value'] as $key => $value) {
                if ($value == 'name') {
                    $custom_arr[] = $data->name . ' ' . $data->last_name;
                } else if ($value == 'phone') {
                    $custom_arr[] = $data->country_code . ' ' . $data->phone;
                } else {
                    $custom_arr[] = $data->$value;
                }
            }

            $default_arr = [
                $data->zapping_category,
                $data->location,
                date('Y-m-d h:i:s A', strtotime($data->zapping_date))
            ];

            $arr = array_merge($custom_arr, $default_arr);
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function exportZappingReport($location, $category)
    {
        $zapping_data = DB::table('zapping_data')
            ->leftJoin('users', 'users.unique_code', '=', 'zapping_data.unique_code')
            ->where('zapping_data.location', $location)
            ->where('zapping_data.category', $category)
            ->select(['users.*', 'zapping_data.category as zapping_category', 'zapping_data.location', 'zapping_data.created_at as zapping_date'])
            ->get();

        $header_row_arr = $this->export_arr['key'];

        $headerRow = array_merge($header_row_arr, ['Category', 'Location', 'Created']);

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Redemption Report.csv');
        fputcsv($csvFile, $headerRow, $delimiter, $enclosure);

        $i = 1;
        foreach ($zapping_data as $data) {
            $custom_arr = [];
            foreach ($this->export_arr['value'] as $key => $value) {
                if ($value == 'name') {
                    $custom_arr[] = $data->name . ' ' . $data->last_name;
                } else if ($value == 'phone') {
                    $custom_arr[] = $data->country_code . ' ' . $data->phone;
                } else {
                    $custom_arr[] = $data->$value;
                }
            }

            $default_arr = [
                $data->zapping_category,
                $data->location,
                date('Y-m-d h:i:s A', strtotime($data->zapping_date))
            ];

            $arr = array_merge($custom_arr, $default_arr);
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }
}
