<?php

namespace App\Http\Controllers\Admin;

use App\Models\User;
use App\Models\Permissions;
use App\Models\Admin\Admin;
use App\Models\Interfaces;
use App\Models\Uniquecode;
use App\Models\UserMember;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Route;
use App\Mail\RegisterEmail;
use Carbon\Carbon;
use App\Models\FieldSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use App\Http\Controllers\Controller;
use DB, Storage, Validator, Config;
use Illuminate\Pagination\Paginator;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Facades\Hash;
use DataTables;


class PermissionsController extends Controller
{

    public function __construct()
    {
        $this->now = Carbon::now();
    }

    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = DB::table('admins')->select(['id', 'role','email'])->get()->toArray();
            return Datatables::of($data)->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<div class="dropdown ticket-option">
                                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
                                    <div class="dot-three"></div>
                                </button>
                                <div class="dropdown-menu">
									<a href="'.(route('permissions-edit', ["user_id" => $row->id])).'" class="dropdown-item " >
                                        <i class="fas fa-pencil-alt"></i> Edit Permissions
                                    </a>
									<a href="'.(route('permissions-view',["user_id" => $row->id])).'" class="dropdown-item " >
                                        <i class="fas fa-eye"></i> View Permissions
                                    </a>
                                    
                                </div>
                            </div>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return View('admin.permissions.index');
    }

    public function adduser(Request $request)
    {
        return view('admin.permissions.adduser');
    }

    public function usersave(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'email' => 'required|unique:admins',
            'password' => 'required'

        ], [
            'email.required' => 'Email id is required.',
            'password.required' => 'Password is required.',

        ]);

        if ($validator->passes()) {
            $data = [
                'email' => $request->email,
                'password' =>  Hash::make($request->password),
                'pass' => $request->password,
                'role' => 'Admin',
                'created_at' => $this->now,
                'updated_at' => $this->now
            ];
            $result = DB::table('admins')->insert($data);

            $data = array('status' => true, 'message' => "User Created Successfully");
        } else {
            return response()->json(['error' => $validator->errors()->all()]);
        }

        echo json_encode($data);
        exit();
    }

    public function addmenus()
    {
        return view('admin.permissions.addmenus');
    }

    public function menusave(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|unique:sidebar_menus'

        ], [
            'name.required' => 'Menu name is required.'

        ]);

        if ($validator->passes()) {
            $data = [
                'name' => $request->name,
                'created_at' => $this->now,
                'updated_at' => $this->now
            ];
            $result = DB::table('sidebar_menus')->insert($data);

            $data  = array('status' => true, 'message' => "Menu Created Successfully");
        } else {
            return response()->json(['error' => $validator->errors()->all()]);
        }

        echo json_encode($data);
        exit();
    }

    public function edit($user_id)
    {
        $user_details = DB::table('admins')->where('id', $user_id)->select('id', 'permissions')->first();        

        $tickets = DB::table('tickets')->groupBy('category')->pluck('category', 'id');
        $user_types = DB::table('user_types')->where('type', 'Interface')->pluck('category', 'id');

        $menu = DB::table('sidebar_menus')->pluck('name', 'id');

        return view('admin.permissions.edit', compact('tickets', 'user_types', 'menu', 'user_id', 'user_details'));
    }

    public function update(Request $request)
    {
        $permissions = json_encode($request->menu);

        $update_user = DB::table('admins')->where('id', $request->admin_id)->update(['permissions' => $permissions]);

        $data = array('type' => 'Success', 'message' => 'Permission updated successfully.');

        echo json_encode($data);
        exit();
    }

    public function view($user_id)
    {
        $user_details = DB::table('admins')->where('id', $user_id)->first();

        return view('admin.permissions.view', compact('user_details'));
    }
}
