<?php
    
namespace App\Http\Controllers\Admin;
    
use App\Models\User;
use App\Models\Uniquecode;
use App\Models\UserMember;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Illuminate\Support\Facades\Mail;
use App\Mail\RegisterEmail;
use Carbon\Carbon;
use App\Models\FieldSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Str; 
use App\Http\Controllers\Controller;
use DB, Storage, Validator, Config;
use Illuminate\Pagination\Paginator;
use Illuminate\Pagination\LengthAwarePaginator;
    
class RegisteruserController extends Controller
{

    public function __construct() {
        $this->now = Carbon::now();
    }
    
    public function index(Request $request)
    {
        $title = 'Registered Users';

        if( !empty( $request->input(['post_type'])) ){
                $pagination = 'no';
                $users = User::select("*")
                    ->when($request->input('name'), function ($query) use ($request) {
                       $query->where('name', 'LIKE', '%' . $request->name .'%');
                    })
                    ->when($request->input('email'), function ($query) use ($request) {
                       $query->where('email', 'LIKE', '%' . $request->email .'%');
                    })
                    ->when($request->input('phone'), function ($query) use ($request) {
                        $query->where('phone', 'LIKE', '%' . $request->phone .'%');
                    })
                    ->when($request->input('qr'), function ($query) use ($request) {
                        $query->where('unique_code',$request->qr);
                    })
                    //->paginate(5)
                    ->where('status',1)
                    ->get();

        } else {

            $users = User::where('status',1)->paginate(50);
            $pagination = 'yes';

        }

        return View('admin.user.index', compact('title','users','pagination'));
    }

    public function usermembers(Request $request)
    {
            $title = 'Registered Members';
            $id = $request->user_id;
            $users = UserMember::where('status',1)->where('user_id',$id)->paginate(50);
            $pagination = 'yes';
            return View('admin.user.usermembers', compact('title','users','pagination'));
    }

    public function emailUser(Request $request)
    {
        $id = $request->id;
        $user = DB::table('users')->where('id',$id)->first();
        if(empty($user->unique_code)){
            $unique_code = 'ID22'.$this->unique_code();
            $qrcode_path = $this->generateQr($unique_code, 'General');
            $eticket_path = $this->eticket($qrcode_path, $unique_code);

            User::where('id', $id)->update(['unique_code' => $unique_code, 'qrcode_path' => $qrcode_path,
             'eticket_path' => $eticket_path]);
        } else{
            $unique_code = $user->unique_code;
            $qrcode_path = $user->qrcode_path;
            $eticket_path = $user->eticket_path;

        }

        
        $body = [
            'id' => $id,
            'name' => $user->name,
            'email' => $user->email,
            'qr_code' => $unique_code,
            'qr_path' => $qrcode_path,
            'dates' => $user->festival_dates,
        ];

        $this->resend_registration_confirm_whatsapp($user->name,$user->phone,$eticket_path,$user->festival_dates);
        Mail::to($user->email)->send(new RegisterEmail($body));

        $data = array('type' => 'Success', 'message' => 'Email Resend Successfully.');       
        echo json_encode($data);
        exit();
    }

    public function emailMember(Request $request)
    {
        $id = $request->id;
        $user = DB::table('user_members')->where('id',$id)->first();
        if(empty($user->unique_code)){
            $unique_code = 'ID22'.$this->unique_code();
            $qrcode_path = $this->generateQr($unique_code, 'General');
            $eticket_path = $this->eticket($qrcode_path, $unique_code);

            UserMember::where('id', $id)->update(['unique_code' => $unique_code, 'qrcode_path' => $qrcode_path,
             'eticket_path' => $eticket_path]);
        } else{
            $unique_code = $user->unique_code;
            $qrcode_path = $user->qrcode_path;
            $eticket_path = $user->eticket_path;

        }

        $mainuser = DB::table('users')->where('id',$user->id)->first();        

        
        $body = [
            'id' => $id,
            'name' => $user->name,
            'email' => $user->email,
            'qr_code' => $unique_code,
            'qr_path' => $qrcode_path,
            'dates' => $mainuser->festival_dates,
        ];

        $this->resend_registration_confirm_whatsapp($user->name,$user->phone,$eticket_path,$mainuser->festival_dates);
        Mail::to($user->email)->send(new RegisterEmail($body));

        $data = array('type' => 'Success', 'message' => 'Email Resend Successfully.');       
        echo json_encode($data);
        exit();
    }

    public function unique_code()
    {
        $unique_code = DB::table('online_registration_unique_codes')->where('is_used', 0)->first();
        DB::table('online_registration_unique_codes')->where('id', $unique_code->id)->update(['is_used' => 1, 'modified' => Carbon::now()]);
        return $unique_code->unique_code;        
    }

    public function generateQr($unique_code, $category)
    {
        $arr=array("$unique_code","$category");
        $document = QrCode::format('png')->margin(2)->size(150)->backgroundColor(255, 255, 255)->generate(json_encode($arr));
        
        $upload_name= "IndiadesignID/qrcodes/".$unique_code.".png";
        $document_path = Storage::disk('s3')->put($upload_name, $document, 'public');
        $document_path = Storage::disk('s3')->url($upload_name);
    
        return $document_path;
    }

    public function eticket($qr_path, $unique_code)
    {
        $new = imagecreatefrompng($qr_path);
        $master = imagecreatefrompng('public/ID22.png');
                            
        imagealphablending($master, false);
        imagecopymerge($master, $new, 150, 510, 0, 0, 150, 150, 100);
        // imagecopymerge ( resource $dst_im , resource $src_im , int $dst_x , int $dst_y , int $src_x , int $src_y , int $src_w , int $src_h , int $pct )
        
        $white = imagecolorallocate($master, 0, 0, 0);
        $font = realpath('public/arial.ttf');
        imagettftext($master, 20, 0, 150, 690, $white, $font, $unique_code);

        // header('Content-Type: image/png');
        $masterImg= imagepng($master,'public/eticket/'.$unique_code.".png");

        if($masterImg)
        {
            $upload_name= "IndiadesignID/eticket/".$unique_code.".png";
            $eticketpath = Storage::disk('s3')->put($upload_name, file_get_contents('public/eticket/'.$unique_code.'.png'), 'public');
            $eticketpath = Storage::disk('s3')->url($upload_name);
            unlink('public/eticket/'.$unique_code.'.png');

        }
        return $eticketpath;
    }

    public function resend_registration_confirm_whatsapp($name,$mobile,$eticketpath,$date)
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://api.interakt.ai/v1/public/message/',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS =>'{
            "countryCode": "+91",
            "phoneNumber": "'.$mobile.'",
            "callbackData": "some text here",
            "type": "Template",
            "template": {
                "name": "registration_confirmation_emailer_id22",
                "languageCode": "en",
                "headerValues": [
                    "'.$eticketpath.'"
                ],
                "bodyValues": [
                    "'.$name.'",
                    "'.$date.'"
                ]
            }
        }',
            CURLOPT_HTTPHEADER => array(
            'Authorization: Basic N2t6c29fZ1FsTk1haUtwOTQ4SGQzVTdfT3dOX1g3aHYwODJQYUJQYjhIZzo=',
            'Content-Type: application/json',
            'Cookie: ApplicationGatewayAffinity=a8f6ae06c0b3046487ae2c0ab287e175; ApplicationGatewayAffinityCORS=a8f6ae06c0b3046487ae2c0ab287e175'
            ),
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        // dd($response);                
    }

    public function userpaymentdetails($user_id) 
    {
        $title = 'Payment Details';

        $payment_details = User::with('paymentDetails')->where('id', $user_id)->first();
        
        return View('admin.user.userpaymentdetails', compact('title'))->with('payment_details',$payment_details);
    }

    public function attendees(Request $request)
    {
        $title = 'Attendees';

        $post_type =  $request->input('post_type');

        $users['search_fields']         = FieldSetting::where('search', 1)->get();
        $users['view_fields']           = FieldSetting::where('view', 1)->get();
        $users['upload_field_setting']  = FieldSetting::where('upload', 1)->get();
        $users['unique_field_setting']  = FieldSetting::where('unique_field', 1)->get();

        if( !empty( $request->input(['post_type'])) )
        {
            $search = true;

            $where = "WHERE id != '' AND total_log != 0";
            $users['search_for'] = '';

            foreach($users['search_fields'] as $key => $value)
            {
                $field_rec = $request->input($value['field_value']);
                $field_value = $value['field_value'];
                
                if(!empty($field_rec))
                {
                    $where .= " AND ".$field_value." LIKE '%".$field_rec."%'";

                    if($field_rec != '')
                    {
                        $users['search_for'] .= $field_rec.', ';
                    }
                }
            }
            
            if($users['search_for'] != '')
            {
                $users['record'] = DB::select( DB::raw("SELECT * FROM users $where") );
                $users['pagination'] = 'no';
            }
            else
            {
                $users['record'] = User::where('total_log', '!=', 0)->paginate(50);
                $users['pagination'] = 'yes';
            }
        }
        else
        {
            $users['record'] = User::where('total_log', '!=', 0)->paginate(50);
            $users['pagination'] = 'yes';
        }

        return View('admin.user.attendees', compact('title'))->with('users',$users);
    }

    public function adduser(Request $request)
    {
        $users['unique_field_setting'] =   FieldSetting::where('unique_field', 1)->get();
        $unique_field = $users['unique_field_setting'][0]['field_value'];
        
        if (isset($_POST[$unique_field]) && !empty($_POST[$unique_field]))
        {
            $Totalusers = User::where($unique_field, $_POST[$unique_field])->count();

            if(!empty($Totalusers))
            {
                $data = array('type' => 'Error', 'message' => ucwords($users['unique_field_setting'][0]['field_value']).' is already registered.');
            }
            else
            {
                $webinarId = 1;
                $unique_field  = $_POST[$unique_field];
                
                $error = false;

                if($unique_field == 'email') {
                    if(filter_var($unique_field, FILTER_VALIDATE_EMAIL)) {
                        $error = false;
                    }
                    else {
                        $error = true;
                        $error_msg = 'Invalid email.';
                    }
                }

                if($error == false)
                {
                    $users['upload_field_setting'] =   FieldSetting::select('field_value')->where('upload', 1)->get();

                    foreach ($users['upload_field_setting'] as $key => $value) {
                        $FieldArray[] = $value->field_value;
                        $FieldArray1[] = $_POST[$value->field_value];
                    }

                    $FieldArrayfinal = array_combine($FieldArray, $FieldArray1);

                    $last_id = User::insertGetId($FieldArrayfinal);

                    if(!empty($last_id))
                    {
                        $data = array('type' => 'Success', 'message' => 'Registration successful.');
                    }
                    else
                    {
                        $data = array('type' => 'Error', 'message' => 'Something went wrong, please try again.');
                    }
                }
                else
                {
                    $data = array('type' => 'Error', 'message' => $error_msg);
                }
            }
        }
        else
        {
            $data = array('type' => 'Error', 'message' => ucwords($upload_array['unique_field']).' is required.');  
        }

        echo json_encode($data);
        exit();
    }

    public function sampleExcel()
    {
        $filename = " samplecsv.csv";

        $headerRowrec = FieldSetting::select('field_value')->where('upload', 1)->get();

        foreach ($headerRowrec as $key => $value) {
            $headerRow[] = $value->field_value;
        }
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/csv');
        header('Content-Disposition: attachment; filename="'.$filename.'"');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        fclose($csvFile);
    }

    public function uploadUsercsv()
    {
        if(isset($_POST["submit"]))
        {
            if(!empty($_FILES['file']['name']))
            {
                $target_file = $_FILES["file"]["tmp_name"];
                // Open uploaded CSV file with read-only mode
                $date = date('Y-m-d H:i:s');    
                
                $csvFile = fopen($target_file, 'r');
                fgetcsv($csvFile);
                
                $filename = "userlist.csv";


                $field_values =   FieldSetting::select('field_title', 'field_value')->where('upload', 1)->get();
                $unique_fields =   FieldSetting::select('field_title', 'field_value')->where('unique_field', 1)->get();

                $unique_field_title = $unique_fields[0]['field_title'];
                $unique_field_value = $unique_fields[0]['field_value'];


                foreach ($field_values as $fields)
                {
                    $field_title_pass[] = $fields->field_title;
                    $field_value_pass[] = $fields->field_value;
                }
                $headerRow = $field_title_pass;

                $delimiter = ',';
                $enclosure = '"';

                
                $csvFile1 = fopen('php://output', 'w');
                header('Content-type: application/csv');
                header('Content-Disposition: attachment; filename="'.$filename.'"');
                fputcsv($csvFile1,$headerRow, $delimiter, $enclosure);


                while(($line = fgetcsv($csvFile)) !== FALSE)
                {
                    // Get row data
                    $arr = [];
                    $unique_value = '';

                    foreach($field_value_pass as $key => $value)
                    {
                        $variable_name = '$'.$value;
                        $variable_name = $line[$key];

                        if($value == $unique_field_value)
                        {
                            $unique_value = $variable_name;
                        }

                        array_push($arr,$variable_name);
                    }

                    $Totalusers = User::where($unique_field_value, '=', $unique_value)->count();
     
                    if($Totalusers == 0)
                    {
                        $values = $field_value_pass;
                        $FieldArray = $values;
                        $FieldArray1 = $arr;

                        $FieldArrayfinal = array_combine($FieldArray, $FieldArray1);
                        $last_id = User::insertGetId($FieldArrayfinal);
                    }
                    else
                    {
                        $values = $field_value_pass;                     
                        $FieldArray = $values;
                        $FieldArray1 = $arr;

                        $FieldArrayfinal = array_combine($FieldArray, $FieldArray1);

                        $last_id = User::where('email', $unique_value)->update($FieldArrayfinal);
                    }
                }

                fclose($csvFile1);

                if(!empty($last_id))
                {
                    if (!empty($last_id))
                    {
                        return redirect()->route('users');
                        exit();
                    }
                }
                else
                {
                    $data = array('type' => 'Error', 'message' => 'Something went wrong, please try again.');
                }
            }
            else
            {
                $data = array('type' => 'Error', 'message' => 'Please Select File.');
            }
        }

        echo json_encode($data);
        exit();
    }

    public function changeStatus(Request $request)
    {
        $status = $request->input('status');
        $id = $request->input('id');

        $last_id = User::where('id', $id)->update(['status' => $status]);

        $data = array('type' => 'Success', 'message' => 'Status updated successful.');

        echo json_encode($data);
        exit();
    }

    public function deleteUser(Request $request)
    {
        $id = $request->input('id');

        User::where('id', $id)->delete();

        $data = array('type' => 'Success', 'message' => 'User deleted successful.');       

        echo json_encode($data);
        exit();
    }
    
    public function resetUsers()
    {
        User::truncate();
        exit();
    }

    public function exportUsers() {
        $result = User::get();
        $filename = " Registrations Reports.csv";
        $field_values = FieldSetting::select('field_title', 'field_value')->where('export', 1)->get();

        foreach ($field_values as $fields) {
            $field_title_pass[] = $fields->field_title;
            $field_value_pass[] = $fields->field_value;
        }

        array_push($field_title_pass, "Stay Time","Registration Date","Last Login");
        array_push($field_value_pass, "stayTime","created_at","lastLogin");

        $headerRow = $field_title_pass;
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/csv');
        header('Content-Disposition: attachment; filename="'.$filename.'"');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $value_array = $field_value_pass;

        foreach($result as $data)
        {
            $arr = [];
            foreach($value_array as $value)
            {
                if($value == 'created_at')
                {
                    $arr[] = date('Y-m-d h:i:s A', strtotime($data[$value]));
                }   
                else if($value == 'stayTime')
                {
                    if($data[$value] != '')
                    {
                        $init = $data[$value];
                        $hours = floor($init / 3600);
                        $minutes = floor(($init / 60) % 60);
                        $seconds = $init % 60;
                        
                        $arr[] = "$hours:$minutes:$seconds";
                    }
                    else
                    {
                        $arr[] = "00:00:00";
                    }
                
                }
                else if($value == 'lastLogin')
                {
                    if($data[$value] != '')
                    {
                        $arr[] = date('Y-m-d H:i:s', strtotime($data[$value]));
                    }
                    else
                    {
                        $arr[] = '';
                    }
                }
                else
                {
                    $arr[] = $data[$value];
                }

            }
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
        }
        fclose($csvFile);
    }

    public function exportAttendees() {
        $result = User::where('total_log', '>', 0)->get();

        $filename = "Attendees Reports.csv";

        $field_values =   FieldSetting::select('field_title', 'field_value')->where('export', 1)->get();

        foreach ($field_values as $fields) {
            $field_title_pass[] = $fields->field_title;
            $field_value_pass[] = $fields->field_value;                        
        }

        array_push($field_title_pass, "Stay Time","Registration Date","Last Login");
        array_push($field_value_pass, "stayTime","created_at","lastLogin");

        $headerRow = $field_title_pass;

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/csv');
        header('Content-Disposition: attachment; filename="'.$filename.'"');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);

        $value_array = $field_value_pass;

        foreach($result as $data)
        {
            $arr = [];
            foreach($value_array as $value)
            {                
                if($value == 'created_at')
                {
                    $arr[] = date('Y-m-d h:i:s A', strtotime($data[$value]));
                }   
                else if($value == 'stayTime')
                {
                    if($data[$value] != '')
                    {
                        $init = $data[$value];
                        $hours = floor($init / 3600);
                        $minutes = floor(($init / 60) % 60);
                        $seconds = $init % 60;
                        
                        $arr[] = "$hours:$minutes:$seconds";
                    }
                    else
                    {
                        $arr[] = "00:00:00";
                    }
                
                }
                else if($value == 'lastLogin')
                {
                    if($data[$value] != '')
                    {
                        $arr[] = date('Y-m-d H:i:s', strtotime($data[$value]));
                    }
                    else
                    {
                        $arr[] = '';
                    }
                }
                else
                {
                    $arr[] = $data[$value];
                }            
            }

            fputcsv($csvFile, $arr, $delimiter, $enclosure);
        }

        fclose($csvFile);
    }

    public function exportAttendeesByDate(Request $request) 
    {
        $start_date = $request->start_date;
        $end_date = $request->end_date;
        $result = DB::table('user_logs as c')
            ->select(DB::raw("COUNT(c.created_at) AS login_count"), 'u.*', DB::Raw('DATE(c.created_at) log_date'))
            ->leftJoin('users as u', 'c.user_id', '=', 'u.id')
            ->whereDate('c.created_at', '>=', $start_date)
            ->whereDate('c.created_at', '<=', $end_date)
            ->where('u.total_log', '>', 0)
            ->where('c.action', '=', 'Login')
            ->groupBy('c.user_id', 'log_date')
            ->get();
        
        $filename = "Attendees Reports.csv";

        $field_values = FieldSetting::select('field_title', 'field_value')->where('export', 1)->get();

        foreach ($field_values as $fields) {
            $field_title_pass[] = $fields->field_title;
            $field_value_pass[] = $fields->field_value;                        
        }

        array_push($field_title_pass, "Stay Time","Registration Date","Login Date","Login Count");
        array_push($field_value_pass, "stayTime","created_at","log_date","login_count");

        $headerRow = $field_title_pass;

        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/csv');
        header('Content-Disposition: attachment; filename="'.$filename.'"');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);

        $value_array = $field_value_pass;

        foreach($result as $data)
        {
            $arr = [];
            foreach($value_array as $value)
            {                
                if($value == 'created_at')
                {
                    $arr[] = date('Y-m-d h:i:s A', strtotime($data->$value));
                }
                else if($value == 'stayTime')
                {
                    if($data[$value] != '')
                    {
                        $init = $data[$value];
                        $hours = floor($init / 3600);
                        $minutes = floor(($init / 60) % 60);
                        $seconds = $init % 60;
                        
                        $arr[] = "$hours:$minutes:$seconds";
                    }
                    else
                    {
                        $arr[] = "00:00:00";
                    }
                
                }
                else if($value == 'lastLogin')
                {
                    if($data[$value] != '')
                    {
                        $arr[] = date('Y-m-d H:i:s', strtotime($data[$value]));
                    }
                    else
                    {
                        $arr[] = '';
                    }
                } 
                else if($value == 'log_date')
                {
                    $arr[] = date('d-m-Y', strtotime($data->$value));
                }
                else
                {
                    $arr[] = $data->$value;
                }            
            }

            fputcsv($csvFile, $arr, $delimiter, $enclosure);
        }

        fclose($csvFile);
        exit;
    }

//Naveen Views
    public function coupon()
    {
        return view('admin.user.coupon');
    }
    public function sales_report()
    {
        return view('admin.user.sales-report');
    }
    public function ticket_type_detail()
    {
        return view('admin.user.ticket-type-detail');
    }
    public function form()
    {
        return view('admin.user.form');
    } 
    public function attendee_enquiries()
    {
        return view('admin.user.attendee-enquiries');
    }
    public function email_builder()
    {
        return view('admin.user.email-builder');
    }
    public function edit_email()
    {
        return view('admin.user.edit-email');
    }

}