<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Mail;
use Illuminate\Http\Request;
use DB, Storage;
use Carbon\Carbon;
use Validator;
use App\Mail\Invitees\Groupinviteesmail;
use App\Mail\Invitees\InviteesConfirmationEmail;

class RsvpController extends Controller
{
    public function __construct()
    {
        session_start();
        $this->now = Carbon::now();
    }

    public function index(Request $request)
    {
        unset($_SESSION['advance_search_filter']);
        unset($_SESSION['export_advance_search_filter']);

        $title = 'RSVP';
        $search = '';
        $userType = DB::table('user_types')->where('type', 'RSVP')->first();
        // dd($userType);

        if (isset($request->search) && $request->search != '')
        {
            $search = $request->search;
            $orders = DB::table('users')->select("*")
                        ->when($request->search, function ($query) use ($request) {
                            $query->where('user_type_id', $userType->id);
                            $query->where('name', 'LIKE', '%' . $request->search . '%');
                            $query->orWhere('email', 'LIKE', '%' . $request->search . '%');
                            $query->orWhere('phone', 'LIKE', '%' . $request->search . '%');
                        })
                        ->get();

            $pagination = 'no';
        }
        else
        {
            $orders = DB::table('users')->where('user_type_id', $userType->id)->paginate(50);

            $pagination = 'yes';
        }

        return view('admin.rsvp.index', compact('title', 'orders', 'search', 'pagination')); 
    }
    
    public function add(Request $request)
    {
        $title = 'Add RSVP';
        return view('admin.rsvp.add', compact('title'));
    }

    public function save(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|unique:users',
            'phone' => 'required|min:10|max:10|unique:users',
        ], [
            'name.required' => 'Name is required.',
            'email.required' => 'Email is required.',
            'phone.required' => 'Phone is required.',
        ]);        

        if ($validator->passes())
        {
            $categories = 'RSVP';
            $festival_dates = DB::table('events')->pluck('festival_dates')->first();            

            $userTypes = DB::table('user_types')->where('type', $request->user_type)->where('status', '1')->first();

            $data = [
                'user_type_id' => $userTypes->id,
                'name' => $request->name,
                'email' => $request->email,
                'phone' => $request->phone,
                'country_code' => '+91',
                'festival_dates' => $festival_dates,
                'status' => "1",
                'created_at' => $this->now,
                'updated_at' => $this->now
            ];

            $inserted_id = DB::table('users')->insertGetId($data);            
            return true;
        }
        else
        {
            return response()->json(['error' => $validator->errors()->all()]);
        }
    }

    public function edit(Request $request)
    {
        $title = 'Edit Inviter';

        $inviteer_data = DB::table('invitees')->where('id', $request->id)->where('status', '1')->first();
        
        return view('admin.inviteer.edit', compact('title', 'inviteer_data'));
    }

    public function update(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required',
            'phone' => 'required|min:10|max:10',
            'password' => 'required',

        ], [
            'name.required' => 'Name is required.',
            'email.required' => 'Email is required.',
            'phone.required' => 'Phone is required.',
            'password.required' => 'Password is required.'

        ]);

        if ($validator->passes()) {

            $emailcount = DB::table('invitees')->where('email', $request->email)->where('id', '!=', $request->id)->count();
            $phonecount = DB::table('invitees')->where('phone', $request->phone)->where('id', '!=', $request->id)->count();

            if($emailcount == 0 && $phonecount == 0) {
                $data = [
    
                    'name' => $request->name,
                    'email' => $request->email,
                    'phone' => $request->phone,
                    'password' => $request->password,
                    'updated_at' => $this->now
                ];
    
                $result = DB::table('invitees')->where('id', $request->id)->update($data);
    
                return true;
            } else {
                return response()->json(['error' => ['This email or phone has already been taken.']]);
            }

        } else {
            return response()->json(['error' => $validator->errors()->all()]);
        }
    }

    public function view(Request $request)
    {
        $invitee_users = DB::table('invitees')->where('id', $request->id)->first();

        return view('admin.inviteer.view')->with('invitee_users', $invitee_users);
    }

    public function status(Request $request)
    {
        $Coupon = DB::table('invitees')->where('id', $request->id)->update(['status' => $request->status]);

        return true;
    }

    public function delete(Request $request)
    {
        $Coupon = DB::table('invitees')->where('id', $request->id)->delete();

        return true;
    }

    private function random_strings()
    {
        // String of all alphanumeric character
        $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        
        // Shuffle the $str_result and returns substring
        // of specified length
        $rend_no = substr(str_shuffle($str_result), 0, 10);

        $exist_count = DB::table('invitees')->where('access_token', $rend_no)->count();

        if($exist_count == 0) {
            return $rend_no;
        } else {
            $this->random_strings();
        }
    }

    public function invitedInvites($invitee_id)
    {
        $invites_data = DB::table('invitees')->where('parent_id', $invitee_id)->get();

        $invites_user_data = DB::table('invited_users')->where('inviter_id', $invitee_id)->get();

        return view('admin.inviteer.invitedInvites')->with('invites_data', $invites_data)->with('invites_user_data', $invites_user_data)->with('parent_id', $invitee_id);
    }

    public function invitedInvitesChild($parent_id, $invitee_id)
    {
        $invites_data = DB::table('invitees')->where('parent_id', $invitee_id)->get();

        $invites_user_data = DB::table('invited_users')->where('inviter_id', $invitee_id)->get();
        
        return view('admin.inviteer.invitedInvitesChild')->with('invites_data', $invites_data)->with('invites_user_data', $invites_user_data)->with('parent_id', $parent_id)->with('invitee_id', $invitee_id);
    }

    public function invitedInvitesChildUsers($master_id,$parent_id,$invitee_id)
    {
        $invites_data = DB::table('invitees')->where('parent_id', $invitee_id)->get();

        $invites_user_data = DB::table('invited_users')->where('inviter_id', $invitee_id)->get();

        return view('admin.inviteer.invitedInvitesChildUsers')->with('invites_user_data', $invites_user_data)->with('master_id', $master_id)->with('parent_id', $parent_id)->with('invitee_id', $invitee_id);
    }

    //invited users reporting functions
    public function invitedUsers(Request $request, $category)
    {
        
        $title = 'Invited Users ('.$category.')';

        $search = '';
        if (isset($request->search) && $request->search != '') {

            $pagination = 'no';
            $search = $request->search;
            $data = DB::table('invited_users')
                ->leftJoin('invitees', 'invitees.id', 'invited_users.inviter_id')
                ->select([
                    'invited_users.*',
                    'invitees.name as invited_by',
                ])
                ->when($request->search, function ($query) use ($request) {
                    $query->where('invited_users.name', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('invited_users.referral_user', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('invitees.name', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('invited_users.email', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('invited_users.phone', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('invited_users.unique_code', 'LIKE', '%' . $request->search . '%');
                })
                ->where('invited_users.category',$category)
                ->get();
        } else {

            $data = DB::table('invited_users')
                ->leftJoin('invitees', 'invitees.id', 'invited_users.inviter_id')
                ->select([
                    'invited_users.*',
                    'invitees.name as invited_by',
                ])
                ->where('invited_users.category',$category)
                ->paginate(50);
            $pagination = 'yes';
        }

        return view('admin.inviteer.invitedUsers', compact('title', 'data', 'search', 'pagination', 'category')); 
    }

    public function exportInvitedUsers($category)
    {
        $invited_user_data = DB::table('invited_users')
            ->leftJoin('invitees', 'invitees.id', 'invited_users.inviter_id')
            ->select([
                'invited_users.*',
                'invitees.name as invited_by',
            ])
            ->where('invited_users.category',$category)
            ->get();

        $headerRow = ['Name', 'Email', 'Country Code', 'Phone', 'Company Name', 'Designation', 'Linkedin Profile Link', 'Unique Code', 'Profile Image', 'Created', 'RSVP Status'];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename='.$category.' Invited Users Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($invited_user_data as $data)
        {
            if($data->rsvp_status == 1) {
                $rsvp_status = 'Pending';
            } else if($data->rsvp_status == 2) {
                $rsvp_status = 'Accepted';
            } elseif($data->rsvp_status == 3) {
                $rsvp_status = 'Rejected';
            } elseif($data->rsvp_status == 4) {
                $rsvp_status = 'Badge Sent';
            }

            $arr = [
                $data->name,
                $data->email,
                $data->country_code,
                $data->phone,
                $data->company_name,
                $data->designation,
                $data->linkedin_profile_link,
                $data->unique_code,
                $data->image,
                date('Y-m-d h:i:s A', strtotime($data->created_at)),
                $rsvp_status,
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function allInviters(Request $request)
    {
        $title = 'All Inviters';

        $search = '';
        if (isset($request->search) && $request->search != '') {

            $pagination = 'no';
            $search = $request->search;
            $all_inviters_data = DB::table('invitees as a')
                ->leftJoin('invitees as b', 'b.id', 'a.parent_id')
                ->select([
                    'a.id',
                    'a.parent_id',
                    'b.name as parent_name',
                    'a.referral_user',
                    'a.category',
                    'a.name',
                    'a.email',
                    'a.country_code',
                    'a.phone',
                    'a.festival_dates',
                    'a.qty',
                    'a.remaining_qty',
                    'a.access_token',
                    'a.password',
                    'a.status',
                    'a.created_at',
                ])
                ->when($request->search, function ($query) use ($request) {
                    $query->where('b.name', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('a.referral_user', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('a.category', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('a.name', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('a.email', 'LIKE', '%' . $request->search . '%');
                    $query->orWhere('a.phone', 'LIKE', '%' . $request->search . '%');
                })
                ->get();
        } else {
            $all_inviters_data = DB::table('invitees as a')
                ->leftJoin('invitees as b', 'b.id', 'a.parent_id')
                ->select([
                    'a.id',
                    'a.parent_id',
                    'b.name as parent_name',
                    'a.referral_user',
                    'a.category',
                    'a.name',
                    'a.email',
                    'a.country_code',
                    'a.phone',
                    'a.festival_dates',
                    'a.qty',
                    'a.remaining_qty',
                    'a.access_token',
                    'a.password',
                    'a.status',
                    'a.created_at',
                ])
                ->paginate(50);
            $pagination = 'yes';
        }

        //dd($all_inviters_data);
        return view('admin.inviteer.allinviters', compact('title', 'all_inviters_data', 'search', 'pagination')); 
    }

    //advance filter
    public function advanceFilter(Request $request)
    {   
        if ($request->isMethod('post')) {
            $_SESSION['advance_search_filter'] = $request->all();
            $_SESSION['export_advance_search_filter'] = $request->all();

            $data = $this->getAdvanceFilterData($request->all());
            
            $data = $data->paginate();
            
            $pagination = 'yes';
            $title = 'Attendees Users';
            return view('admin.rsvp.index', compact('title'))->with('orders', $data)->with('pagination', $pagination);
        } else {
            return redirect()->route('rsvp');
        }
    }

    public function exportAdvanceFilterData()
    {
        $ticket_booking_data = $this->getAdvanceFilterData($_SESSION['export_advance_search_filter']);
        
        $ticket_booking_data = $ticket_booking_data->get();
        
        $headerRow = [
            'Sr.No.', 
            'Ticket Name', 
            'Name', 
            'Email ID', 
            'Phone',
            'Company',
            'Job Title',
            'Festival Dates',
            'What role does your organization play in the marketing ecosystem?',
            'Job Level',
            'Picture',
            'Refer1 Name',
            'Refer1 Email',
            'Refer1 Job Title',
            'Refer1 Company',
            'Refer2 Name',
            'Refer2 Email',
            'Refer2 Job Title',
            'Refer2 Company',
            'Eticket',
            'Created'
        ];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Attendees Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($ticket_booking_data as $data)
        {
            $arr = [
                $i,
                $data->ticket_name,
                $data->name,
                $data->email,
                $data->country_code.' '.$data->phone,
                $data->organization,
                $data->designation,
                $data->festival_dates,
                $data->organization_role,
                $data->job_level,
                $data->image_url,
                $data->refer1_full_name,
                $data->refer1_email,
                $data->refer1_job_title,
                $data->refer1_company,
                $data->refer2_full_name,
                $data->refer2_email,
                $data->refer2_job_title,
                $data->refer2_company,
                $data->eticket_path,
                date('Y-m-d h:i:s A', strtotime($data->created_at))
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function getAdvanceFilterData($data)
    {
        // dd($data);
        $ticket_booking =   DB::table('users')
                            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
                            ->where('users.status', 1)
                            ->select('users.*', 'tickets.name as ticket_name');
        
        if(!empty($data['search_field']))
        {
            $ticket_booking->where(function($query) use($data)
            {
                if(isset($data['search_type']) && $data['search_type'] == 'All')
                {
                    foreach($data['search_field'] as $key => $value)
                    {
                        if($value == 'name')
                        {
                            if($data['input_value'][$key] != '')
                            {
                                $query->where('users.name', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'email')
                        {
                            if($data['input_value'][$key] != '')
                            {
                                $query->where('users.email', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'phone')
                        {
                            if($data['input_value'][$key] != '')
                            {
                                $query->where(DB::raw("CONCAT(users.country_code,'',users.phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }
                    }

                    if(in_array('ticket_name',$data['search_field']))
                    {
                        $query->whereHas('tickets', function ($query2) use($data)
                        {
                            foreach($data['search_field'] as $key => $value)
                            {
                                if($value == 'ticket_name')
                                {
                                    if($data['input_value'][$key] != '')
                                    {
                                        $query2->where('name', 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                }
                            }
                        });
                    }
                }
                else
                {
                    foreach($data['search_field'] as $key => $value) {
                        if($value == 'name') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('users.name', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'email') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('users.email', 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }

                        if($value == 'phone') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere(DB::raw("CONCAT(users.country_code,'',users.phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                            }
                        }
                    }
                }
                
            });
        }

        return $ticket_booking;
    }

    public function advanceFilterAddMore(Request $request)
    {
        $numItems = $request->numItems+1;

        return view('admin.attendees.ajax.add_more_fields')->with('numItems', $numItems);
    }

    public function resendEmailGroup(Request $request)
    {
        $get_user_data = DB::table('invitees')
            ->select(['id', 'name', 'email', 'phone', 'category', 'qty', 'access_token','referral_user','festival_dates'])
            ->where('id', $request->user_id)
            ->first();
                
        $user_data = [
            'category' => $get_user_data->category,
            'qty' => $get_user_data->qty,
            'access_token' => $get_user_data->access_token,
            'name' => $get_user_data->name,
            'email' => $get_user_data->email,
            'from_name' => $get_user_data->referral_user,
            'referral_user'=>$get_user_data->referral_user,
            'festival_dates'=>$get_user_data->festival_dates,
        ];

        if($get_user_data->email != '') {
            Mail::to($get_user_data->email)->send(new Groupinviteesmail($user_data));
        }

        if($get_user_data->phone != '') {
            $updated_country_code = str_replace('+', '', $get_user_data->country_code);
            $phone_num = $updated_country_code.$get_user_data->phone;
            $url = route('invitees', ['access_token' => $get_user_data->access_token]);
            $this->whatsappInvitee($get_user_data->qty,$get_user_data->phone, $get_user_data->id, $url);
        }

        $data = array('status' => 'Success', 'message' => 'Resend mail successfully.');

        echo json_encode($data);
        exit();
    }

    public function resendEmailSingle(Request $request)
    {
        $get_user_data = DB::table('invited_users')
            ->select(['id', 'name', 'email', 'phone', 'category', 'unique_code', 'qrcode_path','eticket_path','referral_user','festival_dates'])
            ->where('id', $request->user_id)
            ->first();
                
        $user_data = [
            'category' => $get_user_data->category,
            'unique_code' => $get_user_data->unique_code,
            'qrcode_path' => $get_user_data->qrcode_path,
            'eticket_path' => $get_user_data->eticket_path,
            'name' => $get_user_data->name,
            'email' => $get_user_data->email,
            'referral_user'=>$get_user_data->referral_user,
            'festival_dates'=>$get_user_data->festival_dates,
        ];

        if($get_user_data->email != '') {
            Mail::to($get_user_data->email)->send(new InviteesConfirmationEmail($user_data));
        }

        if($get_user_data->phone != '') {
            $updated_country_code = str_replace('+', '', $get_user_data->country_code);
            $phone_num = $updated_country_code.$get_user_data->phone;
            $url = route('invitees', ['access_token' => $get_user_data->access_token]);
            $this->whatsappConfirmation($get_user_data->category, $get_user_data->phone, $get_user_data->eticket_path, $get_user_data->id);
        }

        $data = array('status' => 'Success', 'message' => 'Resend mail successfully.');

        echo json_encode($data);
        exit();
    }

    /* ============================================================================= Whatsapp messages =============================================================================== */

    private function whatsappConfirmation($category,$phone,$eticket_path, $saveInvitee_id)
    {
        
        if($category == 'Friend of the Festival' || $category == 'Guest Delegate') {
            $template_id = "9148967";
        } else {
            $template_id = "9148969";
        }
        
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => 'https://api.myvfirst.com/psms/servlet/psms.JsonEservice',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS =>'{
            "@VER": "1.2",
            "USER": {
                "@CH_TYPE": "4",
                "@UNIXTIMESTAMP": ""
            },
            "SMS": [
                {
                    "@UDH": "0",
                    "@CODING": "1",
                    "@TEXT": "",
                    "@TEMPLATEINFO": "'.$template_id.'",
                    "@CONTENTTYPE": "image/png",
                    "@TYPE": "image",
                    "@PROPERTY": "0",
                    "@MSGTYPE": "3",
                    "@MEDIADATA": "'.$eticket_path.'",
                    "@ID": "'.$saveInvitee_id.'",
                    "ADDRESS": [
                        {
                            "@FROM": "919999744527",
                            "@TO": "'.$phone.'",
                            "@SEQ": "1",
                            "@TAG": "C"
                        }
                    ]
                }
            ]
        }',
        CURLOPT_HTTPHEADER => array(
            'Authorization: Bearer eyJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJodHRwczovL2FwaS5teXZhbHVlZmlyc3QuY29tL3BzbXMiLCJzdWIiOiJqYWlwdXJsaXRmZXN0d2EiLCJleHAiOjMyMzM2MTI4MDZ9.ju5DP9B0UwKItI4Mn2y1ZKUA-M8Dc9YEbD8coeHI-wY',
            'Content-Type: application/json'
        ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        //echo $response;
    }

    private function whatsappInvitee($qty,$phone,$saveInvitee_id,$url)
    {
        
        if($qty == 1) {
            $template_id = "9148677~".$url;
        } else {
            $template_id = "9148683~".$qty."~".$url;
        }
        
        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => 'https://api.myvfirst.com/psms/servlet/psms.JsonEservice',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'POST',
        CURLOPT_POSTFIELDS =>'{
            "@VER": "1.2",
            "USER": {
                "@CH_TYPE": "4",
                "@UNIXTIMESTAMP": ""
            },
            "SMS": [
                {
                    "@UDH": "0",
                    "@CODING": "1",
                    "@TEXT": "",
                    "@TEMPLATEINFO": "'.$template_id.'",
                    "@CONTENTTYPE": "image/jpg",
                    "@TYPE": "image",
                    "@PROPERTY": "0",
                    "@MSGTYPE": "3",
                    "@MEDIADATA": "https://s3.ap-south-1.amazonaws.com/test.bucket.in/Smart-Reg/jlf2023/event/1673351664invitee_banner.jpg",
                    "@ID": "'.$saveInvitee_id.'",
                    "ADDRESS": [
                        {
                            "@FROM": "919999744527",
                            "@TO": "'.$phone.'",
                            "@SEQ": "1",
                            "@TAG": "C"
                        }
                    ]
                }
            ]
        }',
        CURLOPT_HTTPHEADER => array(
            'Authorization: Bearer eyJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJodHRwczovL2FwaS5teXZhbHVlZmlyc3QuY29tL3BzbXMiLCJzdWIiOiJqYWlwdXJsaXRmZXN0d2EiLCJleHAiOjMyMzM2MTI4MDZ9.ju5DP9B0UwKItI4Mn2y1ZKUA-M8Dc9YEbD8coeHI-wY',
            'Content-Type: application/json'
        ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        //echo $response;
    }
}