<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Models\Order;
use App\Models\TicketCategory;
use App\Models\TicketBooking;
use App\Models\HotelBooking;
use App\Models\Ticket;
use DB, Storage, Validator, Config, URL;

class SalesReportingController extends Controller
{
    public function __construct() 
    {
        session_start();
        $this->now = Carbon::now();
    }

    public function index()
    {
        $order_data = Order::where('orders.status', 1)
            ->where('orders.due', 0)
            ->join('users', 'orders.unique_order_id', '=', 'users.order_id')
            ->get();
        //dd($order_data);
        return view('admin.sales_reporting.index');
    }

    public function reportingLists($slug)
    {
        unset($_SESSION['advance_search_filter']);
        unset($_SESSION['export_advance_search_filter']);

        $ticket_ids = DB::table('tickets')->where('slug', $slug)->pluck('id', 'name')->toArray();
        $package_name = DB::table('tickets')->where('slug', $slug)->pluck('category', 'id')->first();

        $order_data = DB::table('orders')
            ->leftJoin('users', 'users.order_id', '=', 'orders.unique_order_id')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('users.master_user', 1)
            ->where('users.status', 1)
            ->whereIn('users.ticket_id',$ticket_ids)
            ->where('orders.status', 1)
            ->where('orders.due', 0)
            ->where('orders.transection_id', '!=', 'bypass')
            ->select([
                'orders.transection_id',
                'orders.payment_option',
                'orders.total',
                'orders.created_at',
                'tickets.category as ticket_category',
                'tickets.name as ticket_name',
                'users.order_id',
                'users.name as user_name',
                'users.last_name as user_last_name',
                'users.email as user_email',
                'users.country_code as country_code',
                'users.phone as user_phone',
            ])
            ->paginate(50);

        $pagination = 'yes';
        return view('admin.sales_reporting.reporting_lists')->with('pagination', $pagination)->with('order_data', $order_data)->with('package_name', $package_name)->with('slug', $slug);
    }

    public function orderSummary(Request $request)
    {   
        $slug = $request->slug;
        
        $users_data = DB::table('users')
            ->join('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('order_id', $request->order_id)
            ->select([
                'users.name',
                'users.last_name',
                'users.email',
                'users.country_code',
                'users.phone',
                'users.festival_dates',
                'tickets.price',
                'tickets.name as ticket_name',
            ])
            ->get();

        $order_data = Order::where('status', 1)
            ->where('due', 0)
            ->where('unique_order_id', $request->order_id)
            ->first();

        return view('admin.sales_reporting.order_summary')->with('users_data', $users_data)->with('order_data', $order_data);
    }

    public function advanceFilter(Request $request)
    {   
        $slug = $request->slug;
        if ($request->isMethod('post')) {
            $_SESSION['advance_search_filter'] = $request->all();
            $_SESSION['export_advance_search_filter'] = $request->all();

            $package_data = DB::table('tickets')->select('id', 'name')->where('slug', $slug)->first();
            $package_name = $package_data->name;

            $order_data = $this->getAdvanceFilterData($request->all(), $slug);
            
            $order_data = $order_data->paginate();
            //dd($order_data);
            $pagination = 'yes';
            return view('admin.sales_reporting.reporting_lists')->with('pagination', $pagination)->with('order_data', $order_data)->with('package_name', $package_name)->with('slug', $slug);
        } else {
            $slug = $_SESSION['advance_search_filter']['slug'];
            return redirect()->route('salesreporting-reportingLists', ['slug' => $slug]);
        }
    }

    public function exportAdvanceFilterData()
    {
        $slug = $_SESSION['export_advance_search_filter']['slug'];
        
        $order_data = $this->getAdvanceFilterData($_SESSION['export_advance_search_filter'], $slug);
        
        $order_data = $order_data->get();
        
        $headerRow = [
            'Sr.No.', 
            'Name', 
            'Email ID', 
            'Phone',
            'Company',
            'Job Title',
            'Festival Dates',
            'What role does your organization play in the marketing ecosystem?',
            'Job Level',
            'Picture',
            'Refer1 Name',
            'Refer1 Email',
            'Refer1 Job Title',
            'Refer1 Company',
            'Refer2 Name',
            'Refer2 Email',
            'Refer2 Job Title',
            'Refer2 Company',
            'Eticket',
            'Transaction ID', 
            'Payment Gateway', 
            'Order ID', 
            'Amount', 
            'Created'
        ];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Sales Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($order_data as $data)
        {
            $arr = [
                $i,
                $data->user_name,
                $data->user_email,
                $data->country_code.' '.$data->user_phone,
                $data->organization,
                $data->designation,
                $data->festival_dates,
                $data->organization_role,
                $data->job_level,
                $data->image_url,
                $data->refer1_full_name,
                $data->refer1_email,
                $data->refer1_job_title,
                $data->refer1_company,
                $data->refer2_full_name,
                $data->refer2_email,
                $data->refer2_job_title,
                $data->refer2_company,
                $data->eticket_path,
                $data->transection_id,
                $data->payment_option,
                $data->order_id,
                'USD '.$data->total,
                date('Y-m-d h:i:s A', strtotime($data->created_at))
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function getAdvanceFilterData($data, $slug)
    {
        //dd($data);
        $order_data = Order::query();
        $order_data->leftJoin('users', 'users.order_id', '=', 'orders.unique_order_id')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('users.master_user', 1)
            ->where('users.status', 1)
            ->where('orders.status', 1)
            ->where('orders.due', 0)
            ->where('orders.transection_id', '!=', 'bypass')
            ->select([
                'orders.transection_id',
                'orders.payment_option',
                'orders.total',
                'orders.created_at',
                'tickets.category as ticket_category',
                'tickets.name as ticket_name',
                'users.order_id',
                'users.name as user_name',
                'users.last_name as user_last_name',
                'users.email as user_email',
                'users.country_code as country_code',
                'users.phone as user_phone',
                'users.organization as organization',
                'users.designation as designation',
                'users.festival_dates as festival_dates',
                'users.organization_role as organization_role',
                'users.job_level as job_level',
                'users.image_url as image_url',
                'users.refer1_full_name as refer1_full_name',
                'users.refer1_email as refer1_email',
                'users.refer1_job_title as refer1_job_title',
                'users.refer1_company as refer1_company',
                'users.refer2_full_name as refer2_full_name',
                'users.refer2_email as refer2_email',
                'users.refer2_job_title as refer2_job_title',
                'users.refer2_company as refer2_company',
                'users.eticket_path as eticket_path',
            ]);

        if(!empty($data['search_field'])) {
            $order_data->where(function($query) use($data) {
                if(isset($data['search_type']) && $data['search_type'] == 'All') {
                    foreach($data['search_field'] as $key => $value) {
                        if($value == 'transaction_id') {
                            if($data['input_value'][$key] != '') {
                                $query->where('transection_id', '=', $data['input_value'][$key]);
                            }
                        }

                        if($value == 'payment_gateway') {
                            if($data['payment_gateway'][$key] != '') {
                                $query->where('payment_option', '=', $data['payment_gateway'][$key]);
                            }
                        }

                        if($value == 'order_id') {
                            if($data['input_value'][$key] != '') {
                                $query->where('unique_order_id', '=', $data['input_value'][$key]);
                            }
                        }
                        
                        if($value == 'date') {
                            if($data['date_type'][$key] == 'between') {
                                if($data['start_date'][$key] != '' && $data['end_date'][$key] != '') {
                                    $query->where(function($query3) use($data, $key) {
                                        $query3->whereDate('orders.created_at', '>=', $data['start_date'][$key]);
                                        $query3->whereDate('orders.created_at', '<=', $data['end_date'][$key]);
                                    });
                                }
                            } else if($data['date_type'][$key] == 'equal_to') {
                                if($data['start_date'][$key] != '') {
                                    $query->whereDate('orders.created_at', '=', $data['start_date'][$key]);
                                }
                            }
                        }
                    }

                    if(in_array('name',$data['search_field']) || in_array('email',$data['search_field']) || in_array('phone',$data['search_field'])) {
                        $query->whereHas('users', function ($query2) use($data){
                            foreach($data['search_field'] as $key => $value) {
                                if($value == 'name') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where(DB::raw("CONCAT(name,' ',last_name)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                } else if($value == 'email') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where($value, 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                } else if($value == 'phone') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where(DB::raw("CONCAT(country_code,'',phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                }
                            }
                        });
                    }
                } else {
                    foreach($data['search_field'] as $key => $value) {
                        if($value == 'transaction_id') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('transection_id', '=', $data['input_value'][$key]);
                            }
                        }

                        if($value == 'payment_gateway') {
                            if($data['payment_gateway'][$key] != '') {
                                $query->orWhere('payment_option', '=', $data['payment_gateway'][$key]);
                            }
                        }

                        if($value == 'order_id') {
                            if($data['input_value'][$key] != '') {
                                $query->orWhere('unique_order_id', '=', $data['input_value'][$key]);
                            }
                        }

                        if($value == 'date') {
                            if($data['date_type'][$key] == 'between') {
                                if($data['start_date'][$key] != '' && $data['end_date'][$key] != '') {
                                    $query->where(function($query3) use($data, $key) {
                                        $query3->whereDate('orders.created_at', '>=', $data['start_date'][$key]);
                                        $query3->whereDate('orders.created_at', '<=', $data['end_date'][$key]);
                                    });
                                }
                            } else if($data['date_type'][$key] == 'equal_to') {
                                if($data['start_date'][$key] != '') {
                                    $query->orWhereDate('orders.created_at', '=', $data['start_date'][$key]);
                                }
                            }
                        }
                    }

                    if(in_array('name',$data['search_field']) || in_array('email',$data['search_field']) || in_array('phone',$data['search_field'])) {
                        $query->orWhereHas('users', function ($query2) use($data){
                            $user_val_arr = [];
                            foreach($data['search_field'] as $key => $value) {
                                if($value == 'email') {
                                    $user_val_arr[] = $value;
                                    if($data['input_value'][$key] != '') {
                                        if(count($user_val_arr) == 1) {
                                            $query2->where($value, 'LIKE', '%'.$data['input_value'][$key].'%');
                                        } else {
                                            $query2->orWhere($value, 'LIKE', '%'.$data['input_value'][$key].'%');
                                        }
                                    }
                                }  else if($value == 'name') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where(DB::raw("CONCAT(name,' ',last_name)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                }  else if($value == 'phone') {
                                    if($data['input_value'][$key] != '') {
                                        $query2->where(DB::raw("CONCAT(country_code,'',phone)"), 'LIKE', '%'.$data['input_value'][$key].'%');
                                    }
                                }
                            }
                        });
                    }
                }
                
            });
        }

        return $order_data->groupBy('orders.unique_order_id');
    }

    public function exportAllData($slug)
    {
        $ticket_ids = DB::table('tickets')->where('slug', $slug)->pluck('id', 'name')->toArray();

        $order_data = DB::table('orders')
            ->leftJoin('users', 'users.order_id', '=', 'orders.unique_order_id')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->where('users.master_user', 1)
            ->where('users.status', 1)
            ->whereIn('users.ticket_id',$ticket_ids)
            ->where('orders.status', 1)
            ->where('orders.due', 0)
            ->where('orders.transection_id', '!=', 'bypass')
            ->select([
                'orders.transection_id',
                'orders.payment_option',
                'orders.total',
                'orders.created_at',
                'tickets.category as ticket_category',
                'tickets.name as ticket_name',
                'users.order_id',
                'users.name as user_name',
                'users.last_name as user_last_name',
                'users.email as user_email',
                'users.country_code as country_code',
                'users.phone as user_phone',
                'users.organization as organization',
                'users.designation as designation',
                'users.festival_dates as festival_dates',
                'users.organization_role as organization_role',
                'users.job_level as job_level',
                'users.image_url as image_url',
                'users.refer1_first_name as refer1_first_name',
                'users.refer1_last_name as refer1_last_name',
                'users.refer1_email as refer1_email',
                'users.refer1_job_title as refer1_job_title',
                'users.refer1_company as refer1_company',
                'users.refer2_first_name as refer2_first_name',
                'users.refer2_last_name as refer2_last_name',
                'users.refer2_email as refer2_email',
                'users.refer2_job_title as refer2_job_title',
                'users.refer2_company as refer2_company',
                'users.eticket_path as eticket_path',
            ])
            ->get();

        $headerRow = [
            'Sr.No.', 
            'Name', 
            'Email ID', 
            'Phone',
            'Company',
            'Job Title',
            'Festival Dates',
            'What role does your organization play in the marketing ecosystem?',
            'Job Level',
            'Picture',
            'Refer1 Name',
            'Refer1 Email',
            'Refer1 Job Title',
            'Refer1 Company',
            'Refer2 Name',
            'Refer2 Email',
            'Refer2 Job Title',
            'Refer2 Company',
            'Eticket',
            'Transaction ID', 
            'Payment Gateway', 
            'Order ID', 
            'Amount', 
            'Created'
        ];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Sales Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($order_data as $data)
        {
            $arr = [
                $i,
                $data->user_name,
                $data->user_email,
                $data->country_code.' '.$data->user_phone,
                $data->organization,
                $data->designation,
                $data->festival_dates,
                $data->organization_role,
                $data->job_level,
                $data->image_url,
                $data->refer1_full_name,
                $data->refer1_email,
                $data->refer1_job_title,
                $data->refer1_company,
                $data->refer2_full_name,
                $data->refer2_email,
                $data->refer2_job_title,
                $data->refer2_company,
                $data->eticket_path,
                $data->transection_id,
                $data->payment_option,
                $data->order_id,
                'USD '.$data->total,
                date('Y-m-d h:i:s A', strtotime($data->created_at))
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile);
    }

    public function advanceFilterAddMore(Request $request)
    {
        $numItems = $request->numItems+1;

        return view('admin.sales_reporting.ajax.add_more_fields')->with('numItems', $numItems);
    }
}
