<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use DB, Storage;
use Carbon\Carbon;
use Validator;

class SearchController extends Controller
{
    public function __construct()
    {
        $this->now = Carbon::now();
    }

    public function index(Request $request)
    {
        $title = 'Search Users';
		$pagination = 'yes'; 
		
		$keyword 	= $request->search;
		$startDate 	= $request->start_date;

		$query = DB::table('users')
			->leftJoin('user_types', 'users.user_type_id', '=', 'user_types.id')
			->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
			->select('users.*','tickets.name as ticket_name', 'user_types.category');

		if ($keyword) {
			$title = 'Search of '.$keyword.' Users';
			if ($keyword == 'success') {
				$query->where('users.status', '=', 1);
			} else { 
				$query->where(function($query) use($keyword){
					$columns = ['users.name', 'users.email', 'users.phone', 'users.country', 'users.state', 'users.city', 'users.created_at', 'users.order_id','tickets.name','user_types.category'];
					foreach ($columns as $column) {
						$query->orWhere($column, 'like', '%' . $keyword . '%');
					}
				});
			}
		}
		if ($startDate) {
			$query->whereDate('users.created_at', '=', $startDate);
		}
		$users = $query->paginate(50);
        return view('admin.search.index', compact('title', 'users', 'keyword', 'startDate', 'pagination')); 
    }
    
    public function view(Request $request)
    {
        $users = DB::table('users')->where('id', $request->id)->first();
        $orders = DB::table('orders')->where('user_id', $request->id)->first();

        return view('admin.search.view')->with('users', $users)->with('orders', $orders);
    }

    public function filterData(Request $request)
    {	
		$keyword 	= $request->search;
		$startDate 	= $request->start_date;

		$query = DB::table('users')
			->leftJoin('user_types', 'users.user_type_id', '=', 'user_types.id')
			->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
			->select('users.*','tickets.name as ticket_name', 'user_types.category');

		if ($keyword) {
			$title = 'Search of '.$keyword.' Users';
			if ($keyword == 'success') {
				$query->where('users.status', '=', 1);
			} else {
				$query->where(function($query) use($keyword){
					$columns = ['users.name', 'users.email', 'users.phone', 'users.country', 'users.state', 'users.city', 'users.created_at', 'users.order_id','tickets.name','user_types.category'];
					foreach ($columns as $column) {
						$query->orWhere($column, 'like', '%' . $keyword . '%');
					}
				});
			}
		}
		if ($startDate) {
			$query->whereDate('users.created_at', '=', $startDate);
		}
		$users = $query->get();
		// dd($users);
		$headerRow = [
            'Sr.No.', 
            'Ticket Name', 
            'Name', 
            'Email ID', 
            'Phone',
            // 'Age Group',
            'Country',
            'State',
            'City',
            // 'School Name',
            // 'School Address', 
            'Organization',
            'Designation',
            'Festival Dates',
            'Organization Role',
            'Job Level',
            'Picture', 
            'Eticket',
            'Status',
            'Created'
        ];
        
        $delimiter = ',';
        $enclosure = '"';
        $csvFile = fopen('php://output', 'w');
        header('Content-type: application/download');
        header('Content-Disposition: attachment; filename=Attendees Reports.csv');
        fputcsv($csvFile,$headerRow, $delimiter, $enclosure);
        
        $i=1;
        foreach($users as $data)
        {
			$fail = $data->status == 0 ? 'Failure' : 'Success';			
            $arr = [
                $i,
                $data->ticket_name,
                $data->name,
                $data->email,
                $data->country_code.' '.$data->phone,
                // $data->age_group,
                $data->country,
                $data->state,
                $data->city,
                // $data->school_name,
                // $data->school_address, 
                $data->organization,
                $data->designation,
                $data->festival_dates,
                $data->organization_role,
                $data->job_level,
                $data->image_url, 
                $data->eticket_path,
				$fail,
                date('Y-m-d h:i:s A', strtotime($data->created_at))
            ];
            fputcsv($csvFile, $arr, $delimiter, $enclosure);
            $i++;
        }
        fclose($csvFile); 
    }
}
