<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Models\Ticket;
use Illuminate\Http\JsonResponse;
use App\Libraries\Rest;
use App\Rules\FestivalDates;
use Illuminate\Support\Str; 
use DB, Storage, Validator, Config;
use DataTables;

class TicketsController extends Controller
{
    public function __construct() {
        $this->now = Carbon::now();
    }

    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = DB::table('tickets')->select(['id', 'name','shortcode', 'category', 'price', 'gst', 'other_charges', 'status'])->get()->toArray();
            return Datatables::of($data)->addIndexColumn()
                ->editColumn('status', function ($data) {
                    if ($data->status == 0) return 'Deactivated';
                    if ($data->status == 1) return 'Activated';
                })
                ->addColumn('action', function($row){
                    
                    if($row->status == 1) {
                        $status = 0;
                        $status_label = 'Deactivate';
                    } else {
                        $status = 1;
                        $status_label = 'Activate';
                    }

                    $btn = '<div class="dropdown ticket-option">
                                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
                                    <div class="dot-three"></div>
                                </button>
                                <div class="dropdown-menu">
									<a href="'.(route('tickets-edit',$row->id)).'" class="dropdown-item " >
                                        <i class="fas fa-pencil-alt"></i> Edit
                                    </a>
									<a href="'.(route('tickets-view',$row->id)).'" class="dropdown-item " >
                                        <i class="fas fa-eye"></i> View
                                    </a>
                                    <a class="dropdown-item delete-action" type="button" data-id="'.$row->id.'">
                                        <i class="fas fa-trash"></i> Delete
                                    </a> 
                                    <button class="dropdown-item status-action-btn" type="button" data-status="'.$status.'" data-id="'.$row->id.'">
                                        <i class="fas fa-cog"></i> '.$status_label.'
                                    </button>
                                    
                                </div>
                            </div>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view('admin.tickets.index');
    }

    public function add()
    {
        $ticket_category = DB::table('user_types')->where('type', 'Online')->pluck('category', 'id')->toArray();
        return view('admin.tickets.add')->with('ticket_category', $ticket_category);
    }
    
	public function edit(Request $request)
    {
        $ticket_category = DB::table('tickets')->groupBy("category")->pluck('category', 'id')->toArray();
		$ticket = Ticket::where('id', $request->id)->where('status', 1)->first();
		return View('admin.tickets.edit', compact('ticket_category','ticket'));
    }

    public function save(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'category' => 'required',
            'category_name' =>  'required_if:category,=,Other',
            'name' => 'required',
            'shortcode' => 'required',
            'description' => 'required',
            'price' => 'required|numeric|min:0|',
            'min_qty' => 'required|numeric|min:0|',
            'max_qty' => 'required|numeric|min:1|',
            'day_qty' => 'required',
        ],[
            'category.required' => 'Category is required.',
            'category_name.required' => 'Category Name is required.',
            'name.required' => 'Name is required.',
            'shortcode.required' => 'Shortcode is required.',
            'description.required' => 'Description is required.',
            'price.required' => 'Price is required.',
            'min_qty.required' => 'Min quantity is required.',
            'max_qty.required' => 'Max quantity is required.',
            'day_qty.required' => 'Day quantity is required.',
        ]);

        if ($validator->passes()) {
            $tickets = new Ticket();
			
            $category_name = $request->category == 'Other' ? $request->category_name : $request->category;

            if(!empty($category_name)) {
				$slug = str_replace(' ', '', $category_name);
			}

            $data = [
                'name' => $request->name,
                'shortcode' => $request->shortcode,
                'category' => $category_name,
                'description' => $request->description,
                'price' => $request->price,
                'min_qty' => $request->min_qty,
                'max_qty' => $request->max_qty,
                'day_qty' => $request->day_qty,
            ]; 
			if(empty($request->id)) {
				$ndata = [
					'slug' => $slug,
					'status' => 1,
					'created_at' => $this->now,
					'updated_at' => $this->now,
				];

				$data = array_merge($data,$ndata);
				$tickets->create($data);
			}else{
				$ndata = [
                    'slug' => $slug,
                    'updated_at' => $this->now
                ];

				$data = array_merge($data,$ndata); 
				$tickets = $tickets->where('id', $request->id)->update($data);
			}
			return true;
        } else {
            return response()->json(['error'=>$validator->errors()->all()]);
        }
    }

    public function view(Request $request)
    {
        $tickets = Ticket::where('id', $request->id)->first();

        return view('admin.tickets.view')->with('tickets', $tickets);
    }

    public function status(Request $request)
    { 
        $tickets = Ticket::where('id', $request->id)->update(['status' => $request->status]);
        return true;
    }

    function delete(Request $request): JsonResponse{
		if (empty($request->id)) {
			return (new Rest)->json([
                "status"	=>	true,
                "message"	=>	'Ticket id been not Not found.'
            ]);
		}
        $obj			=	Ticket::Find($request->id);
        if ($obj) {
            Ticket::where("id",$request->id)->delete();
            return (new Rest)->json([
                "status"	=>	true,
                "message"	=>	'Ticket has been deleted successfully.'
            ]);
        } else {
            return (new Rest)->json([
                "status"	=>	false,
                "message"	=>	'Something went wrong. Please try again.'
            ]);
        }
    } 
	
}
