<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Models\Ticket;
use App\Models\TicketCategory;
use App\Rules\FestivalDates;
use DB, Storage, Validator, Config;
use DataTables;
use App\Services\S3Service;

class UserTypesController extends Controller
{
    public function __construct() {
        $this->now = Carbon::now();
    }

    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = DB::table('user_types')->get()->toArray();
            
            return Datatables::of($data)->addIndexColumn()
            ->editColumn('mbadge', function ($data) {
                return '<a href="'.$data->mbadge.'" target="_blank">View M-Badge</a>';
                
            })
            ->addColumn('action', function($row){
                    
                    $btn = '<div class="dropdown ticket-option">
                                <button type="button" class="btn btn-default dropdown-toggle" data-toggle="dropdown">
                                    <div class="dot-three"></div>
                                </button>
                            </div>';
                    return $btn;
                })
                ->rawColumns(['action','mbadge'])
                ->make(true);
        }

        return view('admin.usertypes.index');
    }

    public function add()
    {
        return view('admin.usertypes.add');
    }

    public function save(Request $request,S3Service $S3Service)
    {
        $validator = Validator::make($request->all(), [
            'type' => 'required',
            'category' => 'required',
            //'shortcode' => 'required|unique:user_types', 
        ]);

        if (!$validator->passes()) {
            return response()->json(['error' => $validator->errors()->messages()]);
        }

        if($request->type != 'Online' && $request->shortcode == '') {
            return response()->json(['error' => ['shortcode' =>['Shortcode is required.']]]);
        }

        $mbadge = '';
        if(isset($request->mbadge) && !empty($request->mbadge)) {
            $mbadge = $S3Service->getImageUrl('mbadges', $request->mbadge);
        }

        if ($validator->passes()) {
            $data = [
                'type' => $request->type,
                'category' => $request->category,
                'shortcode' => $request->shortcode,
                'mbadge' => $mbadge,
                'slug' => str_replace(' ', '', $request->category),
                'status' => 1,
                'created_at' => $this->now,
                'updated_at' => $this->now,
            ];
            
            $save = DB::table('user_types')->insertGetId($data);
            
            return response()->json(['type'=>'Success', 'message'=> 'User type added successfully.']);

        }
    }

    public function view(Request $request)
    {
        $tickets = Ticket::where('id', $request->id)->first();

        return view('admin.tickets.view')->with('tickets', $tickets);
    }

    public function status(Request $request)
    {
        $tickets = Ticket::where('id', $request->id)->update(['status' => $request->status]);

        return true;
    }
}
