<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Session, Config, DB, Storage;
use JeroenDesloovere\VCard\VCard;

class OngroundController extends Controller
{
    public function __construct()
    {
        $this->now = Carbon::now();

        $prefix_uniquecode = DB::table('events')->select('prefix_uniquecode', 's3bucket_name', 'mbadge_bg', 'gst')->where('status', 1)->first();
        $this->unique_code_prefix = $prefix_uniquecode->prefix_uniquecode;
        $this->s3bucket_name = $prefix_uniquecode->s3bucket_name;
        $this->mbadge_bg = $prefix_uniquecode->mbadge_bg;
        $this->gst = $prefix_uniquecode->gst;
    }

    public function usersave(Request $request)
    {
        //Validation dynamic
        $formcategory  =  'Onground Registrations';
        $interface_data = DB::table('user_types')->where('category', $formcategory)->first();

        $interface_form_data = DB::table('form_templates')->where('alias', $formcategory)->get();
        
        $user_data_array = [];
        foreach ($interface_form_data as $key => $value) {
            //dd($value->required);
            /* dd($request->$value->field_name); */

            if ($value->required == 1) {
                $field = $value->field_name;
                if ($request->input($field) == Null) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is required.']]);
                }
            }

            if ($value->unique_field == 1) {
                $field = $value->field_name;
                $field_value = $request->input($field);
                $data_exist = User::where($field, $field_value)->where('id', $request->interface_id)->count();
                if ($data_exist != 0) {
                    return response()->json(['error' => [ucwords($value->field_title) . ' is already exist.']]);
                }
            }

            if ($value->field_type == 'phone') {
                $field = $value->field_name;
                $field_value = $request->input($field);
                $field_length = strlen($field_value);

                if ($field_length > 0) {
                    $user_data_array['country_code'] = $request->input('country_code');
                    if (strpos($field_value, '+') !== false || $field_length < 8 || $field_length > 15) {
                        return response()->json(['error' => [ucwords($value->field_title) . ' is invalid.']]);
                    }
                }
            }

            if ($value->field_type == 'checkbox') {
                $user_data_array[$value->field_name] = implode(',', $request->input($value->field_name));
            } else {
                $user_data_array[$value->field_name] = $request->input($value->field_name);
            }
        }
       
        $unique_code = $this->unique_code_prefix.$interface_data->shortcode.$this->unique_code();
        $qrcode_path = $this->generateQr($unique_code);
        $eticket_path = $this->eticket($qrcode_path, $unique_code, ucwords($user_data_array['name']), ucwords($user_data_array['organization']));
        
        $data = [
            'user_type_id' => $interface_data->id,
            'unique_code' => $unique_code,
            'qrcode_path' => $qrcode_path,
            'eticket_path' => $eticket_path,
            'status' => 1,
            'created_at' => $this->now,
            'is_printed' => 1,
            'printed_at' => $this->now,
            'updated_at' => $this->now,
        ];


        $emailid  = "";
        if (isset($user_data_array['email'])) {
            $emailid   =  $user_data_array['email'];
        }
        $user_data_array['email'] = strtolower($emailid);

        if($user_data_array['category'] == 'BLANK') {
            unset($user_data_array['category']);
            $user_data_array['category'] = '';
        }

        $alloted_festival_dates = [];
        $festival_dates = explode(',', $user_data_array['festival_dates']);
        
        if(in_array('All', $festival_dates)) {
            $alloted_festival_dates[] = 18;
            $alloted_festival_dates[] = 19;
            $alloted_festival_dates[] = 20;
        } 
        if(in_array('18 Full Day', $festival_dates)) {
            $alloted_festival_dates[] = 18;
        }
        if(in_array('19 Full Day', $festival_dates) || in_array('19 Evening', $festival_dates)) {
            $alloted_festival_dates[] = 19;
        }
        if(in_array('20 Full Day', $festival_dates) || in_array('20 Evening', $festival_dates)) {
            $alloted_festival_dates[] = 20;
        }

        $user_data_array['festival_dates'] = implode(',',$alloted_festival_dates);

        $final_user_data = array_merge($user_data_array, $data);

        $result_id = DB::table('users')->insertGetId($final_user_data);
        
        if($result_id) {
            $vcard_path = $this->generateVcard($result_id,ucwords($user_data_array['name']), ucwords($user_data_array['organization']), '', $user_data_array['email'], $user_data_array['phone']);
            $update_users = DB::table('users')->where('id', $result_id)->update(['vcard' => $vcard_path]);

            $data1 = [
                'user_id'       => $result_id,
                'created_at'    => $this->now,
                'updated_at'    => $this->now
            ];

            DB::table('redeem')->insert($data1);
        }
        $name_arr = explode(' ', $user_data_array['name']);

        $final_user_data['first_name'] = $name_arr[0];
        unset($name_arr[0]);
        $last_name = implode(' ', $name_arr);
        $final_user_data['last_name'] = $last_name;
        $final_user_data['vcard'] = 'https://eemagine.eemaindia.com/share/'.$result_id;
        
        return response()->json(['status' => true, 'message' => 'User added successfully.', 'data' => $final_user_data]);
    }

    private function generateVcard($user_id,$first_name, $organization, $designation, $email, $phone) 
    {
        $vcard = new VCard();

        // define variables
        $firstname = $first_name;
        $lastname = '';
        $additional = '';
        $prefix = '';
        $suffix = '';

        // add personal data
        $vcard->addName($lastname, $firstname, $additional, $prefix, $suffix);

        // add work data
        $vcard->addCompany($organization);
        $vcard->addJobtitle($designation);
        $vcard->addRole($designation);
        $vcard->addEmail($email);
        //$vcard->addPhoneNumber(6367302304, 'PREF;WORK');
        $vcard->addPhoneNumber($phone, 'WORK');
        //$vcard->addAddress(null, null, 'abc', 'def', null, 'ghi', 'Jaipur');
        //$vcard->addLabel('street, worktown, workpostcode Belgium');
        //$vcard->addURL('http://www.jeroendesloovere.be');

        //$vcard->addPhoto(__DIR__ . '/landscape.jpeg');

        // return vcard as a string
        //return $vcard->getOutput();

        // return vcard as a download
        //$vcard->save();
        //dd($vcard->getOutput());
        //return $vcard->download();

        $upload_name = "Smart-Reg/eemaindia/vcard/$first_name$user_id.vcf";
        $eticketpath = Storage::disk('s3')->put($upload_name, $vcard->getOutput(), 'public');
        $eticketpath = Storage::disk('s3')->url($upload_name);

        return $eticketpath;
    }

    /* ------------------------------------------------------------------------Comman functions--------------------------------------------------------------------------------------- */
    public function unique_code()
    {
        $unique_code = DB::table('online_registration_unique_codes')->where('is_used', 0)->first();
        DB::table('online_registration_unique_codes')->where('id', $unique_code->id)->update(['is_used' => 1, 'modified' => Carbon::now()]);
        return $unique_code->unique_code;
    }

    public function generateQr($unique_code)
    {
        $document = QrCode::format('png')->margin(2)->size(500)->backgroundColor(255, 255, 255)->generate($unique_code);
        $upload_name = "Smart-Reg/'.$this->s3bucket_name.'/qrcodes/" . $unique_code . ".png";
        $document_path = Storage::disk('s3')->put($upload_name, $document, 'public');
        $document_path = Storage::disk('s3')->url($upload_name);

        return $document_path;
    }

    public function eticket($qr_path, $unique_code, $name, $company)
    {
        $new        = @imagecreatefrompng($qr_path);
        $master     = @imagecreatefrompng($this->mbadge_bg);

        imagecopyresampled($master, $new, 220, 550, 0, 0, 300, 300, 500, 500);

        //$font = realpath('public/arial.ttf');
        $white = imagecolorallocate($master, 255, 255, 255);
        $orange = imagecolorallocate($master, 255, 107, 1);
        $black = imagecolorallocate($master, 0, 0, 0);
        $bold_font = realpath('public/SF-Pro-Text-Bold.otf');
        $font = realpath('public/SF-Pro-Text-Regular.otf');

        //Unique Code Print ============================================================================================================================
        $unique_code_font_size = 20;

        list($x, $y) = $this->ImageTTFCenter($master, $unique_code, $font, $unique_code_font_size);
        imagettftext($master, $unique_code_font_size, 0, $x, $y + 220, $white, $font, $unique_code);

        //Name Print ===================================================================================================================================
        $name = explode(' ', $name);
        $first_name = $name[0];
        unset($name[0]);
        $last_name = implode(' ', $name);

        if (!empty($last_name)) {
            $first_name_font_size = 40;
            list($x, $y) = $this->ImageTTFCenter($master, $first_name, $font, $first_name_font_size);
            imagettftext($master, $first_name_font_size, 0, $x, 380, $orange, $font, $first_name);

            $last_name_font_size = 30;
            list($x, $y) = $this->ImageTTFCenter($master, $last_name, $font, $last_name_font_size);
            imagettftext($master, $last_name_font_size, 0, $x, 430, $white, $font, $last_name);
        } else {
            $first_name_font_size = 40;
            list($x, $y) = $this->ImageTTFCenter($master, $first_name, $font, $first_name_font_size);
            imagettftext($master, $first_name_font_size, 0, $x, 400, $orange, $font, $first_name);
        }

        //company Print ============================================================================================================================
        $new_company = wordwrap($company, 45, "<br>\n");
        $new_company = str_replace(array("\r", "\n"), '', $new_company);
        $final_company = explode('<br>', $new_company);

        $company_count = count($final_company);
        //dd($name_count);
        if ($company_count == 1) {
            $company_font_size = 25;

            $company1 = $final_company[0];
            list($x, $y) = $this->ImageTTFCenter($master, $company1, $font, $company_font_size);
            imagettftext($master, $company_font_size, 0, $x, 490, $white, $font, $company1);
        } else {

            $company_font_size = 25;

            $company1 = $final_company[0];
            $company2 = $final_company[1];

            list($x, $y) = $this->ImageTTFCenter($master, $company1, $font, $company_font_size);
            imagettftext($master, $company_font_size, 0, $x, 490, $white, $font, $company1);

            list($x, $y) = $this->ImageTTFCenter($master, $company2, $font, $company_font_size);
            imagettftext($master, $company_font_size, 0, $x, 530, $white, $font, $company2);
        }

        // header('Content-Type: image/png');

        $masterImg = imagepng($master, 'public/eticket/' . $unique_code . ".png");

        if ($masterImg) {
            $upload_name = "Smart-Reg/" . $this->s3bucket_name . "/eticket/" . time() . $unique_code . ".png";
            $eticketpath = Storage::disk('s3')->put($upload_name, file_get_contents('public/eticket/' . $unique_code . '.png'), 'public');
            $eticketpath = Storage::disk('s3')->url($upload_name);
            unlink('public/eticket/' . $unique_code . '.png');
        }

        // echo $eticketpath; die();
        return $eticketpath;
    }

    function ImageTTFCenter($image, $text, $font, $size, $angle = 0)
    {
        $xi = imagesx($image);
        $yi = imagesy($image);

        $box = imagettfbbox($size, $angle, $font, $text);

        $xr = abs(max($box[2], $box[4]));
        $yr = abs(max($box[5], $box[7]));

        $x = intval(($xi - $xr) / 2);
        $y = intval(($yi + $yr) / 2);
        // die();

        return array($x, $y);
    }
}
