<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Payment;
use Illuminate\Support\Facades\Mail;
use App\Models\Interfaces;
use App\Models\User;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Carbon\Carbon;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Session, Config, DB, Storage;
use App\Mail\UserRegistrationOrderSummaryEmail;
use App\Mail\UserRegistrationEmail;
use App\Mail\ExhibitorRegistrationEmail;
use App\Mail\SpeakerRegistrationEmail;
use App\Mail\OngroundUserEmail;
use App\Mail\VipRegistrationEmail;
use App\Http\Controllers\WhatsappmsgController;
use JeroenDesloovere\VCard\VCard;

class RedemptionController extends Controller
{
    public function __construct()
    {
        $this->now = Carbon::now();
    }

    public function search_user($search)
    {
        $userdata = User::where('status', 1)
            ->where(function ($q) use ($search) {
                $q->where('name', 'LIKE', '%' . $search . '%')
                    ->orWhere('email', 'LIKE', '%' . $search . '%')
                    ->orWhere('phone', 'LIKE', '%' . $search . '%')
                    ->orWhere('unique_code', 'LIKE', '%' . $search . '%');
            })
            ->get();

        $final_data = [];
        foreach ($userdata as $key => $value) {
            $new_name = explode(' ', $value->name);
            $first_name = $new_name[0];
            unset($new_name[0]);
            $last_name = implode(' ', $new_name);


            $final_data[] = [
                'id' => $value->id,
                'first_name' => $first_name,
                'last_name' => $last_name . ' ' . $value->last_name,
                'email' => $value->email,
                'phone' => $value->phone,
                'organization' => $value->organization,
                'designation' => $value->designation,
                'unique_code' => $value->unique_code,
                'category' => $value->category,
                'vcard' => 'https://eemagine.eemaindia.com/share/' . $value->id,
                //'vcard' => 'https://t.com/'.$value->id,
                'is_printed' => $value->is_printed
            ];
        }

        if (!empty($userdata)) {
            $response = ['status' => true, 'message' => 'User Found.', 'data' =>  $final_data];
        } else {
            $response = ['status' => false, 'message' => 'User Not Found.'];
        }

        echo json_encode($response);
        die;
    }

    public function search_user_k2mini($search)
    {
        $userdata = User::where('status', 1)->where('unique_code', $search)->get();

        $final_data = [];
        foreach ($userdata as $key => $value) {
            $new_name = explode(' ', $value->name);
            $first_name = $new_name[0];
            unset($new_name[0]);
            $last_name = implode(' ', $new_name);


            $final_data[] = [
                'id' => $value->id,
                'first_name' => $first_name,
                'last_name' => $last_name . ' ' . $value->last_name,
                'email' => $value->email,
                'phone' => $value->phone,
                'organization' => $value->organization,
                'designation' => $value->designation,
                'unique_code' => $value->unique_code,
                'category' => $value->category,
                'vcard' => 'https://eemagine.eemaindia.com/share/' . $value->id,
                //'vcard' => 'https://t.com/'.$value->id,
                'is_printed' => $value->is_printed
            ];
        }

        if (!empty($userdata)) {
            $response = ['status' => true, 'message' => 'User Found.', 'data' =>  $final_data];
        } else {
            $response = ['status' => false, 'message' => 'User Not Found.'];
        }

        echo json_encode($response);
        die;
    }

    public function update($unique_code)
    {
        $users = DB::table('users')->where('unique_code', $unique_code)->where('status', 1)->first();
        if (!empty($users)) {
            if ($users->is_printed == 0) {
                DB::table('users')->where('unique_code', $unique_code)->update(['is_printed' => 1, 'printed_at' => $this->now]);
            }

            $user_id =  $users->id;

            $data1 = [
                'user_id'       => $user_id,
                'created_at'    => $this->now,
                'updated_at'    => $this->now
            ];

            DB::table('redeem')->insert($data1);

            $response = ['status' => true, 'message' => 'Redeemed Successfully.'];
        } else {
            $response = ['status' => false, 'message' => 'User Not found.'];
        }

        echo json_encode($response);
        die;
    }

    public function edit_user($id, Request $request)
    {
        $data = User::where('id', $id)->first();

        if (!empty($data)) {
            /* if (!empty($request->phone)) {
                $checkUser = User::where('email', $request->email)->where('phone', $request->phone)->where('id', '!=', $id)->count();

                if ($checkUser > 0) {
                    $response = ['status' => false, 'message' => 'This Email or Phone Number already registered'];
                    echo json_encode($response);
                    die;
                }
            } */

            if (isset($request->phone)) {
                $phone = $request->phone;
            } else {
                $phone = '';
            }

            $user_update = User::where('id', $id)->update([
                'name' => $request->name,
                'email' => $request->email,
                'organization' => $request->company,
                'designation' => $request->designation,
                'country_code' => '+91',
                'phone' => $phone,
            ]);

            if ($user_update) {
                $user_data = User::where('id', $id)->first();

                $vcard = new VCard();

                $firstname = $user_data->name;
                $lastname = '';
                $additional = '';
                $prefix = '';
                $suffix = '';

                // add personal data
                $vcard->addName($lastname, $firstname, $additional, $prefix, $suffix);

                // add work data
                $vcard->addCompany($user_data->organization);
                $vcard->addJobtitle($user_data->designation);
                $vcard->addRole($user_data->designation);
                $vcard->addEmail($user_data->email);
                $vcard->addPhoneNumber($user_data->phone, 'WORK');

                $first_name = explode(' ', $user_data->name);
                $upload_name = "Smart-Reg/eemaindia/vcard/$first_name[0]$user_data->id.vcf";
                $vcard_path = Storage::disk('s3')->put($upload_name, $vcard->getOutput(), 'public');
                $vcard_path = Storage::disk('s3')->url($upload_name);

                $update_user = DB::table('users')->where('id', $user_data->id)->update(['vcard' => $vcard_path]);
            }

            $response = ['status' => true, 'message' => 'User Update Successfully'];
        } else {
            $response = ['status' => false, 'message' => 'Sorry No Record found with this match'];
        }

        echo json_encode($response);
        die;
    }
}
