<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\ContactOrganizar;
use Carbon\Carbon;
use Validator;

class ContactOrganizarController extends Controller
{
    /**
     * Instantiate a new controller instance.
     *
     * @return void
     */

    public function __construct() 
    {
        $this->now = Carbon::now();
    }

    public function save(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required',
            'contact_number' => 'required|numeric|min:8',
            'message' => 'required',
        ],[
            'name.required' => 'Name is required.',
            'email.required' => 'Email is required.',
            'mobile.required' => 'Contact number is required.',
            'mobile.numeric' => 'Contact number is invalid.',
            'mobile.min' => 'Contact number is invalid.',
            'message.required' => 'Message is required.',
        ]);

        if ($validator->passes()) {
            $data = [
                "name" => $request->name,
                "email" => $request->email,
                "country_code" => $request->country_code,
                "contact_number" => $request->contact_number,
                "enquiry_about" => $request->enquiry_about,
                "message" => $request->message,
                "created_at" => $this->now,
                "updated_at" => $this->now,
            ];

            ContactOrganizar::create($data);

            return response()->json(['type'=>'Success', 'message'=> 'Query submited successfully. We will contact you shortly.']);
        } else {
            return response()->json(['error'=>$validator->errors()->all()]);
        }
    }
}
