<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Payment;
use Illuminate\Support\Facades\Mail;
use App\Models\User;
use App\Models\Ticket;
use App\Models\TicketBooking;
use App\Models\WhatsAppAutomation;
use App\Models\Order;
use App\Models\Event;
use App\Mail\MasterRegistrationOrderSummaryEmail;
use App\Mail\UserRegistrationOrderSummaryEmail;
use App\Mail\RegistrationSummaryEmail;
use Session, Config, DB;
use Carbon\Carbon;
use Razorpay\Api\Api;
use Illuminate\Support\Facades\{Http, URL};

class PaymentController extends Controller
{
    public function __construct()
    {
        session_start();
        $this->now = Carbon::now();
    }

    //payment token for ticket packages
    public function payments($order_id)
    {
        $order_data = Order::where('unique_order_id', $order_id)->first();
        $event_data = Event::where('id', 1)->first();
        $ticket_bookings_master = DB::table('users')
            ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
            ->select('users.ticket_id', 'users.name as user_first_name', 'users.last_name as user_last_name', 'users.email as user_email', 'users.phone as user_phone', 'tickets.*')
            ->where('order_id', $order_id);
        $ticket_count = $ticket_bookings_master->count();
        $ticket_bookings_master = $ticket_bookings_master->where('master_user', 1)
            ->first();

        $all_tickets[] = [
            "ticket_id" => $ticket_bookings_master->ticket_id,
            "ticket_name" => $ticket_bookings_master->name,
            "ticket_description" => $ticket_bookings_master->description,
            "ticket_type" => "Paid",
            "ticket_currency" => "INR",
            "ticket_currency_symbol" => "₹",
            "ticket_price" => $ticket_bookings_master->price,
            "ticket_quantity" => $ticket_count
        ];

        $order_details = [
            "order_id" => $order_id,
            "order_amount" => $order_data->total,
            "order_currency" => "INR",
            "orderDetails" => [
                "name" => $ticket_bookings_master->user_first_name . ' ' . $ticket_bookings_master->user_last_name,
                "email" => $ticket_bookings_master->user_email,
                "phone" => $ticket_bookings_master->user_phone,
                "event_name" => $event_data->title,
                "event_address" => $event_data->event_address,
                "event_website" => $event_data->url,
                "event_logo" => $event_data->logo,
                "event_date_range" => [$event_data->event_date, $event_data->event_time],
                "package_name" => $ticket_bookings_master->category,
                "tickets" => $all_tickets
            ],
            "redirectUrl" => route('PaymentSuccess')
        ];
        $order_details = json_encode($order_details);

        /* to make the payment bypasss */
        $event_data = DB::table('events')->select('id', 'payment_bypass')->where('id', 1)->first();
        $bypass_ids = explode(',', trim($event_data->payment_bypass));

        if (in_array($ticket_bookings_master->user_email, $bypass_ids)) {
            $success_url = route('PaymentSuccess');
            return redirect($success_url . '/success/' . $order_id . '/bypass/bypass');
        } else if ($order_data->total == 0) {
            $success_url = route('PaymentSuccess');
            return redirect($success_url . '/success/' . $order_id . '/Free/Free');
        } else {
            $headers     = [];
            $apiURL     = 'https://live.dreamcast.in/eventbot/paymentgateway/api/order';
            $response     = Http::withHeaders($headers)->withToken('11111111110')->withBody($order_details, 'application/json')->post($apiURL);
            $statusCode = $response->status();
            $responseBody = json_decode($response->getBody(), true);
            $redirect_gateway_url = $responseBody['data'];

            return redirect($redirect_gateway_url);
        }
    }

    //payment success for ticket packages
    public function PaymentSuccess($status, $order_id, $payment_type, $payment_details)
    {
        if ($status == 'error' || $status == 'unsuccessful') {
            return redirect()->route('unsuccessful');
        }

        $payment_details = json_decode(base64_decode($payment_details));

        $order_data = Order::where('unique_order_id', $order_id)->with('coupons')->first();

        $ticket_bookings_master_user = DB::table('users')->where('order_id', $order_id)->where('master_user', 1)->first();

        if ($payment_type == 'Razorpay') {
            $payment_data = [
                'transection_id' => $payment_details->transection_id,
                'payment_option' => $payment_type,
                'payment_data' => json_encode($payment_details),
                'user_id' => $ticket_bookings_master_user->id,
                'due' => 0,
                'status' => 1,
            ];
        } else if ($payment_type == 'Paypal') {
            $payment_data = [
                'transection_id' => $payment_details->purchase_units[0]->payments->captures[0]->id,
                'payment_option' => $payment_type,
                'payment_data' => json_encode($payment_details),
                'user_id' => $ticket_bookings_master_user->id,
                'due' => 0,
                'status' => 1,
            ];
        } else if ($payment_type == 'ccavenue') {
            $payment_data = [
                'transection_id' => $payment_details->transection_id,
                'payment_option' => $payment_type,
                'payment_data' => json_encode($payment_details),
                'user_id' => $ticket_bookings_master_user->id,
                'due' => 0,
                'status' => 1,
            ];
        } else if ($payment_type == 'bypass') {
            $payment_data = [
                'transection_id' => 'bypass',
                'payment_option' => 'bypass',
                'payment_data' => 'bypass',
                'user_id' => $ticket_bookings_master_user->id,
                'due' => 0,
                'status' => 2,
            ];
        } else if ($payment_type == 'Free') {
            $payment_data = [
                'transection_id' => 'Free',
                'payment_option' => 'Free',
                'payment_data' => 'Free',
                'user_id' => $ticket_bookings_master_user->id,
                'due' => 0,
                'status' => 1,
            ];
        }

        $order_update = Order::where('id', $order_data->id)->update($payment_data);

        if ($payment_type == 'bypass' || $payment_type == 'Free') {

            if ($payment_type == 'bypass') {
                $ticket_booking_update = Order::where('id', $order_data->id)->update(['status' => 2]);
                $user_update = DB::table('users')->where('order_id', $order_id)->update(['status' => 2]);
            } else if ($payment_type == 'Free') {
                $ticket_booking_update = Order::where('id', $order_data->id)->update(['status' => 1]);
                $user_update = DB::table('users')->where('order_id', $order_id)->update(['status' => 1]);
            }

            if ($order_update) {
                $all_ticket_bookings = DB::table('users')
                    ->leftJoin('tickets', 'tickets.id', '=', 'users.ticket_id')
                    ->select('users.id as user_id', 'users.ticket_id', 'users.name as user_name', 'users.email as user_email', 'users.country_code', 'users.phone as user_phone', 'users.master_user', 'users.eticket_path', 'tickets.category as ticket_category', 'tickets.name as ticket_name')
                    ->where('order_id', $order_id)->get();

                if (!empty($all_ticket_bookings)) {
					$event = DB::table('events')->where('id', 1)->first();
					$eventData = [
						'##SITE_URL##' => $event->url, 
						'##TITLE##' => $event->name, 
						'##SITE_LOGO##' => $event->logo, 
						'##BANNER_IMAGE##' => $event->banner_img,
						'##BANNER_IMG##' => $event->banner_img, 
						'##EVENT_DATE##' => $event->event_date, 
						'##EVENT_TIME##' => $event->event_time, 
						'##EVENT_ADDRESS##' => $event->event_address, 
						'##CURRENCY##' => $event->Currency, 
						'##FACEBOOK##' => $event->facebook, 
						'##TWITTER##' => $event->twitter, 
						'##LINKEDIN##' => $event->linkedin, 
						'##WHATSAPP##' => $event->whatsapp, 
						'##FESTIVAL_DATES##' => $event->festival_dates." ".$event->festival_month_year, 
					];
                    foreach ($all_ticket_bookings as $key => $value) {
						$dynamicData = [ 
							'##NAME##' => $value->user_name,
							'##BOOKING_ID##' => $order_data->unique_order_id,  
							'##PACKAGE_NAME##' => $value->ticket_category,
							'##TICKET_QTY##' => $order_data->ticket_qty,
							'##SUB_TOTAL##' => $order_data->sub_total,
							'##TOTAL##' => $order_data->total,
							'##DISCOUNT_PRICE##' => $order_data->discount_price, 
							'##TICKET_NAME##' => $value->ticket_name, 
							'##ETICKET_PATH##' => $value->eticket_path, 
						];
						$templateVar	=	array_merge($eventData,$dynamicData);
						$user_email  	= 	$value->user_email;
						if ($value->master_user == 1) {
							$this->mailTemplate($templateVar,$user_email,'ConfirmationMasterOrderSummaryRegistration');
                        } else {
							$this->mailTemplate($templateVar,$user_email,'UserRegistrationOrderSummaryEmail');
                        }
                    }
                }
            }
        }
        return redirect()->route('thankyou', ['user_id' => $ticket_bookings_master_user->id, 'order_id' => $order_id]);
    }
	public function mailTemplate($templateVar,$user_email,$emailType){
		$email 					= DB::table('email_templates')->where('alias', $emailType)->first(); 
		$email->body  			= strtr($email->body, $templateVar);
		$email->subject			= strtr($email->subject, $templateVar); 
		$email->eticket_path	= strtr($email->eticket_path, $templateVar); 
		Mail::to($user_email)->send(new RegistrationSummaryEmail($email));
	}

    public function thankyou($user_id, $order_id)
    {
        session_destroy();
        $user_data = User::where('id', $user_id)->first();

        return view('payment.thankyou-payment')->with('order_id', $order_id)->with('user_data', $user_data);
    }

    public function unsuccessful()
    {
        session_destroy();
        return view('payment.unsuccessful-payment');
    }
}
