<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use Storage, DB, Validator;
use App\Models\WhatsAppAutomation;

class SendMessageController extends Controller
{
    public function __construct()
    {
        $this->data_limit = 40;
    }

    public function index()
    {
        $send_message = DB::table('send_message')->orderBy('id', 'DESC')->get();


        $userCategory = DB::table('user_types')->select('category')->where('status', 1)->get();

        return View('send_messgae', compact('send_message', 'userCategory'));
    }

    public function getupdatedcount(Request $request)
    {
        $temp_data = DB::table('send_message')->where('id', $request->temp_id)->pluck('total_send_numbers', 'id')->first();

        return $temp_data;
    }

    public function campaignsave(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'update_status' => 'required|unique:send_message',
            'template_name' => 'required',
            'callback' => 'required',
            'user_type' => 'required',
        ]);

        if (!$validator->passes()) {
            return response()->json(['error' => $validator->errors()->messages()]);
        }

        $save_data = [
            'template_name' => $request->template_name,
            'callback' => $request->callback,
            'user_type' => $request->user_type,
            'update_status' => $request->update_status,
            'header_data_type' => $request->header_data_type,
            'header_data' => $request->header_data,
            'body_values' => implode(',', $request->body_values),
            'button_values' => $request->button_values,
        ];

        $save_result = DB::table('send_message')->insert($save_data);

        return response()->json(['type' => 'Success', 'message' => 'Campaign added successfully.']);
    }

    public function startMessages(Request $request)
    {
        $update = DB::table('send_message')->where('id', $request->template_id)->update(['status' => 1]);

        return redirect()->route('sendmessage-index');
    }

    public function checkMessages(Request $request)
    {
       
        $query = DB::table('send_message')->where('id', $request->template_id);
        $template_data = $query->first();
      

        if ($request->phone != '') {
            $send_check_data = [
                'user_id' => base64_encode('1'),
                'country_code' => '91',
                'phone' => $request->phone,
                'name' => ucwords($request->name),
                'movei' => 'Gader 2',
                'agenda' => "https://s3.ap-south-1.amazonaws.com/test.bucket.in/Smart-Reg/kpmg/event/1688018526dummy.pdf",
                'eticket_path' => 'https://s3.ap-south-1.amazonaws.com/test.bucket.in/Smart-Reg/edca/eticket/SHE23TSFB5XC4.png',
            ];

            $this->checkMessage($template_data->id, $send_check_data);

            $total_send_numbers = $template_data->total_send_numbers + 1;
            $update = $query->update(['total_send_numbers' => $total_send_numbers]);
        }

        return redirect()->route('sendmessage-index');
    }

    private function getData($type, $template_name)
    {
        $message_update_status = DB::table('send_message')->where('template_name', $template_name)->where('status', 1)->pluck('update_status', 'id')->first();
        $users_data = [];
        
        $users_data = DB::table('users')->select(['id', 'name', 'email', 'country_code', 'phone', 'eticket_path', 'agenda','movei', DB::raw("'users' as data_table")])
            ->where('message_status', '!=', $message_update_status)
            ->where('status', 1)
            ->limit($this->data_limit)
            ->get()
            ->toArray();

        return $users_data;
    }

    private function checkMessage($temp_id, $user_data)
    {
        $query = DB::table('send_message')->where('id', $temp_id);
        $template_data = $query->first();

        if ($user_data['country_code'] != '') {
            $countryCodeFinal = str_replace('+', '', $user_data['country_code']);
        }

        $componentsHeaderValues = [];
        if (!empty($template_data->header_data)) {
            $componentsHeaderValues['type'] = "header";
            $componentsHeaderValues['parameters'][] = array(
                "type" => $template_data->header_data_type,
                $template_data->header_data_type => array(
                    "link" => $template_data->header_data
                )
            );
        }

        $componentsbodyValues = [];
        if (!empty($template_data->body_values)) {
            $body_values_data = [];
            $temp_body_values_data = explode(',', $template_data->body_values);
            foreach ($temp_body_values_data as $key => $value) {
                $body_values_data["type"] = "text";
                $body_values_data["text"] = $user_data[$value];
            }
            $componentsbodyValues['type'] = "body";
            $componentsbodyValues['parameters'][] = $body_values_data;

            // $componentsbodyValues['parameters'][] = $body_values_data = [
            //     "type" => "text",
            //     "text" => $user_data['name'],   
            // ];
            // $componentsbodyValues['parameters'][] = $body_values_data = [
            //     "type" => "text",
            //     "text" => "lastname",   
            // ];
            // $componentsbodyValues['parameters'][] = $body_values_data = [
            //     "type" => "text",
            //     "text" => "email",   
            // ];
        }

        /*  $buttonValues = [];
        if(!empty($template_data->button_values)) {
            $button_values_data = [];
            $temp_button_values_data = explode(',', $template_data->button_values);
            
            foreach($temp_button_values_data as $key => $value) {
                $button_values_data[$key+1] = [$user_data[$value]];
            }
            
            $buttonValues = $button_values_data;
        }*/

        $template_arr = [];
        $components_arr = [];
        if (!empty($componentsHeaderValues)) {
            $components_arr[] = $componentsHeaderValues;
        }
        if (!empty($componentsbodyValues)) {
            $components_arr[] = $componentsbodyValues;
        }
        // dd($components_arr);

        if (!empty($template_data->template_name)) {
            $template_arr["name"] = $template_data->template_name;
            $template_arr["language"]["code"] = "en";
            $template_arr["components"] = $components_arr;
        }

        $curl_data = [
            "messaging_product" => "whatsapp",
            "recipient_type"    => "individual",
            "to"                => $user_data['phone'],
            "type"              => "template",
            "template"          => $template_arr
        ];

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://graph.facebook.com/v13.0/107671198647853/messages',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => stripslashes(json_encode($curl_data, JSON_UNESCAPED_SLASHES)),
            CURLOPT_HTTPHEADER => array(
                'Authorization: Bearer EAApIDhXg8lwBACGP2fHW544Fwqs3f0WBfi9N5bOYwhWay5nZB1JFvhBT8Y9wK1eFWZBN2jv1C9woSJqZCZBpCsGXqNqzWyQaZBs2shzeyqDycuA5LGlnXZBn2U6HyDHDT0YVLHX8gC48Wo22ZCtGLTkECY6VY9ZAVmu84YemLVsvAHYDhv59ZCx0q',
                'Content-Type: application/json'
            ),
        ));

        $response = curl_exec($curl);
        curl_close($curl);
        echo $response;
        // die;
    }

    public function sendingMessages()
    {
        $query = DB::table('send_message')->where('status', 1);
        $check_active_template = $query->first();
        
        if (!empty($check_active_template)) {
            $data = $this->getData($check_active_template->user_type, $check_active_template->template_name);
            
            if (!empty($data)) {
                //$message_fun_name = 'message_' . $check_active_template->template_name;

                foreach ($data as $key => $value) {
                    
                    if (!empty($value->phone)) {
                        //$this->$message_fun_name(base64_encode($value->id), $value->country_code, $value->phone, ucwords($value->name), $check_active_template->media_file, $check_active_template->callback);

                        if ($value->country_code != '') {
                            $countryCodeFinal = str_replace('+', '', $value->country_code);
                        }

                        $componentsHeaderValues = [];
                        if (!empty($check_active_template->header_data)) {
                            $componentsHeaderValues['type'] = "header";
                            $componentsHeaderValues['parameters'][] = array(
                                "type" => $check_active_template->header_data_type,
                                $check_active_template->header_data_type => array(
                                    "link" => $check_active_template->header_data
                                )
                            );
                        }

                        $componentsbodyValues = [];
                        if (!empty($check_active_template->body_values)) {
                            $body_values_data = [];
                            $temp_body_values_data = explode(',', $check_active_template->body_values);
                            foreach ($temp_body_values_data as $key2 => $value2) {
                                $body_values_data["type"] = "text";
                                $body_values_data["text"] = $value->$value2;
                            }
                            $componentsbodyValues['type'] = "body";
                            $componentsbodyValues['parameters'][] = $body_values_data;
                        }

                        /*  $buttonValues = [];
                        if(!empty($template_data->button_values)) {
                            $button_values_data = [];
                            $temp_button_values_data = explode(',', $template_data->button_values);
                            
                            foreach($temp_button_values_data as $key => $value) {
                                $button_values_data[$key+1] = [$user_data[$value]];
                            }
                            
                            $buttonValues = $button_values_data;
                        }*/

                        $template_arr = [];
                        $components_arr = [];
                        if (!empty($componentsHeaderValues)) {
                            $components_arr[] = $componentsHeaderValues;
                        }
                        if (!empty($componentsbodyValues)) {
                            $components_arr[] = $componentsbodyValues;
                        }

                        if (!empty($check_active_template->template_name)) {
                            $template_arr["name"] = $check_active_template->template_name;
                            $template_arr["language"]["code"] = "en";
                            $template_arr["components"] = $components_arr;
                        }

                        $curl_data = [
                            "messaging_product" => "whatsapp",
                            "recipient_type"    => "individual",
                            "to"                => $value->phone,
                            "type"              => "template",
                            "template"          => $template_arr
                        ];

                        $curl = curl_init();
                        curl_setopt_array($curl, array(
                            CURLOPT_URL => 'https://graph.facebook.com/v13.0/107671198647853/messages',
                            CURLOPT_RETURNTRANSFER => true,
                            CURLOPT_ENCODING => '',
                            CURLOPT_MAXREDIRS => 10,
                            CURLOPT_TIMEOUT => 0,
                            CURLOPT_FOLLOWLOCATION => true,
                            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                            CURLOPT_CUSTOMREQUEST => 'POST',
                            CURLOPT_POSTFIELDS => stripslashes(json_encode($curl_data, JSON_UNESCAPED_SLASHES)),
                            CURLOPT_HTTPHEADER => array(
                                'Authorization: Bearer EAApIDhXg8lwBACGP2fHW544Fwqs3f0WBfi9N5bOYwhWay5nZB1JFvhBT8Y9wK1eFWZBN2jv1C9woSJqZCZBpCsGXqNqzWyQaZBs2shzeyqDycuA5LGlnXZBn2U6HyDHDT0YVLHX8gC48Wo22ZCtGLTkECY6VY9ZAVmu84YemLVsvAHYDhv59ZCx0q',
                                'Content-Type: application/json'
                            ),
                        ));

                        $response = curl_exec($curl);
                        curl_close($curl);
                        echo $response.'<br/>';
                    }

                    DB::table($value->data_table)->where('id', $value->id)->update(['message_status' => $check_active_template->update_status]);
                }

                $total_send_numbers = $check_active_template->total_send_numbers + count($data);
                $update = $query->where('id', $check_active_template->id)->update(['total_send_numbers' => $total_send_numbers]);
                
                echo 'Message Sent';
            } else {
                $update = $query->where('id', $check_active_template->id)->update(['status' => 2]);
            }
        }
    }
}
