<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Hash;
use Session;
use DB, Storage, Validator, Config, URL;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Carbon\Carbon;
use App\Models\User;
use App\Models\Ticket;
use App\Models\Country;
use App\Models\Order;
use App\Models\TicketBooking;
use App\Models\Coupon;
use App\Models\EventTab;
use App\Mail\ConfirmationEmail;
use App\Mail\RegisterEmail;
use Razorpay\Api\Api;
use Log;
use App\Services\MbadgeService;
use \Milon\Barcode\DNS1D;
use \Milon\Barcode\DNS2D;
use Intervention\Image\Facades\Image;

class UserController extends Controller
{
    /**
     * Instantiate a new controller instance.
     *
     * @return void
     */

    public function __construct()
    {
        session_start();
        $this->now = Carbon::now();

        $prefix_uniquecode = DB::table('events')->select('prefix_uniquecode', 's3bucket_name')->where('status',1)->first();
        $this->unique_code_prefix = $prefix_uniquecode->prefix_uniquecode;
        $this->s3bucket_name = $prefix_uniquecode->s3bucket_name;

    }

    public function rafale1()
    {
        return view('rafale1');
    }

    public function winner1()
    {
        $winner_name = DB::table('users')->select('id', 'name')->where('user_type_id', 4)->where('winner', 0)->inRandomOrder()->first();
        
        if(!empty($winner_name)) {
            $update = DB::table('users')->where('id', $winner_name->id)->update(['winner' => 1]);
        }

        return view('winner1',compact('winner_name'));
    }

    public function rafale2()
    {
        return view('rafale2');
    }

    public function winner2()
    {
        $winner_name = DB::table('users')->select('id', 'name')->where('user_type_id', 5)->where('winner', 0)->inRandomOrder()->first();
        
        if(!empty($winner_name)) {
            $update = DB::table('users')->where('id', $winner_name->id)->update(['winner' => 1]);
        }

        return view('winner2',compact('winner_name'));
    }

    public function index() 
    {
        session_destroy();

        $EventTab = EventTab::where('status',1)->get();

        $ticket_types = DB::table('tickets')
            ->where('status', 1)
            ->select('category', 'slug', DB::raw('SUM(display_status) as display_status'), DB::raw('SUM(sold_out) as sold_out'))
            ->groupBy('category')->get()->toArray();
        
        return view('user.index',compact('EventTab','ticket_types'));
    }

    public function ticketBookings(Request $request, $slug) 
    {
        if(isset($request->utm_source) && $request->utm_source != '') {
            $_SESSION['utm_source'] = $request->utm_source;
        }
        if(isset($request->utm_medium) && $request->utm_medium != '') {
            $_SESSION['utm_medium'] = $request->utm_medium;
        }
        if(isset($request->utm_campaign) && $request->utm_campaign != '') {
            $_SESSION['utm_campaign'] = $request->utm_campaign;
        }
        
        unset($_SESSION['coupon_applied']);
        unset($_SESSION['order_id']);

        $tickets = Ticket::where('slug', $slug)->where('status', 1)->get();
        
        return view('user.ticket_bookings')->with('tickets', $tickets)->with('slug', $slug);
    }

    public function addToCart(Request $request)
    {
        if($request->ticket_id != '') {
            if($request->ticket_quantity == 0) {
                unset($_SESSION[$request->ticket_slug][$request->ticket_id]);
            } else {
                $tickets = Ticket::select('id', 'name', 'price', 'day_qty')
                    ->where('id', $request->ticket_id)
                    ->where('status', 1)
                    ->first();

                $ticket_price = $request->ticket_quantity*$tickets->price;

                $_SESSION[$request->ticket_slug][$request->ticket_id] = [
                    'id' => $tickets->id,
                    'ticket_name' => $tickets->name,
                    'ticket_quantity' => $request->ticket_quantity,
                    'ticket_price' => $ticket_price,
                    'day_qty' => $tickets->day_qty
                ];
            }
        }

        $data = array('type' => 'Success', 'data' => $_SESSION[$request->ticket_slug]);
        echo json_encode($data);
        exit();
    }

    public function removeToCart(Request $request)
    {
        if($request->ticket_id != '' && $request->slug != '') {
            unset($_SESSION[$request->slug][$request->ticket_id]);
        }
        
        $data = array('type' => 'Success', 'data' => $_SESSION[$request->slug]);
        echo json_encode($data);
        exit();
    }

    public function addAttendees($slug)
    {
        if(!isset($_SESSION[$slug]) || empty($_SESSION[$slug])) {
            return redirect()->route('main');
        }
        unset($_SESSION['coupon_applied']);   
        unset($_SESSION['order_id']);
        $country = Country::select('id', 'name')->get();
        
        $form_details = DB::table('form_templates')->where('alias', $slug)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required'])->orderBy('sr_no', 'asc')->get()->toArray();
        
        return view('user.attendees')->with('slug', $slug)->with('country', $country)->with('form_details', $form_details);
    }

    public function loadCart(Request $request) 
    {
        $slug = $request->slug;
        return view('user.cart', compact('slug'));
    }

    public function manageUserDetails(Request $request, MbadgeService $MbadgeService)
    {
        $data = $request->all();

        //generate or update order details
        $total_ticket_qty = array_sum(array_column($_SESSION[$request->slug], 'ticket_quantity'));
        $sub_total = array_sum(array_column($_SESSION[$request->slug], 'ticket_price'));
        $total_discount = isset($_SESSION['coupon_applied']['total_discount']) ? $_SESSION['coupon_applied']['total_discount'] : 0;
        $total = $sub_total-$total_discount;

        if(isset($_SESSION['order_id']) && $_SESSION['order_id'] != '') {
            $order = Order::where('unique_order_id', $_SESSION['order_id'])->update([
                'ticket_qty' => $total_ticket_qty, 
                'sub_total' => $sub_total,
                'coupon_code' => isset($_SESSION['coupon_applied']['coupon_code']) ? $_SESSION['coupon_applied']['coupon_code'] : '',
                'discount_price' => $total_discount,
                'total' => $total,
                'due' => 1,
                'status' => 0,
                'updated_at' => $this->now
            ]);

            DB::table('users')->where('order_id', $_SESSION['order_id'])->delete();
        } else {
            $order = Order::create([
                'ticket_qty' => $total_ticket_qty, 
                'sub_total' => $sub_total,
                'coupon_code' => isset($_SESSION['coupon_applied']['coupon_code']) ? $_SESSION['coupon_applied']['coupon_code'] : '',
                'discount_price' => $total_discount,
                'total' => $total,
                'due' => 1,
                'status' => 0,
                'created_at' => $this->now
            ]);

            if($order) {
                $_SESSION['order_id'] = $this->unique_code_prefix.$order->id;
                Order::where('id', $order->id)->update(['unique_order_id' => $_SESSION['order_id']]);
            }
        }

        //validation for the form
        $email_arr = $phone_arr = [];
        foreach($data['attendee'] as $key => $value) {

            $validation_errors = $this->formValidate($value, $data['slug']);
            
            if (!empty($validation_errors)) {
                return response()->json(['error' => $validation_errors, 'validation_field_numb' => $key]);
            }
            
            if($value['email'] != '') {
                if(in_array($value['email'], $email_arr)) {
                    return response()->json(['error' => ['email' =>['Please enter unique email.']], 'validation_field_numb' => $key]);
                } else {
                    array_push($email_arr, $value['email']);
                }
            }

            if($value['phone'] != '') {
                if(in_array($value['phone'], $phone_arr)) {
                    return response()->json(['error' => ['phone' =>['Please enter unique phone number.']], 'validation_field_numb' => $key]);
                } else {
                    array_push($phone_arr, $value['phone']);
                }
            }
        }
        

        if (empty($validation_errors)) {
            
            $form_details = DB::table('form_templates')->where('alias', $data['slug'])->pluck('field_name', 'id')->toArray();
            
            $all_users_data = [];
            $attendee_num = 1;
            foreach($data['attendee'] as $key => $value) {
                if($attendee_num == 1) {
                    $master_user = 1;
                } else {
                    $master_user = 0;
                }

                $ticket_details = DB::table('tickets')->where('id', $value['ticket_id'])->select('category','shortcode', 'id')->first();

                $user_type_details = DB::table('user_types')
                    ->where('type', 'Online')
                    ->where('category', $ticket_details->category)
                    ->where('slug', $data['slug'])
                    ->select('mbadge', 'mbadge', 'id')
                    ->first();
                
                $unique_code = $this->unique_code_prefix.$ticket_details->shortcode.$MbadgeService->unique_code();
                $qrcode_path = $MbadgeService->generateQr($unique_code);
                $eticket_path = $MbadgeService->eticket($qrcode_path, $unique_code,ucwords($value['name']),ucwords($value['organization']),$user_type_details->mbadge);

                $utm_data = [];
                if(isset($_SESSION['utm_source']) && $_SESSION['utm_source'] != '') {
                    $utm_data['utm_source'] = $_SESSION['utm_source'];
                }
                if(isset($_SESSION['utm_medium']) && $_SESSION['utm_medium'] != '') {
                    $utm_data['utm_medium'] = $_SESSION['utm_medium'];
                }
                if(isset($_SESSION['utm_campaign']) && $_SESSION['utm_campaign'] != '') {
                    $utm_data['utm_campaign'] = $_SESSION['utm_campaign'];
                }

                $head_data = [
                    'user_type_id' => $user_type_details->id,
                    'ticket_id' => $value['ticket_id'],
                    'order_id' => $_SESSION['order_id'],
                    'master_user' => $master_user,
                    'unique_code' => $unique_code,
                    'qrcode_path' => $qrcode_path,
                    'eticket_path' => $eticket_path,
                    'created_at' => $this->now,
                    'updated_at' => $this->now,
                ];

                $users_data=[];
                foreach($form_details as $field_name) {
                    $field_type = gettype($value[$field_name]);
                    if($field_type == 'array') {
                        $users_data[$field_name] = implode(',', $value[$field_name]);
                    } else {
                        $users_data[$field_name] = $value[$field_name];
                    }
                }

                if(in_array('phone', $form_details)) {
                    $users_data['country_code'] = $value['country_code'];
                }

                $users_data = array_merge($head_data, $users_data);
                $users_data = array_merge($users_data, $utm_data);
                
                $all_users_data[] = $users_data;
                $attendee_num++;
            }

            $save_users = DB::table('users')->insert($all_users_data);

            return response()->json(['type'=>'success', 'message'=> 'Attendee added successfully.', 'order_id' => $_SESSION['order_id']]);

        }
    }

    protected function formValidate($form_data, $slug)
    {
        $form_details = DB::table('form_templates')->where('alias', $slug)->select(['id', 'field_title', 'field_name', 'field_type', 'value', 'required', 'unique_field'])->get()->toArray();
        $form_fields = [];
        foreach($form_details as $key => $value) {
            $form_fields[$value->field_name] = $value;
        }
        $errors = [];
        foreach($form_data as $key => $value) {
            if(isset($form_fields[$key])) {
                $required = $form_fields[$key]->required;
                $unique = $form_fields[$key]->unique_field;
                $email = $form_fields[$key]->field_type == 'email' ? 1 : 0;
                $phone = $form_fields[$key]->field_type == 'phone' ? 1 : 0;
                
                if($required == 1 && empty($value)) {
                    $errors[$key][] = ucwords($form_fields[$key]->field_title.' is required.');
                }
    
                if($unique == 1) {
                    $exist = DB::table('users')->where($key, $value)->where('ticket_id', $form_data['ticket_id'])->where('status', 1)->count();
                    if($exist > 0) {
                        $errors[$key][] = ucwords($form_fields[$key]->field_title.' is already exists.');
                    }
                }
    
                if($email == 1 && $value != '') {
                    $field_value = $value;
                    if (!filter_var($field_value, FILTER_VALIDATE_EMAIL)) {
                        $errors[$key][] = ucwords($form_fields[$key]->field_title.' is invalid.');
                    }
                }

                if($phone == 1 && $value != '') {
                    $field_value = $value;
                    $field_length = strlen($value);
                    
                    if(strpos($field_value, '+') !== false || $field_length < 8 || $field_length > 15){
                        $errors[$key][] = ucwords($form_fields[$key]->field_title.' is invalid.');
                    }
                }
            }
            
        }

        return $errors;
    }

    protected function checkout($slug)
    {
        if(!isset($_SESSION[$slug]) || empty($_SESSION[$slug])) {
            return redirect()->route('main');
        }

        return view('user.checkout')->with('slug', $slug);
    }

    public function addCouponCode(Request $request)
    {
        $slug = $request->slug;

        if($request->coupon_code != '') {
            $coupon = Coupon::where('code', $request->coupon_code)->where('status', 1)->first();
            if(!empty($coupon)) {
                /* for limit check  max coupon code used */
                $all_ticket_quantity = array_column($_SESSION[$slug], 'ticket_quantity');
                $total_ticket_quantity=array_sum($all_ticket_quantity);

                $curent_time = strtotime($this->now);
                $starting_time = strtotime($coupon->starting_at);
                $ending_time = strtotime($coupon->ending_at);
                
                if($curent_time < $starting_time || $curent_time > $ending_time) {
                    $data = array('type' => 'Error', 'message' => 'Invalid coupon code.');
                } else {

                    $coupon_applied = [];
                    $all_ticket_prices = array_column($_SESSION[$slug], 'ticket_price');
                    $total_cart_price = array_sum($all_ticket_prices);

                    $total_code_used=Order::select(DB::raw("SUM(ticket_qty) as used_coupon_code"))->where('status',1)->where('coupon_code',$request->coupon_code)->first();
                    log::info($total_code_used->used_coupon_code.'total_code_used');
                    log::info($total_ticket_quantity.'total_ticket_quantity');
                    $no_of_tickets=$total_code_used->used_coupon_code+$total_ticket_quantity;
                      log::info($no_of_tickets.'total_code_used');
                    if($no_of_tickets <= $coupon->max_ticket)
                    {

                        if($coupon->ticket_id == 'Cart') {

                            if($coupon->coupon_type == 1) {
                                $discount = $coupon->amount;
                            } else {
                                $discount = ($total_cart_price*$coupon->amount)/100;
                            }

                            $coupon_applied['Cart'] = [
                                'ticket_price' => $total_cart_price,
                                'discount_price' => $total_cart_price-$discount,
                                'discount' => $discount
                            ];
                        } else {
                            $coupon_ticket_cat = explode(',',$coupon->ticket_id);
                            foreach($coupon_ticket_cat as $key => $value) {
                                foreach($_SESSION[$slug] as $session_cat) {
                                    if($session_cat["id"] == $value) {

                                        if($coupon->coupon_type == 1) {
                                            $discount = $coupon->amount;
                                        } else {
                                            $discount = ($session_cat["ticket_price"]*$coupon->amount)/100;
                                        }

                                        $coupon_applied[$value] = [
                                            'ticket_price' => $session_cat["ticket_price"],
                                            'discount_price' => $session_cat["ticket_price"]-$discount,
                                            'discount' => $discount
                                        ];

                                    }
                                }
                            }
                        }

                        $discount_price_arr = array_column($coupon_applied, 'discount_price');
                        $negative_value = array_filter($discount_price_arr, function($x) {
                            return $x < 0;
                        });

                        if(empty($negative_value)) {
                            Log::info($negative_value);
                            $coupon_applied['coupon_code'] = $request->coupon_code;
                            $coupon_applied['total_discount'] = array_sum(array_column($coupon_applied, 'discount'));

                            $_SESSION['coupon_applied'] = $coupon_applied;

                            if($coupon->coupon_type == 1) {
                                $discount_order = strval('₹ '.$coupon->amount);
                            } else {
                                $discount_order = strval($coupon->amount.'%');
                            }

                            if(isset($_SESSION['order_id']) && $_SESSION['order_id'] != '') {
                                $order = new Order();
                    
                                $get_order_data = $order->where('unique_order_id', $_SESSION['order_id'])->first();
                                
                                $update_order = $order->where('unique_order_id',$_SESSION['order_id'])->update([
                                    'coupon_code' => $coupon_applied['coupon_code'],
                                    'discount_price' => $coupon_applied['total_discount'],
                                    'total' => $get_order_data->sub_total-$coupon_applied['total_discount'],
                                ]);
                            }
                             if($coupon_applied['total_discount']==0)
                            {
                               $data = array('type' => 'Error', 'message' => 'This coupon is not applicable on any of the selected ticket categories.');
                            }
                            else
                            {
                                $data = array('type' => 'Success', 'message' => 'Coupon code applied successfully.');
                            }
                            
                        }
                        else {
                            $data = array('type' => 'Error', 'message' => 'This coupon is not applicable on any of the selected ticket categories.');
                        }
                    }
                    else{
                        $data = array('type' => 'Error', 'message' => 'This coupon code no longer valid.');
                    }
                }
            } else {
                $data = array('type' => 'Error', 'message' => 'Invalid coupon code.');
            }
        } else {
            $data = array('type' => 'Error', 'message' => 'Please enter a valid coupon code.');
        }
        
        echo json_encode($data);
        exit();
    }

    public function removeCouponCode()
    {
        unset($_SESSION['coupon_applied']);
        
        if(isset($_SESSION['order_id']) && $_SESSION['order_id'] != '') {
            $order = new Order();

            $get_order_data = $order->where('unique_order_id', $_SESSION['order_id'])->first();
            
            $update_order = $order->where('unique_order_id',$_SESSION['order_id'])->update([
                'coupon_code' => '', 
                'discount' => '',
                'discount_price' => '',
                'total' => $get_order_data->sub_total,
            ]);
        }
        $data = array('type' => 'Success', 'message' => 'Coupon code removed.');

        echo json_encode($data);
        exit();
    }

    public function termsConditions()
    {
        return view('user.terms-conditions');
    }
    
    public function privacyPolicy()
    {
        return view('user.privacy-policy');
    }

    public function generateBarcode(MbadgeService $MbadgeService)
    {
        //dd($barcode);
        $barcode = new DNS2D();
        
        $code = '123456';

        $barcode = $MbadgeService->baarcode($code);
        dd($barcode);
        //$barcode->setStorPath(public_path().'/barcodes/');
        //dd($barcode->getBarcodePNGPath('123456', 'CODABAR',4,50));

        dd($document_path);
        return view('barcode', compact('upload_name', 'code'));
    }
}
