<?php

namespace App\Libraries;

use App\Models\Email\Smtp;
use App\Models\Email\Template;
use Illuminate\Mail\Message;
use Mail,DB,DateTimeZone,DateTime;

class Email
{
    private static array $from = [];
    private static array $to = [];
    private static array $replyTo = [];
    private static array $cc = [];
    private static array $bcc = [];
    private static string $subject = "";
    private static string $body = "";

    private static string $calendar=""; 
    private static array $attachments=[];
    private static array $smtpParams = [
        "host" => "",
        "port" => "",
        "username" => "",
        "password" => "",
        "tls" => false
    ];
    private static string $template="";
    private static array $templateVariables = [];

    static function setTemplate(string $template,array $templateVariables=[]): Email
    {
        if(!empty($template)){
            self::$template = $template;
            self::$templateVariables = $templateVariables;
        }
        return new self;
    }
    /**
    *
     * Email::setTemplate('signup-email',[])->send($toEmail, $toName)
     */
    private static function makeBody()
    {
        extract(self::getTemplateParams());
        self::$replyTo=$replyTo??[];
        self::$cc=$cc??[];
        self::$bcc=$bcc??[]; 
        self::$subject=strtr($subject??"", self::$templateVariables);
        
        self::$body=strtr($body??"", self::$templateVariables);
    }

    static function send($toEmail, $toName): bool
    {
        if(empty($toEmail) OR empty($toName)){
            return false;
        }
        self::$to=["email"=>$toEmail, "name"=>$toName];
        extract(self::getSmtpConfig());
        self::$smtpParams=$smtp??[];
        self::$from=$from??[];
        self::makeBody();
        $data['to'] =   self::$to;
        Mail::send([],[], function ( $message){
            $message->from(self::$from["email"],self::$from["name"])
                ->to(self::$to["email"],self::$to["name"])
                ->subject(self::$subject)
                ->setBody(self::$body, 'text/html');

            if(!empty(self::$cc["email"])){
                $message->cc(self::$cc["email"],self::$cc["name"]);
            }
            if(!empty(self::$bcc["email"])){
                $message->bcc(self::$bcc["email"],self::$bcc["name"]);
            }
            if(!empty(self::$replyTo["email"])){
                $message->replyTo(self::$replyTo["email"],self::$replyTo["name"]);
            }
            if(!empty(self::$calendar)){
                $message->attachData(base64_encode(self::$calendar), 'invite.ics', [
                    'mime' => 'text/calendar; name="invite.ics"',
                    "as"=> 'invite.ics'
                ]);
            }
            if(!empty(self::$attachments)){
                foreach (self::$attachments as $attachment) {
                    $message->attach($attachment["path"], [
                        'as' => $attachment["as"],
                        'mime' => $attachment["mime"],
                    ]);
                }
            }

            $transport = (new \Swift_SmtpTransport(self::$smtpParams["host"], self::$smtpParams["port"], self::$smtpParams["host"]?'tls':''))
                ->setUsername(self::$smtpParams["username"])
                ->setPassword(self::$smtpParams["password"]);

            $mailer = new \Swift_Mailer($transport);

            Mail::setSwiftMailer($mailer);
        });
        return true;

    }
    private static function getSmtpConfig():array
    {
        $config=[
            "smtp"=>self::$smtpParams,
            "from"=>[],
        ];
        $smtpConfig = Smtp::where("status", 1)
            ->select("host", "port", "username", "password", "sender_email", "sender_name", "tls")
            ->first();
        if (!empty($smtpConfig)) {
            $config["smtp"]=$smtpConfig->toArray();
            $config["from"]=[
                "name"=>$config["smtp"]["sender_name"],
                "email"=>$config["smtp"]["sender_email"],
            ];
            unset($config["smtp"]["sender_email"],$config["smtp"]["sender_name"]);
        }
        return $config;
    }
    private static function getTemplateParams():array
    {
        $template = Template::query()
                    ->where("alias",self::$template)
                    ->select("id","alias","body","subject","reply_to_email","reply_to_name","cc_email","cc_name","bcc_email","bcc_name",
                    DB::raw("null as reply_to"),
                    DB::raw("null as bcc"),
                    DB::raw("null as cc"))
                    ->first();
        if (!empty($template)) {
            $template=$template->toArray(); 
            foreach (["reply_to","bcc","cc"] as $key){ 
               $template[$key]=[
                   "name"=>$template[$key."_email"],
                   "email"=>$template[$key."_email"]
               ];
               unset($template[$key."_email"],$template[$key."_email"]);
            } 
            return $template;
        }
        return [];
    }
    private static function getTimezoneOffset($timezone) : string{
        $timezone = new DateTimeZone($timezone);
        $offset = $timezone->getOffset(new DateTime());
        $hours = floor(abs($offset) / 3600);
        $minutes = floor((abs($offset) % 3600) / 60);
        $sign = $offset < 0 ? '-' : '+';
        return sprintf('%s%02d%02d', $sign, $hours, $minutes);
    }
    private static function escapeString(?string $string): string
    {
        if (!$string) {
            return '';
        }
        return preg_replace('/([\,;])/', '\\\$1', $string);
    }
    private static function formatMultiline(?string $string): string
    {
        if (!$string) {
            return '';
        }
        return str_replace(["\r\n", "\n"], "\\r\\n", $string);
    } 
}
