<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    public $table = "orders";
    use HasFactory;

    public $fillable = [
        'transection_id', 
        'payment_option', 
        'payment_data',
        'user_id',
        'ticket_qty',
        'taxes',
        'sub_total',
        'coupon_code',
        'discount',
        'discount_price',
        'total', 
        'tickets_sold',
        'due',
        'description',
        'unique_order_id',
        'status',
        'date_only',
        'time_only',
        'utm_source',
        'utm_medium',
        'utm_campaign',
        'created_at',
        'updated_at',
    ];

    public function coupons()
    {
        return $this->hasOne(Coupon::class, 'code', 'coupon_code');
    }

    public function single_ticket_bookings()
    {
        return $this->hasOne(TicketBooking::class, 'order_id', 'unique_order_id');
    }
    
    public function ticket_bookings()
    {
        return $this->hasMany(TicketBooking::class, 'order_id', 'unique_order_id');
    }

    public function hotel_bookings()
    {
        return $this->hasMany(HotelBooking::class, 'order_id', 'unique_order_id');
    }

    public function users()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

}
