<?php
namespace App\Services;

use Storage,DB;
use Carbon\Carbon;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use \Milon\Barcode\DNS1D;
use \Milon\Barcode\DNS2D;

class MbadgeService {
    public function __construct()
    {
        $this->now = Carbon::now();

        $prefix_uniquecode = DB::table('events')->select('prefix_uniquecode', 's3bucket_name')->where('status',1)->first();
        $this->unique_code_prefix = $prefix_uniquecode->prefix_uniquecode;
        $this->s3bucket_name = $prefix_uniquecode->s3bucket_name;

    }

    public function unique_code()
    {
        $unique_code = DB::table('online_registration_unique_codes')->where('is_used', 0)->first();
        DB::table('online_registration_unique_codes')->where('id', $unique_code->id)->update(['is_used' => 1, 'modified' => Carbon::now()]);
        return $unique_code->unique_code;
    }
    
    public function generateQr($unique_code)
    {
        $document = QrCode::format('png')->margin(2)->size(500)->backgroundColor(255, 255, 255)->generate($unique_code);
        $upload_name= "Smart-Reg/'.$this->s3bucket_name.'/qrcodes/".$unique_code.".png";
        $document_path = Storage::disk('s3')->put($upload_name, $document, 'public');
        $document_path = Storage::disk('s3')->url($upload_name);
        
        return $document_path;
    }

    public function getCode()
    {
        $code = $this->randomNumber();

        $code_exist = DB::table('users')->where('unique_code', $code)->select('id')->get()->toArray();

        if(count($code_exist) > 1) {
            $this->getCode();
        }

        return $code;
    }

    private function randomNumber() 
    {
        $six_digit_random_number = random_int(10000000, 99999999);
        return $six_digit_random_number;
    }
    
    public function baarcode($code)
    {
        $barcode = new DNS1D();

        $barcode->setStorPath(public_path().'/barcodes/');
        
        $upload_name= "barcodes/$code.png";
        $document_path = Storage::disk('s3')->put($upload_name, base64_decode($barcode->getBarcodePNG($code, "CODABAR",4,50)), 'public');
        $document_path = Storage::disk('s3')->url($upload_name);
        return $document_path;
    }

    public function eticket($qr_path, $unique_code, $name,$mbadge)
    {

        $new        = imagecreatefrompng($qr_path);
        $master     = imagecreatefrompng($mbadge);
        
        imagecopyresampled($master, $new, 75, 150, 0, 0, 300,300,500, 500);
        
        //$font = realpath('public/arial.ttf');
        $white = imagecolorallocate($master, 255,255,255);
        $black = imagecolorallocate($master, 0,0,0);
        $bold_font = realpath('public/SF-Pro-Text-Bold.otf');
        $font = realpath('public/SF-Pro-Text-Regular.otf');
        
        //Unique Code Print ============================================================================================================================
        $unique_code_font_size = 15;

        list($x, $y) = $this->ImageTTFCenter($master, $unique_code, $font, $unique_code_font_size);
        imagettftext($master, $unique_code_font_size, 0, $x, $y+70, $black, $font, $unique_code);

        //Name Print ===================================================================================================================================
        $new_name = wordwrap($name,20,"<br>\n");
        $new_name = str_replace(array("\r", "\n"), '', $new_name);
        $final_name = explode('<br>', $new_name);
        
        $name_count = count($final_name);
        //dd($name_count);
        if($name_count == 1) {
            $name_font_size = 25;

            $name1 = $final_name[0];
            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 100, $black, $font, $name1);
        } else if($name_count == 2) {
            
            $name_font_size = 25;

            $name1 = $final_name[0];
            $name2 = $final_name[1];

            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 80, $black, $font, $name1);

            list($x, $y) = $this->ImageTTFCenter($master, $name2, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 120, $black, $font, $name2);
        } else if($name_count == 3) {
            $name_font_size = 20;

            $name1 = $final_name[0];
            $name2 = $final_name[1];
            $name3 = $final_name[2];

            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 60, $black, $font, $name1);

            list($x, $y) = $this->ImageTTFCenter($master, $name2, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 90, $black, $font, $name2);

            list($x, $y) = $this->ImageTTFCenter($master, $name3, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 120, $black, $font, $name3);
        } else {
            $name_font_size = 20;

            $name1 = $final_name[0];
            $name2 = $final_name[1];
            $name3 = $final_name[2];

            list($x, $y) = $this->ImageTTFCenter($master, $name1, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 60, $black, $font, $name1);

            list($x, $y) = $this->ImageTTFCenter($master, $name2, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 90, $black, $font, $name2);

            list($x, $y) = $this->ImageTTFCenter($master, $name3, $font, $name_font_size);
            imagettftext($master, $name_font_size, 0, $x, 120, $black, $font, $name3);
        }
        
        // header('Content-Type: image/png');

        $masterImg= imagepng($master,'public/eticket/'.$unique_code.".png");

        if($masterImg)
        {
            $upload_name= "Smart-Reg/$this->s3bucket_name/eticket/".time().$unique_code.".png";
            $eticketpath = Storage::disk('s3')->put($upload_name, file_get_contents('public/eticket/'.$unique_code.'.png'), 'public');
            $eticketpath = Storage::disk('s3')->url($upload_name);
            unlink('public/eticket/'.$unique_code.'.png');
        }

        // echo $eticketpath; die();
        return $eticketpath;
    }

    function ImageTTFCenter($image, $text, $font, $size, $angle = 0) 
    {
        $xi = imagesx($image);
        $yi = imagesy($image);

        $box = imagettfbbox($size, $angle, $font, $text);

        $xr = abs(max($box[2], $box[4]));
        $yr = abs(max($box[5], $box[7]));

        $x = intval(($xi - $xr) / 2);
        $y = intval(($yi + $yr) / 2);
        // die();

        return array($x, $y);
    }
}